import type { AttributeConfiguration } from '@microsoft/fast-element';
import { Behavior } from '@microsoft/fast-element';
import type { CaptureType } from '@microsoft/fast-element';
import { ChildViewTemplate } from '@microsoft/fast-element';
import { ComposableStyles } from '@microsoft/fast-element';
import { composedContains } from '@microsoft/fast-element/utilities';
import { composedParent } from '@microsoft/fast-element/utilities';
import { Constructable } from '@microsoft/fast-element';
import { CSSDirective } from '@microsoft/fast-element';
import { Direction } from '@microsoft/fast-web-utilities';
import { ElementStyles } from '@microsoft/fast-element';
import { ElementViewTemplate } from '@microsoft/fast-element';
import { FASTElement } from '@microsoft/fast-element';
import { ItemViewTemplate } from '@microsoft/fast-element';
import { Orientation } from '@microsoft/fast-web-utilities';
import type { PartialFASTElementDefinition } from '@microsoft/fast-element';
import { SyntheticViewTemplate } from '@microsoft/fast-element';
import { ViewTemplate } from '@microsoft/fast-element';

/**
 * An Accordion Custom HTML Element
 * Implements {@link https://www.w3.org/TR/wai-aria-practices-1.1/#accordion | ARIA Accordion}.
 *
 * @fires change - Fires a custom 'change' event when the active item changes
 * @csspart item - The slot for the accordion items
 * @public
 *
 * @remarks
 * Designed to be used with {@link @microsoft/fast-foundation#accordionTemplate} and {@link @microsoft/fast-foundation#(AccordionItem:class)}.
 */
export declare class Accordion extends FoundationElement {
    /**
     * Controls the expand mode of the Accordion, either allowing
     * single or multiple item expansion.
     * @public
     *
     * @remarks
     * HTML attribute: expand-mode
     */
    expandmode: AccordionExpandMode;
    /* Excluded from this release type: accordionItems */
    /* Excluded from this release type: accordionItemsChanged */
    private activeid;
    private activeItemIndex;
    private accordionIds;
    private change;
    private findExpandedItem;
    private setItems;
    private resetItems;
    private removeItemListeners;
    private activeItemChange;
    private getItemIds;
    private isSingleExpandMode;
    private handleItemKeyDown;
    private handleItemFocus;
    private adjust;
    private focusItem;
}

/**
 * Expand mode for {@link Accordion}
 * @public
 */
export declare const AccordionExpandMode: {
    /**
     * Designates only a single {@link @microsoft/fast-foundation#(AccordionItem:class) } can be open a time.
     */
    readonly single: "single";
    /**
     * Designates multiple {@link @microsoft/fast-foundation#(AccordionItem:class) | AccordionItems} can be open simultaneously.
     */
    readonly multi: "multi";
};

/**
 * Type for the {@link Accordion} Expand Mode
 * @public
 */
export declare type AccordionExpandMode = typeof AccordionExpandMode[keyof typeof AccordionExpandMode];

/**
 * An individual item in an {@link @microsoft/fast-foundation#(Accordion:class) }.
 *
 * @slot start - Content which can be provided between the heading and the icon
 * @slot end - Content which can be provided between the start slot and icon
 * @slot heading - Content which serves as the accordion item heading and text of the expand button
 * @slot - The default slot for accordion item content
 * @slot expanded-icon - The expanded icon
 * @slot collapsed-icon - The collapsed icon
 * @fires change - Fires a custom 'change' event when the button is invoked
 * @csspart heading - Wraps the button
 * @csspart button - The button which serves to invoke the item
 * @csspart heading-content - Wraps the slot for the heading content within the button
 * @csspart icon - The icon container
 * @csspart region - The wrapper for the accordion item content
 *
 * @public
 */
export declare class AccordionItem extends FoundationElement {
    /**
     * Configures the {@link https://www.w3.org/TR/wai-aria-1.1/#aria-level | level} of the
     * heading element.
     *
     * @defaultValue 2
     * @public
     * @remarks
     * HTML attribute: heading-level
     */
    headinglevel: 1 | 2 | 3 | 4 | 5 | 6;
    /**
     * Expands or collapses the item.
     *
     * @public
     * @remarks
     * HTML attribute: expanded
     */
    expanded: boolean;
    /**
     * The item ID
     *
     * @public
     * @remarks
     * HTML Attribute: id
     */
    id: string;
    /* Excluded from this release type: expandbutton */
    /* Excluded from this release type: clickHandler */
    private change;
}

/* Excluded declaration from this release type: AccordionItem */

/**
 * Accordion Item configuration options
 * @public
 */
export declare type AccordionItemOptions = FoundationElementDefinition & StartEndOptions & {
    expandedIcon?: string | SyntheticViewTemplate;
    collapsedIcon?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(AccordionItem:class)} component.
 * @public
 */
export declare const accordionItemTemplate: FoundationElementTemplate<ViewTemplate<AccordionItem>, AccordionItemOptions>;

/**
 * The template for the {@link @microsoft/fast-foundation#Accordion} component.
 * @public
 */
export declare const accordionTemplate: FoundationElementTemplate<ViewTemplate<Accordion>>;

/**
 * A decorator and DI resolver that will resolve an array of all dependencies
 * registered with the specified key.
 * @param key - The key to resolve all dependencies for.
 * @param searchAncestors - [optional] Indicates whether to search ancestor containers.
 * @public
 */
export declare const all: (key: any, searchAncestors?: boolean) => ReturnType<typeof DI.inject>;

/**
 * An Anchor Custom HTML Element.
 * Based largely on the {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element }.
 *
 * @slot start - Content which can be provided before the anchor content
 * @slot end - Content which can be provided after the anchor content
 * @slot - The default slot for anchor content
 * @csspart control - The anchor element
 * @csspart content - The element wrapping anchor content
 *
 * @public
 */
export declare class Anchor extends FoundationElement {
    /**
     * Prompts the user to save the linked URL. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element } for more information.
     * @public
     * @remarks
     * HTML Attribute: download
     */
    download: string;
    /**
     * The URL the hyperlink references. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element } for more information.
     * @public
     * @remarks
     * HTML Attribute: href
     */
    href: string;
    /**
     * Hints at the language of the referenced resource. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element } for more information.
     * @public
     * @remarks
     * HTML Attribute: hreflang
     */
    hreflang: string;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element } for more information.
     * @public
     * @remarks
     * HTML Attribute: ping
     */
    ping: string;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element } for more information.
     * @public
     * @remarks
     * HTML Attribute: referrerpolicy
     */
    referrerpolicy: string;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element } for more information.
     * @public
     * @remarks
     * HTML Attribute: rel
     */
    rel: string;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element } for more information.
     * @public
     * @remarks
     * HTML Attribute: target
     */
    target: "_self" | "_blank" | "_parent" | "_top";
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a | <a> element } for more information.
     * @public
     * @remarks
     * HTML Attribute: type
     */
    type: string;
    /* Excluded from this release type: defaultSlottedContent */
    /**
     * References the root element
     */
    control: HTMLAnchorElement;
    /* Excluded from this release type: connectedCallback */
    /**
     * Overrides the focus call for where delegatesFocus is unsupported.
     * This check works for Chrome, Edge Chromium, FireFox, and Safari
     * Relevant PR on the Firefox browser: https://phabricator.services.mozilla.com/D123858
     */
    private handleUnsupportedDelegatesFocus;
}

/* Excluded declaration from this release type: Anchor */

/**
 * An anchored region Custom HTML Element.
 *
 * @slot - The default slot for the content
 * @fires loaded - Fires a custom 'loaded' event when the region is loaded and visible
 * @fires positionchange - Fires a custom 'positionchange' event when the position has changed
 *
 * @public
 */
export declare class AnchoredRegion extends FoundationElement {
    /**
     * The HTML ID of the anchor element this region is positioned relative to
     *
     * @public
     * @remarks
     * HTML Attribute: anchor
     */
    anchor: string;
    protected anchorChanged(): void;
    /**
     * The HTML ID of the viewport element this region is positioned relative to
     *
     * @public
     * @remarks
     * HTML Attribute: anchor
     */
    viewport: string;
    protected viewportChanged(): void;
    /**
     * Sets what logic the component uses to determine horizontal placement.
     * 'locktodefault' forces the default position
     * 'dynamic' decides placement based on available space
     * 'uncontrolled' does not control placement on the horizontal axis
     *
     * @public
     * @remarks
     * HTML Attribute: horizontal-positioning-mode
     */
    horizontalPositioningMode: AxisPositioningMode;
    protected horizontalPositioningModeChanged(): void;
    /**
     * The default horizontal position of the region relative to the anchor element
     *
     * @public
     * @remarks
     * HTML Attribute: horizontal-default-position
     */
    horizontalDefaultPosition: HorizontalPosition;
    protected horizontalDefaultPositionChanged(): void;
    /**
     * Whether the region remains in the viewport (ie. detaches from the anchor) on the horizontal axis
     *
     * @public
     * @remarks
     * HTML Attribute: horizontal-viewport-lock
     */
    horizontalViewportLock: boolean;
    protected horizontalViewportLockChanged(): void;
    /**
     * Whether the region overlaps the anchor on the horizontal axis
     *
     * @public
     * @remarks
     * HTML Attribute: horizontal-inset
     */
    horizontalInset: boolean;
    protected horizontalInsetChanged(): void;
    /**
     * How narrow the space allocated to the default position has to be before the widest area
     * is selected for layout
     *
     * @public
     * @remarks
     * HTML Attribute: horizontal-threshold
     */
    horizontalThreshold: number;
    protected horizontalThresholdChanged(): void;
    /**
     * Defines how the width of the region is calculated
     *
     * @public
     * @remarks
     * HTML Attribute: horizontal-scaling
     */
    horizontalScaling: AxisScalingMode;
    protected horizontalScalingChanged(): void;
    /**
     * Sets what logic the component uses to determine vertical placement.
     * 'locktodefault' forces the default position
     * 'dynamic' decides placement based on available space
     * 'uncontrolled' does not control placement on the vertical axis
     *
     * @public
     * @remarks
     * HTML Attribute: vertical-positioning-mode
     */
    verticalPositioningMode: AxisPositioningMode;
    protected verticalPositioningModeChanged(): void;
    /**
     * The default vertical position of the region relative to the anchor element
     *
     * @public
     * @remarks
     * HTML Attribute: vertical-default-position
     */
    verticalDefaultPosition: VerticalPosition;
    protected verticalDefaultPositionChanged(): void;
    /**
     * Whether the region remains in the viewport (ie. detaches from the anchor) on the vertical axis
     *
     * @public
     * @remarks
     * HTML Attribute: vertical-viewport-lock
     */
    verticalViewportLock: boolean;
    protected verticalViewportLockChanged(): void;
    /**
     * Whether the region overlaps the anchor on the vertical axis
     *
     * @public
     * @remarks
     * HTML Attribute: vertical-inset
     */
    verticalInset: boolean;
    protected verticalInsetChanged(): void;
    /**
     * How short the space allocated to the default position has to be before the tallest area
     * is selected for layout
     *
     * @public
     * @remarks
     * HTML Attribute: vertical-threshold
     */
    verticalThreshold: number;
    protected verticalThresholdChanged(): void;
    /**
     * Defines how the height of the region is calculated
     *
     * @public
     * @remarks
     * HTML Attribute: vertical-scaling
     */
    verticalScaling: AxisScalingMode;
    protected verticalScalingChanged(): void;
    /**
     * Whether the region is positioned using css "position: fixed".
     * Otherwise the region uses "position: absolute".
     * Fixed placement allows the region to break out of parent containers,
     *
     * @public
     * @remarks
     * HTML Attribute: fixed-placement
     */
    fixedPlacement: boolean;
    protected fixedPlacementChanged(): void;
    /**
     * Defines what triggers the anchored region to revaluate positioning
     *
     * @public
     * @remarks
     * HTML Attribute: auto-update-mode
     */
    autoUpdateMode: AutoUpdateMode;
    protected autoUpdateModeChanged(prevMode: AutoUpdateMode, newMode: AutoUpdateMode): void;
    /**
     * The HTML element being used as the anchor
     *
     * @public
     */
    anchorElement: HTMLElement | null;
    protected anchorElementChanged(): void;
    /**
     * The HTML element being used as the viewport
     *
     * @public
     */
    viewportElement: HTMLElement | null;
    protected viewportElementChanged(): void;
    /* Excluded from this release type: initialLayoutComplete */
    /**
     * indicates the current horizontal position of the region
     */
    verticalPosition: AnchoredRegionPositionLabel | undefined;
    /**
     * indicates the current vertical position of the region
     */
    horizontalPosition: AnchoredRegionPositionLabel | undefined;
    /**
     * values to be applied to the component's transform on render
     */
    private translateX;
    private translateY;
    /**
     * the span to be applied to the region on each axis
     */
    private regionWidth;
    private regionHeight;
    private resizeDetector;
    private viewportRect;
    private anchorRect;
    private regionRect;
    /**
     * base offsets between the positioner's base position and the anchor's
     */
    private baseHorizontalOffset;
    private baseVerticalOffset;
    private pendingPositioningUpdate;
    private pendingReset;
    private currentDirection;
    private regionVisible;
    private forceUpdate;
    private updateThreshold;
    private static intersectionService;
    /* Excluded from this release type: connectedCallback */
    /* Excluded from this release type: disconnectedCallback */
    /* Excluded from this release type: adoptedCallback */
    /**
     * update position
     */
    update: () => void;
    /**
     * destroys the instance's resize observer
     */
    private disconnectResizeDetector;
    /**
     * initializes the instance's resize observer
     */
    private initializeResizeDetector;
    /**
     * react to attribute changes that don't require a reset
     */
    private updateForAttributeChange;
    /**
     * fully initializes the component
     */
    private initialize;
    /**
     * Request a reset if there are currently no open requests
     */
    private requestReset;
    /**
     * sets the starting configuration for component internal values
     */
    private setInitialState;
    /**
     * starts observers
     */
    private startObservers;
    /**
     * get position updates
     */
    private requestPositionUpdates;
    /**
     * stops observers
     */
    private stopObservers;
    /**
     * Gets the viewport element by id, or defaults to document root
     */
    private getViewport;
    /**
     *  Gets the anchor element by id
     */
    private getAnchor;
    /**
     *  Handle intersections
     */
    private handleIntersection;
    /**
     *  iterate through intersection entries and apply data
     */
    private applyIntersectionEntries;
    /**
     *  Update the offset values
     */
    private updateRegionOffset;
    /**
     *  compare rects to see if there is enough change to justify a DOM update
     */
    private isRectDifferent;
    /**
     *  Handle resize events
     */
    private handleResize;
    /**
     * resets the component
     */
    private reset;
    /**
     *  Recalculate layout related state values
     */
    private updateLayout;
    /**
     *  Updates the style string applied to the region element as well as the css classes attached
     *  to the root element
     */
    private updateRegionStyle;
    /**
     *  Updates the css classes that reflect the current position of the element
     */
    private updatePositionClasses;
    /**
     * Get horizontal positioning state based on desired position
     */
    private setHorizontalPosition;
    /**
     * Set vertical positioning state based on desired position
     */
    private setVerticalPosition;
    /**
     *  Get available positions based on positioning mode
     */
    private getPositioningOptions;
    /**
     *  Get the space available for a particular relative position
     */
    private getAvailableSpace;
    /**
     * Get region dimensions
     */
    private getNextRegionDimension;
    /**
     * starts event listeners that can trigger auto updating
     */
    private startAutoUpdateEventListeners;
    /**
     * stops event listeners that can trigger auto updating
     */
    private stopAutoUpdateEventListeners;
}

/**
 * A utility interface to store anchored region
 * configurations that correspond to various common flyout
 * positioning schemes
 *
 * @public
 */
export declare interface AnchoredRegionConfig {
    /**
     * Whether the region is positioned using css "position: fixed".
     * Otherwise the region uses "position: absolute".
     * Fixed placement allows the region to break out of parent containers,
     */
    readonly fixedPlacement?: boolean;
    /**
     * The auto-update setting of the component
     */
    readonly autoUpdateMode?: AutoUpdateMode;
    /**
     * Sets what logic the component uses to determine vertical placement.
     */
    readonly verticalPositioningMode?: AxisPositioningMode;
    /**
     * The default vertical position of the region relative to the anchor element
     */
    readonly verticalDefaultPosition?: VerticalPosition;
    /**
     * Whether the region overlaps the anchor on the vertical axis
     */
    readonly verticalInset?: boolean;
    /**
     * Defines how the height of the region is calculated
     */
    readonly verticalScaling?: AxisScalingMode;
    /**
     * How short the space allocated to the default position has to be before the tallest area
     * is selected for layout
     */
    readonly verticalThreshold?: number;
    /**
     * Whether the region remains in the viewport (ie. detaches from the anchor) on the vertical axis
     */
    readonly verticalViewportLock?: boolean;
    /**
     * Sets what logic the component uses to determine horizontal placement.
     */
    readonly horizontalPositioningMode?: AxisPositioningMode;
    /**
     * The default horizontal position of the region relative to the anchor element
     */
    readonly horizontalDefaultPosition?: HorizontalPosition;
    /**
     *  hether the region overlaps the anchor on the horizontal axis
     */
    readonly horizontalInset?: boolean;
    /**
     * Defines how the width of the region is calculate
     */
    readonly horizontalScaling?: AxisScalingMode;
    /**
     * Whether the region remains in the viewport (ie. detaches from the anchor) on the horizontal axis
     */
    readonly horizontalViewportLock?: boolean;
    /**
     * How short the space allocated to the default position has to be before the widest area
     * is selected for layout
     */
    readonly horizontalThreshold?: number;
}

/**
 * Describes the possible positions of the region relative
 * to its anchor. Depending on the axis start = left/top, end = right/bottom
 *
 * @public
 */
export declare type AnchoredRegionPositionLabel = "start" | "insetStart" | "insetEnd" | "end" | "center";

/**
 * The template for the {@link @microsoft/fast-foundation#(AnchoredRegion:class)} component.
 * @public
 */
export declare const anchoredRegionTemplate: FoundationElementTemplate<ViewTemplate<AnchoredRegion>>;

/**
 * Anchor configuration options
 * @public
 */
export declare type AnchorOptions = FoundationElementDefinition & StartEndOptions;

/**
 * The template for the {@link @microsoft/fast-foundation#(Anchor:class)} component.
 * @public
 */
export declare const anchorTemplate: FoundationElementTemplate<ViewTemplate<Anchor>, AnchorOptions>;

/**
 * Apply mixins to a constructor.
 * Sourced from {@link https://www.typescriptlang.org/docs/handbook/mixins.html | TypeScript Documentation }.
 * @public
 */
export declare function applyMixins(derivedCtor: any, ...baseCtors: any[]): void;

/**
 * Some states and properties are applicable to all host language elements regardless of whether a role is applied.
 * The following global states and properties are supported by all roles and by all base markup elements.
 * {@link https://www.w3.org/TR/wai-aria-1.1/#global_states}
 *
 * This is intended to be used as a mixin. Be sure you extend FASTElement.
 *
 * @public
 */
export declare class ARIAGlobalStatesAndProperties {
    /**
     * Indicates whether assistive technologies will present all, or only parts of,
     * the changed region based on the change notifications defined by the aria-relevant attribute.
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-atomic}
     *
     * @public
     * @remarks
     * HTML Attribute: aria-atomic
     */
    ariaAtomic: "true" | "false" | string | null;
    /**
     * Indicates an element is being modified and that assistive technologies MAY want to wait
     * until the modifications are complete before exposing them to the user.
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-busy}
     *
     * @public
     * @remarks
     * HTML Attribute: aria-busy
     */
    ariaBusy: "true" | "false" | string | null;
    /**
     * Identifies the element (or elements) whose contents or presence are controlled by the current element.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-controls}
     * @public
     * @remarks
     * HTML Attribute: aria-controls
     */
    ariaControls: string | null;
    /**
     * Indicates the element that represents the current item within a container or set of related elements.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-current}
     * @public
     * @remarks
     * HTML Attribute: aria-current
     */
    ariaCurrent: "page" | "step" | "location" | "date" | "time" | "true" | "false" | string | null;
    /**
     * Identifies the element (or elements) that describes the object.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-describedby}
     * @public
     * @remarks
     * HTML Attribute: aria-describedby
     */
    ariaDescribedby: string | null;
    /**
     * Identifies the element that provides a detailed, extended description for the object.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-details}
     * @public
     * @remarks
     * HTML Attribute: aria-details
     */
    ariaDetails: string | null;
    /**
     * Indicates that the element is perceivable but disabled, so it is not editable or otherwise operable.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-disabled}
     * @public
     * @remarks
     * HTML Attribute: aria-disabled
     */
    ariaDisabled: "true" | "false" | string | null;
    /**
     * Identifies the element that provides an error message for the object.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-errormessage}
     * @public
     * @remarks
     * HTML Attribute: aria-errormessage
     */
    ariaErrormessage: string | null;
    /**
     * Identifies the next element (or elements) in an alternate reading order of content which, at the user's discretion,
     * allows assistive technology to override the general default of reading in document source order.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-flowto}
     * @public
     * @remarks
     * HTML Attribute: aria-flowto
     */
    ariaFlowto: string | null;
    /**
     * Indicates the availability and type of interactive popup element,
     * such as menu or dialog, that can be triggered by an element.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-haspopup}
     * @public
     * @remarks
     * HTML Attribute: aria-haspopup
     */
    ariaHaspopup: "false" | "true" | "menu" | "listbox" | "tree" | "grid" | "dialog" | string | null;
    /**
     * Indicates whether the element is exposed to an accessibility API
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-hidden}
     * @public
     * @remarks
     * HTML Attribute: aria-hidden
     */
    ariaHidden: "false" | "true" | string | null;
    /**
     * Indicates the entered value does not conform to the format expected by the application.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-invalid}
     * @public
     * @remarks
     * HTML Attribute: aria-invalid
     */
    ariaInvalid: "false" | "true" | "grammar" | "spelling" | string | null;
    /**
     * Indicates keyboard shortcuts that an author has implemented to activate or give focus to an element.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-keyshortcuts}
     * @public
     * @remarks
     * HTML Attribute: aria-keyshortcuts
     */
    ariaKeyshortcuts: string | null;
    /**
     * Defines a string value that labels the current element.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-label}
     * @public
     * @remarks
     * HTML Attribute: aria-label
     */
    ariaLabel: string | null;
    /**
     * Identifies the element (or elements) that labels the current element.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-labelledby}
     * @public
     * @remarks
     * HTML Attribute: aria-labelledby
     */
    ariaLabelledby: string | null;
    /**
     * Indicates that an element will be updated, and describes the types of updates the user agents,
     * assistive technologies, and user can expect from the live region.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-live}
     * @public
     * @remarks
     * HTML Attribute: aria-live
     */
    ariaLive: "assertive" | "off" | "polite" | string | null;
    /**
     * Identifies an element (or elements) in order to define a visual,
     * functional, or contextual parent/child relationship between DOM elements
     * where the DOM hierarchy cannot be used to represent the relationship.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-owns}
     * @public
     * @remarks
     * HTML Attribute: aria-owns
     */
    ariaOwns: string | null;
    /**
     * Indicates what notifications the user agent will trigger when the accessibility tree within a live region is modified.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-relevant}
     * @public
     * @remarks
     * HTML Attribute: aria-relevant
     */
    ariaRelevant: "additions" | "additions text" | "all" | "removals" | "text" | string | null;
    /**
     * Defines a human-readable, author-localized description for the role of an element.
     *
     * {@link https://www.w3.org/TR/wai-aria-1.1/#aria-roledescription}
     * @public
     * @remarks
     * HTML Attribute: aria-roledescription
     */
    ariaRoledescription: string | null;
}

/**
 * Defines if the component updates its position automatically. Calling update() always provokes an update.
 * anchor - the component only updates its position when the anchor resizes (default)
 * auto - the component updates its position when:
 * - update() is called
 * - the anchor resizes
 * - the window resizes
 * - the viewport resizes
 * - any scroll event in the document
 *
 * @public
 */
export declare type AutoUpdateMode = "anchor" | "auto";

/**
 * An Avatar Custom HTML Element
 *
 * @slot media - Used for media such as an image
 * @slot - The default slot for avatar text, commonly a name or initials
 * @slot badge - Used to provide a badge, such as a status badge
 * @csspart backplate - The wrapping container for the avatar
 * @csspart link - The avatar link
 * @csspart content - The default slot
 *
 * @public
 */
export declare class Avatar extends FoundationElement {
    /**
     * Indicates the Avatar should have a color fill.
     *
     * @public
     * @remarks
     * HTML Attribute: fill
     */
    fill: string;
    /**
     * Indicates the Avatar should have a text color.
     *
     * @public
     * @remarks
     * HTML Attribute: color
     */
    color: string;
    /**
     * Indicates the Avatar should have url link
     *
     * @public
     * @remarks
     * HTML Attribute: link
     */
    link: string;
    /**
     * Indicates the Avatar shape should be. By default it will be set to "circle".
     *
     * @public
     * @remarks
     * HTML Attribute: shape
     */
    shape: AvatarShape;
    /**
     * Internal
     */
    connectedCallback(): void;
}

/**
 * Avatar configuration options
 * @public
 */
export declare type AvatarOptions = FoundationElementDefinition & {
    media?: string | SyntheticViewTemplate;
};

declare type AvatarShape = "circle" | "square";

/**
 * The template for {@link @microsoft/fast-foundation#Avatar} component.
 * @public
 */
export declare const avatarTemplate: FoundationElementTemplate<ViewTemplate<Avatar>, AvatarOptions>;

/**
 * Defines the base behavior of an anchored region on a particular axis
 *
 * @public
 */
export declare type AxisPositioningMode = "uncontrolled" | "locktodefault" | "dynamic";

/**
 * Defines the scaling behavior of an anchored region on a particular axis
 *
 * @public
 */
export declare type AxisScalingMode = "anchor" | "fill" | "content";

/**
 * A Badge Custom HTML Element.
 * @slot - The default slot for the badge
 * @csspart control - The element representing the badge, which wraps the default slot
 *
 * @public
 */
export declare class Badge extends FoundationElement {
    /**
     * Indicates the badge should have a filled style.
     * @public
     * @remarks
     * HTML Attribute: fill
     */
    fill: string;
    /**
     * Indicates the badge should have a filled style.
     *
     * @public
     * @remarks
     * HTML Attribute: color
     * @privateRemarks
     * Revisit this once we have a better story for ensuring proper contrast from author defined `fill`
     */
    color: string;
    /**
     * Indicates the element should be circular
     *
     * @public
     * @remarks
     * HTML Attribute: circular
     */
    circular: boolean;
    generateBadgeStyle: () => string | undefined;
}

/**
 * The template for the {@link @microsoft/fast-foundation#Badge} component.
 * @public
 */
export declare const badgeTemplate: FoundationElementTemplate<ViewTemplate<Badge>>;

/**
 * An Progress HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#progressbar | ARIA progressbar }.
 *
 * @slot indeterminate - The slot for a custom indeterminate indicator
 * @csspart progress - Represents the progress element
 * @csspart determinate - The determinate indicator
 * @csspart indeterminate - The indeterminate indicator
 *
 * @public
 */
export declare class BaseProgress extends FoundationElement {
    /**
     * The value of the progress
     * @public
     * @remarks
     * HTML Attribute: value
     */
    value: number | null;
    protected valueChanged(): void;
    /**
     * The minimum value
     * @public
     * @remarks
     * HTML Attribute: min
     */
    min: number;
    protected minChanged(): void;
    /**
     * The maximum value
     * @public
     * @remarks
     * HTML Attribute: max
     */
    max: number;
    protected maxChanged(): void;
    /**
     * Indicates the progress is paused
     * @public
     * @remarks
     * HTML Attribute: paused
     */
    paused: boolean;
    /* Excluded from this release type: percentComplete */
    /* Excluded from this release type: connectedCallback */
    private updatePercentComplete;
}

/**
 * A Breadcrumb Custom HTML Element.
 * @slot - The default slot for the breadcrumb items
 * @csspart list - The element wrapping the slotted items
 *
 * @public
 */
export declare class Breadcrumb extends FoundationElement {
    /* Excluded from this release type: slottedBreadcrumbItems */
    protected slottedBreadcrumbItemsChanged(): void;
    private setItemSeparator;
    /**
     * Finds href on childnodes in the light DOM or shadow DOM.
     * We look in the shadow DOM because we insert an anchor when breadcrumb-item has an href.
     */
    private findChildWithHref;
    /**
     *  Sets ARIA Current for the current node
     * If child node with an anchor tag and with href is found then set aria-current to correct value for the child node,
     * otherwise apply aria-current to the host element, with an href
     */
    private setAriaCurrent;
}

/**
 * A Breadcrumb Item Custom HTML Element.
 *
 * @public
 */
export declare class BreadcrumbItem extends Anchor {
    /* Excluded from this release type: separator */
}

/* Excluded declaration from this release type: BreadcrumbItem */

/**
 * Breadcrumb Item configuration options
 *
 * @slot - The default slot for when no href is provided or for providing your own custom elements
 * @slot separator - The slot for providing a custom separator
 * @csspart listitem - The wrapping container for the item, represents a semantic listitem
 * @csspart separator - The wrapping element for the separator
 *
 * @public
 */
export declare type BreadcrumbItemOptions = FoundationElementDefinition & StartEndOptions & {
    separator?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(BreadcrumbItem:class)} component.
 * @public
 */
export declare const breadcrumbItemTemplate: FoundationElementTemplate<ViewTemplate<BreadcrumbItem>, BreadcrumbItemOptions>;

/**
 * The template for the {@link @microsoft/fast-foundation#Breadcrumb} component.
 * @public
 */
export declare const breadcrumbTemplate: FoundationElementTemplate<ViewTemplate<Breadcrumb>>;

/**
 * A Button Custom HTML Element.
 * Based largely on the {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/button | <button> element }.
 *
 * @slot start - Content which can be provided before the button content
 * @slot end - Content which can be provided after the button content
 * @slot - The default slot for button content
 * @csspart control - The button element
 * @csspart content - The element wrapping button content
 *
 * @public
 */
export declare class Button extends FormAssociatedButton {
    /**
     * Determines if the element should receive document focus on page load.
     *
     * @public
     * @remarks
     * HTML Attribute: autofocus
     */
    autofocus: boolean;
    /**
     * The id of a form to associate the element to.
     *
     * @public
     * @remarks
     * HTML Attribute: form
     */
    formId: string;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/button | <button> element} for more details.
     *
     * @public
     * @remarks
     * HTML Attribute: formaction
     */
    formaction: string;
    protected formactionChanged(): void;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/button | <button> element} for more details.
     *
     * @public
     * @remarks
     * HTML Attribute: formenctype
     */
    formenctype: string;
    protected formenctypeChanged(): void;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/button | <button> element} for more details.
     *
     * @public
     * @remarks
     * HTML Attribute: formmethod
     */
    formmethod: string;
    protected formmethodChanged(): void;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/button | <button> element} for more details.
     *
     * @public
     * @remarks
     * HTML Attribute: formnovalidate
     */
    formnovalidate: boolean;
    protected formnovalidateChanged(): void;
    /**
     * See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/button | <button> element} for more details.
     *
     * @public
     * @remarks
     * HTML Attribute: formtarget
     */
    formtarget: "_self" | "_blank" | "_parent" | "_top";
    protected formtargetChanged(): void;
    /**
     * The button type.
     *
     * @public
     * @remarks
     * HTML Attribute: type
     */
    type: "submit" | "reset" | "button";
    protected typeChanged(previous: "submit" | "reset" | "button" | void, next: "submit" | "reset" | "button"): void;
    /**
     *
     * Default slotted content
     *
     * @public
     * @remarks
     */
    defaultSlottedContent: HTMLElement[];
    /* Excluded from this release type: connectedCallback */
    /* Excluded from this release type: disconnectedCallback */
    /* Excluded from this release type: handleClick */
    /**
     * Submits the parent form
     */
    private handleSubmission;
    /**
     * Resets the parent form
     */
    private handleFormReset;
    control: HTMLButtonElement;
    /**
     * Overrides the focus call for where delegatesFocus is unsupported.
     * This check works for Chrome, Edge Chromium, FireFox, and Safari
     * Relevant PR on the Firefox browser: https://phabricator.services.mozilla.com/D123858
     */
    private handleUnsupportedDelegatesFocus;
}

/* Excluded declaration from this release type: Button */

declare class _Button extends FoundationElement {
}

declare interface _Button extends FormAssociated {
}

/**
 * Button configuration options
 * @public
 */
export declare type ButtonOptions = FoundationElementDefinition & StartEndOptions;

/**
 * The template for the {@link @microsoft/fast-foundation#(Button:class)} component.
 * @public
 */
export declare const buttonTemplate: FoundationElementTemplate<ViewTemplate<Button>, ButtonOptions>;

/**
 * Calendar component
 *
 * @slot - The default slot for calendar content
 * @fires dateselected - Fires a custom 'dateselected' event when Enter is invoked via keyboard on a date
 *
 * @public
 */
export declare class Calendar extends FoundationElement {
    /**
     * date formatter utitlity for getting localized strings
     * @public
     */
    dateFormatter: DateFormatter;
    /**
     * Readonly attribute for turning off data-grid
     * @public
     */
    readonly: boolean;
    /**
     * String repesentation of the full locale including market, calendar type and numbering system
     * @public
     */
    locale: string;
    protected localeChanged(): void;
    /**
     * Month to display
     * @public
     */
    month: number;
    /**
     * Year of the month to display
     * @public
     */
    year: number;
    /**
     * Format style for the day
     * @public
     */
    dayFormat: DayFormat;
    protected dayFormatChanged(): void;
    /**
     * Format style for the week day labels
     * @public
     */
    weekdayFormat: WeekdayFormat;
    protected weekdayFormatChanged(): void;
    /**
     * Format style for the month label
     * @public
     */
    monthFormat: MonthFormat;
    protected monthFormatChanged(): void;
    /**
     * Format style for the year used in the title
     * @public
     */
    yearFormat: YearFormat;
    protected yearFormatChanged(): void;
    /**
     * Minimum number of weeks to show for the month
     * This can be used to normalize the calendar view
     *  when changing or across multiple calendars
     * @public
     */
    minWeeks: number;
    /**
     * A list of dates that should be shown as disabled
     * @public
     */
    disabledDates: string;
    /**
     * A list of dates that should be shown as highlighted
     * @public
     */
    selectedDates: string;
    /* Excluded from this release type: oneDayInMs */
    /**
     * Gets data needed to render about a calendar month as well as the previous and next months
     * @param year - year of the calendar
     * @param month - month of the calendar
     * @returns - an object with data about the current and 2 surrounding months
     * @public
     */
    getMonthInfo(month?: number, year?: number): CalendarInfo;
    /**
     * A list of calendar days
     * @param info - an object containing the information needed to render a calendar month
     * @param minWeeks - minimum number of weeks to show
     * @returns a list of days in a calendar month
     * @public
     */
    getDays(info?: CalendarInfo, minWeeks?: number): CalendarDateInfo[][];
    /**
     * A helper function that checks if a date exists in a list of dates
     * @param date - A date objec that includes the day, month and year
     * @param datesString - a comma separated list of dates
     * @returns - Returns true if it found the date in the list of dates
     * @public
     */
    dateInString(date: Date | string, datesString: string): boolean;
    /**
     * Creates a class string for the day container
     * @param date - date of the calendar cell
     * @returns - string of class names
     * @public
     */
    getDayClassNames(date: CalendarDateInfo, todayString?: string): string;
    /**
     * Returns a list of weekday labels
     * @returns An array of weekday text and full text if abbreviated
     * @public
     */
    getWeekdayText(): WeekdayText[];
    /**
     * Emits the "date-select" event with the day, month and year.
     * @param date - Date cell
     * @public
     */
    handleDateSelect(event: Event, day: CalendarDateInfo): void;
    /**
     * Handles keyboard events on a cell
     * @param event - Keyboard event
     * @param date - Date of the cell selected
     */
    handleKeydown(event: KeyboardEvent, date: CalendarDateInfo): boolean;
}

/**
 * A calendar day template
 * @param context - Element definition context for getting the cell tag for calendar-cell
 * @param todayString - A string representation for todays date
 * @returns - A calendar cell template for a given date
 * @public
 */
export declare const calendarCellTemplate: (context: ElementDefinitionContext, todayString: string) => ItemViewTemplate<CalendarDateInfo>;

/**
 * Caldendar date info
 * used to represent a date
 * @public
 */
export declare type CalendarDateInfo = {
    day: number;
    month: number;
    year: number;
    disabled?: boolean;
    selected?: boolean;
};

/**
 * Calendar information needed for rendering
 * including the next and previous months
 * @public
 */
export declare type CalendarInfo = MonthInfo & {
    previous: MonthInfo;
    next: MonthInfo;
};

/**
 * Calendar configuration options
 * @public
 */
export declare type CalendarOptions = FoundationElementDefinition & StartEndOptions & {
    title?: FoundationElementTemplate<SyntheticViewTemplate<any, Calendar>, CalendarOptions> | SyntheticViewTemplate | string;
};

/**
 *
 * @param context - Element definition context for getting the cell tag for calendar-cell
 * @param todayString - A string representation for todays date
 * @returns - A template for a week of days
 * @public
 */
export declare const calendarRowTemplate: (context: ElementDefinitionContext, todayString: string) => ChildViewTemplate;

/**
 * The template for the {@link @microsoft/fast-foundation#(Calendar:class)} component.
 *
 * @param context - Element definition context for getting the cell tag for calendar-cell
 * @param definition - Foundation element definition
 * @returns - a template for a calendar month
 * @public
 */
export declare const calendarTemplate: FoundationElementTemplate<ViewTemplate<Calendar>, CalendarOptions>;

/**
 * A basic Calendar title template that includes the month and year
 * @returns - A calendar title template
 * @public
 */
export declare const CalendarTitleTemplate: ViewTemplate<Calendar>;

/**
 * Calendar weekday label template
 * @returns - The weekday labels template
 * @public
 */
export declare const calendarWeekdayTemplate: (context: ElementDefinitionContext) => ItemViewTemplate;

/**
 * An Card Custom HTML Element.
 *
 * @slot - The default slot for the card content
 *
 * @public
 */
export declare class Card extends FoundationElement {
}

/**
 * The template for the {@link @microsoft/fast-foundation#Card} component.
 * @public
 */
export declare const cardTemplate: FoundationElementTemplate<ViewTemplate<Card>>;

/**
 * Creates a checkable form associated component.
 * @beta
 */
export declare function CheckableFormAssociated<T extends ConstructableFormAssociated>(BaseCtor: T): T;

/**
 * Base class for providing Custom Element Form Association with checkable features.
 *
 * @beta
 */
export declare interface CheckableFormAssociated extends FormAssociated {
    currentChecked: boolean;
    dirtyChecked: boolean;
    checkedAttribute: boolean;
    defaultChecked: boolean;
    defaultCheckedChanged(oldValue: boolean | undefined, newValue: boolean): void;
    checked: boolean;
    checkedChanged(oldValue: boolean | undefined, newValue: boolean): void;
}

/**
 * Combined type to describe a checkable Form-associated element.
 *
 * @beta
 */
export declare type CheckableFormAssociatedElement = FormAssociatedElement & CheckableFormAssociated & {
    proxy: HTMLInputElement;
};

/**
 * A Checkbox Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#checkbox | ARIA checkbox }.
 *
 * @slot checked-indicator - The checked indicator
 * @slot indeterminate-indicator - The indeterminate indicator
 * @slot - The default slot for the label
 * @csspart control - The element representing the visual checkbox control
 * @csspart label - The label
 * @fires change - Emits a custom change event when the checked state changes
 *
 * @public
 */
export declare class Checkbox extends FormAssociatedCheckbox {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    protected readOnlyChanged(): void;
    /* Excluded from this release type: initialValue */
    /* Excluded from this release type: defaultSlottedNodes */
    /**
     * The indeterminate state of the control
     */
    indeterminate: boolean;
    constructor();
    /* Excluded from this release type: keypressHandler */
    /* Excluded from this release type: clickHandler */
}

declare class _Checkbox extends FoundationElement {
}

declare interface _Checkbox extends CheckableFormAssociated {
}

/**
 * Checkbox configuration options
 * @public
 */
export declare type CheckboxOptions = FoundationElementDefinition & {
    checkedIndicator?: string | SyntheticViewTemplate;
    indeterminateIndicator?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(Checkbox:class)} component.
 * @public
 */
export declare const checkboxTemplate: FoundationElementTemplate<ViewTemplate<Checkbox>, CheckboxOptions>;

/**
 * Defines a column in the grid
 *
 * @public
 */
export declare interface ColumnDefinition {
    /**
     * Identifies the data item to be displayed in this column
     * (i.e. how the data item is labelled in each row)
     */
    columnDataKey: string;
    /**
     * Sets the css grid-column property on the cell which controls its placement in
     * the parent row. If left unset the cells will set this value to match the index
     * of their column in the parent collection of ColumnDefinitions.
     */
    gridColumn?: string;
    /**
     *  Column title, if not provided columnDataKey is used as title
     */
    title?: string;
    /**
     *  Header cell template
     */
    headerCellTemplate?: ViewTemplate;
    /**
     * Whether the header cell has an internal focus queue
     */
    headerCellInternalFocusQueue?: boolean;
    /**
     * Callback function that returns the element to focus in a custom cell.
     * When headerCellInternalFocusQueue is false this function is called when the cell is first focused
     * to immediately move focus to a cell element, for example a cell that is a checkbox could move
     * focus directly to the checkbox.
     * When headerCellInternalFocusQueue is true this function is called when the user hits Enter or F2
     */
    headerCellFocusTargetCallback?: (cell: DataGridCell) => HTMLElement;
    /**
     * cell template
     */
    cellTemplate?: ViewTemplate;
    /**
     * Whether the cell has an internal focus queue
     */
    cellInternalFocusQueue?: boolean;
    /**
     * Callback function that returns the element to focus in a custom cell.
     * When cellInternalFocusQueue is false this function is called when the cell is first focused
     * to immediately move focus to a cell element, for example a cell that is a checkbox could move
     * focus directly to the checkbox.
     * When cellInternalFocusQueue is true this function is called when the user hits Enter or F2
     */
    cellFocusTargetCallback?: (cell: DataGridCell) => HTMLElement;
    /**
     * Whether this column is the row header
     */
    isRowHeader?: boolean;
}

/**
 * A Combobox Custom HTML Element.
 * Implements the {@link https://w3c.github.io/aria-practices/#combobox | ARIA combobox }.
 *
 * @slot start - Content which can be provided before the input
 * @slot end - Content which can be provided after the input
 * @slot control - Used to replace the input element representing the combobox
 * @slot indicator - The visual indicator representing the expanded state
 * @slot - The default slot for the options
 * @csspart control - The wrapper element containing the input area, including start and end
 * @csspart selected-value - The input element representing the selected value
 * @csspart indicator - The element wrapping the indicator slot
 * @csspart listbox - The wrapper for the listbox slotted options
 * @fires change - Fires a custom 'change' event when the value updates
 *
 * @public
 */
export declare class Combobox extends FormAssociatedCombobox {
    /* Excluded from this release type: _value */
    /**
     * The autocomplete attribute.
     *
     * @public
     * @remarks
     * HTML Attribute: autocomplete
     */
    autocomplete: ComboboxAutocomplete | undefined;
    /* Excluded from this release type: control */
    /* Excluded from this release type: listbox */
    /**
     * The collection of currently filtered options.
     *
     * @public
     */
    filteredOptions: ListboxOption[];
    /* Excluded from this release type: filter */
    /* Excluded from this release type: forcedPosition */
    /* Excluded from this release type: formResetCallback */
    private get isAutocompleteInline();
    private get isAutocompleteList();
    private get isAutocompleteBoth();
    /* Excluded from this release type: listboxId */
    /* Excluded from this release type: maxHeight */
    /**
     * The open attribute.
     *
     * @public
     * @remarks
     * HTML Attribute: open
     */
    open: boolean;
    /* Excluded from this release type: openChanged */
    /**
     * The list of options.
     *
     * @public
     * @remarks
     * Overrides `Listbox.options`.
     */
    get options(): ListboxOption[];
    set options(value: ListboxOption[]);
    /**
     * Sets the placeholder value of the element, generally used to provide a hint to the user.
     * @public
     * @remarks
     * HTML Attribute: placeholder
     * Using this attribute is not a valid substitute for a labeling element.
     */
    placeholder: string;
    /* Excluded from this release type: placeholderChanged */
    /**
     * The placement for the listbox when the combobox is open.
     *
     * @public
     */
    positionAttribute?: SelectPosition;
    /**
     * The current state of the calculated position of the listbox.
     *
     * @public
     */
    position?: SelectPosition;
    protected positionChanged(prev: SelectPosition | undefined, next: SelectPosition | undefined): void;
    /**
     * The value property.
     *
     * @public
     */
    get value(): string;
    set value(next: string);
    /* Excluded from this release type: clickHandler */
    connectedCallback(): void;
    /* Excluded from this release type: disabledChanged */
    /**
     * Filter available options by text value.
     *
     * @public
     */
    filterOptions(): void;
    /* Excluded from this release type: focusAndScrollOptionIntoView */
    /* Excluded from this release type: focusoutHandler */
    /* Excluded from this release type: inputHandler */
    /* Excluded from this release type: keydownHandler */
    /* Excluded from this release type: keyupHandler */
    /* Excluded from this release type: selectedIndexChanged */
    /* Excluded from this release type: selectPreviousOption */
    /* Excluded from this release type: setDefaultSelectedOption */
    /* Excluded from this release type: setInlineSelection */
    /**
     * Calculate and apply listbox positioning based on available viewport space.
     *
     * @param force - direction to force the listbox to display
     * @public
     */
    setPositioning(): void;
    /* Excluded from this release type: selectedOptionsChanged */
    /* Excluded from this release type: slottedOptionsChanged */
    /* Excluded from this release type: updateValue */
    /* Excluded from this release type: clearSelectionRange */
}

/* Excluded declaration from this release type: Combobox */

declare class _Combobox extends Listbox {
}

declare interface _Combobox extends FormAssociated {
}

/**
 * Autocomplete values for combobox.
 * @public
 */
export declare const ComboboxAutocomplete: {
    readonly inline: "inline";
    readonly list: "list";
    readonly both: "both";
    readonly none: "none";
};

/**
 * Autocomplete type for combobox.
 * @public
 */
export declare type ComboboxAutocomplete = typeof ComboboxAutocomplete[keyof typeof ComboboxAutocomplete];

/**
 * Combobox configuration options
 * @public
 */
export declare type ComboboxOptions = FoundationElementDefinition & StartEndOptions & {
    indicator?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(Combobox:class)} component.
 * @public
 */
export declare const comboboxTemplate: FoundationElementTemplate<ViewTemplate<Combobox>, ComboboxOptions>;

/**
 * Applies presentation details, such as template and styles, to a component instance.
 * @public
 */
export declare interface ComponentPresentation {
    /**
     * Applies the presentation details to the specified element.
     * @param element - The element to apply the presentation details to.
     * @public
     */
    applyTo(element: FASTElement): void;
}

/**
 * An API gateway to component presentation features.
 * @public
 */
export declare const ComponentPresentation: Readonly<{
    /**
     * Defines a component presentation for an element.
     * @param tagName - The element name to define the presentation for.
     * @param presentation - The presentation that will be applied to matching elements.
     * @param container - The dependency injection container to register the configuration in.
     * @public
     */
    define(tagName: string, presentation: ComponentPresentation, container: Container): void;
    /**
     * Finds a component presentation for the specified element name,
     * searching the DOM hierarchy starting from the provided element.
     * @param tagName - The name of the element to locate the presentation for.
     * @param element - The element to begin the search from.
     * @returns The component presentation or null if none is found.
     * @public
     */
    forTag(tagName: string, element: HTMLElement): ComponentPresentation | null;
}>;

export { composedContains }

export { composedParent }

/**
 * Combined type to describe a Constructable Form-Associated type.
 *
 * @beta
 */
export declare type ConstructableFormAssociated = Constructable<HTMLElement & FASTElement>;

/**
 * Implemented by dependency injection containers.
 * @public
 */
export declare interface Container extends ServiceLocator {
    /**
     * Registers dependencies with the container via registration objects.
     * @param params - The registration objects.
     */
    register(...params: any[]): Container;
    /**
     * Registers dependencies with the container via registration objects, providing
     * the specified context to each register invocation.
     * @param context - The context object to pass to the registration objects.
     * @param params - The registration objects.
     */
    registerWithContext(context: any, ...params: any[]): Container;
    /**
     * Registers a resolver with the container for the specified key.
     * @param key - The key to register the resolver under.
     * @param resolver - The resolver to register.
     */
    registerResolver<K extends Key, T = K>(key: K, resolver: Resolver<T>): Resolver<T>;
    /**
     * Registers a transformer with the container for the specified key.
     * @param key - The key to resolved to register the transformer with.
     * @param transformer - The transformer to register.
     */
    registerTransformer<K extends Key, T = K>(key: K, transformer: Transformer_2<T>): boolean;
    /**
     * Gets a resolver for the specified key.
     * @param key - The key to get the resolver for.
     * @param autoRegister - Indicates whether or not to try to auto-register a dependency for
     * the key if one is not explicitly registered.
     */
    getResolver<K extends Key, T = K>(key: K | Key, autoRegister?: boolean): Resolver<T> | null;
    /**
     * Registers a factory with the container for the specified key.
     * @param key - The key to register the factory under.
     * @param factory - The factory to register.
     */
    registerFactory<T extends Constructable>(key: T, factory: Factory<T>): void;
    /**
     * Gets the factory for the specified key.
     * @param key - The key to get the factory for.
     */
    getFactory<T extends Constructable>(key: T): Factory<T>;
    /**
     * Creates a child dependency injection container parented to this container.
     * @param config - The configuration for the new container.
     */
    createChild(config?: Partial<Omit<ContainerConfiguration, "parentLocator">>): Container;
}

/**
 * The interface key that resolves the dependency injection container itself.
 * @public
 */
export declare const Container: InterfaceSymbol<Container>;

/**
 * Configuration for a dependency injection container.
 * @public
 */
export declare interface ContainerConfiguration {
    /**
     * The locator function used to find the parent of the container.
     */
    parentLocator: ParentLocator;
    /**
     * Indicates whether this container should resolve dependencies that are directly made
     * by its owner. The default is "false" which results in the parent container being used.
     */
    responsibleForOwnerRequests: boolean;
    /**
     * Gets the default resolver to use during auto-registration.
     * @param key - The key to register the dependency with.
     * @param handler - The container that is handling the auto-registration request.
     */
    defaultResolver(key: Key, handler: Container): Resolver;
}

/**
 * Configuration for a dependency injection container.
 * @public
 */
export declare const ContainerConfiguration: Readonly<{
    /**
     * The default configuration used when creating a DOM-disconnected container.
     * @remarks
     * The default creates a root container, with no parent container. It does not handle
     * owner requests and it uses singleton resolution behavior for auto-registration.
     */
    default: Readonly<ContainerConfiguration>;
}>;

/* Excluded from this release type: ContainerImpl */

/**
 * Enables defining an element within the context of a design system.
 * @public
 */
export declare type ContextualElementDefinition = Omit<PartialFASTElementDefinition, "name">;

declare function create<T extends Function>(nameOrConfig: string | DesignTokenConfiguration): never;

declare function create<T extends undefined | void>(nameOrConfig: string | DesignTokenConfiguration): never;

declare function create<T>(nameOrConfig: string): CSSDesignToken<T>;

declare function create<T>(nameOrConfig: Omit<DesignTokenConfiguration, "cssCustomPropertyName"> | (DesignTokenConfiguration & Record<"cssCustomPropertyName", string>)): CSSDesignToken<T>;

declare function create<T>(nameOrConfig: DesignTokenConfiguration & Record<"cssCustomPropertyName", null>): DesignToken<T>;

/**
 * A {@link (DesignToken:interface)} that emits a CSS custom property.
 * @public
 */
export declare interface CSSDesignToken<T extends string | number | boolean | BigInteger | null | Array<any> | symbol | ({
    createCSS?(): string;
} & Record<PropertyKey, any>)> extends DesignToken<T>, CSSDirective {
    /**
     * The {@link (DesignToken:interface)} formatted as a CSS custom property if the token is
     * configured to write a CSS custom property.
     */
    readonly cssCustomProperty: string;
}

/**
 * Define all possible CSS display values.
 * @public
 * @deprecated - fast-foundation is removing styling utilities
 */
export declare type CSSDisplayPropertyValue = "block" | "contents" | "flex" | "grid" | "inherit" | "initial" | "inline" | "inline-block" | "inline-flex" | "inline-grid" | "inline-table" | "list-item" | "none" | "run-in" | "table" | "table-caption" | "table-cell" | "table-column" | "table-column-group" | "table-footer-group" | "table-header-group" | "table-row" | "table-row-group";

/**
 * This can be used to construct a behavior to apply a prefers color scheme: dark only stylesheet.
 * @public
 */
export declare const darkModeStylesheetBehavior: (styles: ElementStyles) => MatchMediaStyleSheetBehavior;

/**
 * A Data Grid Custom HTML Element.
 *
 * @slot - The default slot for custom row elements
 * @public
 */
export declare class DataGrid extends FoundationElement {
    /**
     *  generates a basic column definition by examining sample row data
     */
    static generateColumns: (row: object) => ColumnDefinition[];
    /**
     *  generates a gridTemplateColumns based on columndata array
     */
    private static generateTemplateColumns;
    /**
     * When true the component will not add itself to the tab queue.
     * Default is false.
     *
     * @public
     * @remarks
     * HTML Attribute: no-tabbing
     */
    noTabbing: boolean;
    protected noTabbingChanged(): void;
    /**
     *  Whether the grid should automatically generate a header row and its type
     *
     * @public
     * @remarks
     * HTML Attribute: generate-header
     */
    generateHeader: GenerateHeaderOptions;
    private generateHeaderChanged;
    /**
     * String that gets applied to the the css gridTemplateColumns attribute of child rows
     *
     * @public
     * @remarks
     * HTML Attribute: grid-template-columns
     */
    gridTemplateColumns: string;
    protected gridTemplateColumnsChanged(): void;
    /**
     * The data being displayed in the grid
     *
     * @public
     */
    rowsData: object[];
    protected rowsDataChanged(): void;
    /**
     * The column definitions of the grid
     *
     * @public
     */
    columnDefinitions: ColumnDefinition[] | null;
    protected columnDefinitionsChanged(): void;
    /**
     * The template to use for the programmatic generation of rows
     *
     * @public
     */
    rowItemTemplate: ItemViewTemplate;
    /**
     * The template used to render cells in generated rows.
     *
     * @public
     */
    cellItemTemplate?: ItemViewTemplate;
    /**
     * The template used to render header cells in generated rows.
     *
     * @public
     */
    headerCellItemTemplate?: ItemViewTemplate;
    private headerCellItemTemplateChanged;
    /**
     * The index of the row that will receive focus the next time the
     * grid is focused. This value changes as focus moves to different
     * rows within the grid.  Changing this value when focus is already
     * within the grid moves focus to the specified row.
     *
     * @public
     */
    focusRowIndex: number;
    private focusRowIndexChanged;
    /**
     * The index of the column that will receive focus the next time the
     * grid is focused. This value changes as focus moves to different rows
     * within the grid.  Changing this value when focus is already within
     * the grid moves focus to the specified column.
     *
     * @public
     */
    focusColumnIndex: number;
    private focusColumnIndexChanged;
    /* Excluded from this release type: defaultRowItemTemplate */
    /**
     * Set by the component templates.
     *
     */
    rowElementTag: string;
    /* Excluded from this release type: rowElements */
    private rowsRepeatBehavior;
    private rowsPlaceholder;
    private generatedHeader;
    private isUpdatingFocus;
    private pendingFocusUpdate;
    private observer;
    private rowindexUpdateQueued;
    private columnDefinitionsStale;
    private generatedGridTemplateColumns;
    constructor();
    /* Excluded from this release type: connectedCallback */
    /* Excluded from this release type: disconnectedCallback */
    /* Excluded from this release type: handleRowFocus */
    /* Excluded from this release type: handleFocus */
    /* Excluded from this release type: handleFocusOut */
    /* Excluded from this release type: handleKeydown */
    private focusOnCell;
    private queueFocusUpdate;
    private updateFocus;
    private toggleGeneratedHeader;
    private onChildListChange;
    private queueRowIndexUpdate;
    private updateRowIndexes;
}

/**
 * A Data Grid Cell Custom HTML Element.
 *
 * @fires cell-focused - Fires a custom 'cell-focused' event when focus is on the cell or its contents
 * @slot - The default slot for cell contents.  The "cell contents template" renders here.
 * @public
 */
export declare class DataGridCell extends FoundationElement {
    /**
     * The type of cell
     *
     * @public
     * @remarks
     * HTML Attribute: cell-type
     */
    cellType: DataGridCellTypes;
    private cellTypeChanged;
    /**
     * The column index of the cell.
     * This will be applied to the css grid-column-index value
     * applied to the cell
     *
     * @public
     * @remarks
     * HTML Attribute: grid-column
     */
    gridColumn: string;
    protected gridColumnChanged(): void;
    /**
     * The base data for the parent row
     *
     * @public
     */
    rowData: object | null;
    /**
     * The base data for the column
     *
     * @public
     */
    columnDefinition: ColumnDefinition | null;
    protected columnDefinitionChanged(oldValue: ColumnDefinition | null, newValue: ColumnDefinition | null): void;
    private isActiveCell;
    private customCellView;
    /* Excluded from this release type: connectedCallback */
    /* Excluded from this release type: disconnectedCallback */
    handleFocusin(e: FocusEvent): void;
    handleFocusout(e: FocusEvent): void;
    handleKeydown(e: KeyboardEvent): void;
    private updateCellView;
    private disconnectCellView;
    private updateCellStyle;
}

/**
 * Generates a template for the {@link @microsoft/fast-foundation#DataGridCell} component using
 * the provided prefix.
 * @public
 */
export declare const dataGridCellTemplate: FoundationElementTemplate<ViewTemplate<DataGridCell>>;

/**
 * Enumerates possible data grid cell types.
 *
 * @public
 */
export declare const DataGridCellTypes: {
    readonly default: "default";
    readonly columnHeader: "columnheader";
    readonly rowHeader: "rowheader";
};

/**
 * The possible cell types.
 *
 * @public
 */
export declare type DataGridCellTypes = typeof DataGridCellTypes[keyof typeof DataGridCellTypes];

/**
 * A Data Grid Row Custom HTML Element.
 *
 * @fires row-focused - Fires a custom 'row-focused' event when focus is on an element (usually a cell or its contents) in the row
 * @slot - The default slot for custom cell elements
 * @public
 */
export declare class DataGridRow extends FoundationElement {
    /**
     * String that gets applied to the the css gridTemplateColumns attribute for the row
     *
     * @public
     * @remarks
     * HTML Attribute: grid-template-columns
     */
    gridTemplateColumns: string;
    protected gridTemplateColumnsChanged(): void;
    /**
     * The type of row
     *
     * @public
     * @remarks
     * HTML Attribute: row-type
     */
    rowType: DataGridRowTypes;
    private rowTypeChanged;
    /**
     * The base data for this row
     *
     * @public
     */
    rowData: object | null;
    protected rowDataChanged(): void;
    /**
     * The column definitions of the row
     *
     * @public
     */
    columnDefinitions: ColumnDefinition[] | null;
    /**
     * The template used to render cells in generated rows.
     *
     * @public
     */
    cellItemTemplate?: ItemViewTemplate;
    private cellItemTemplateChanged;
    /**
     * The template used to render header cells in generated rows.
     *
     * @public
     */
    headerCellItemTemplate?: ItemViewTemplate;
    private headerCellItemTemplateChanged;
    /**
     * The index of the row in the parent grid.
     * This is typically set programmatically by the parent grid.
     *
     * @public
     */
    rowIndex: number;
    /* Excluded from this release type: isActiveRow */
    /* Excluded from this release type: activeCellItemTemplate */
    /* Excluded from this release type: defaultCellItemTemplate */
    /* Excluded from this release type: defaultHeaderCellItemTemplate */
    /* Excluded from this release type: cellElements */
    private cellsRepeatBehavior;
    private cellsPlaceholder;
    /* Excluded from this release type: slottedCellElements */
    /* Excluded from this release type: focusColumnIndex */
    private refocusOnLoad;
    /* Excluded from this release type: connectedCallback */
    /* Excluded from this release type: disconnectedCallback */
    handleFocusout(e: FocusEvent): void;
    handleCellFocus(e: Event): void;
    handleKeydown(e: KeyboardEvent): void;
    private updateItemTemplate;
    private updateRowStyle;
}

/**
 * Generates a template for the {@link @microsoft/fast-foundation#DataGridRow} component using
 * the provided prefix.
 *
 * @public
 */
export declare const dataGridRowTemplate: FoundationElementTemplate<ViewTemplate<DataGridRow>>;

/**
 * Enumerates possible data grid row types
 *
 * @public
 */
export declare const DataGridRowTypes: {
    readonly default: "default";
    readonly header: "header";
    readonly stickyHeader: "sticky-header";
};

/**
 * The possible data grid row types
 *
 * @public
 */
export declare type DataGridRowTypes = typeof DataGridRowTypes[keyof typeof DataGridRowTypes];

/**
 * Generates a template for the {@link @microsoft/fast-foundation#DataGrid} component using
 * the provided prefix.
 *
 * @public
 */
export declare const dataGridTemplate: FoundationElementTemplate<ViewTemplate<DataGrid>>;

/**
 * Date formatting utility
 * @public
 */
export declare class DateFormatter {
    /**
     * Localization settings to use for formatting
     * @public
     */
    locale: string;
    /**
     * Formatting for the day
     * @public
     */
    dayFormat: DayFormat;
    /**
     * Formatting for the weekday labels
     * @public
     */
    weekdayFormat: WeekdayFormat;
    /**
     * Formatting for the month
     * @public
     */
    monthFormat: MonthFormat;
    /**
     * Formatting for the year
     * @public
     */
    yearFormat: YearFormat;
    /**
     * Date used for formatting
     */
    date: Date;
    constructor(config?: {});
    /**
     * Helper function to make sure that the DateFormatter is working with an instance of Date
     * @param date - The date as an object, string or Date insance
     * @returns - A Date instance
     * @public
     */
    getDateObject(date: {
        day: number;
        month: number;
        year: number;
    } | string | Date): Date;
    /**
     *
     * @param date - a valide date as either a Date, string, objec or a DateFormatter
     * @param format - The formatting for the string
     * @param locale - locale data used for formatting
     * @returns A localized string of the date provided
     * @public
     */
    getDate(date?: {
        day: number;
        month: number;
        year: number;
    } | string | Date, format?: Intl.DateTimeFormatOptions, locale?: string): string;
    /**
     *
     * @param day - Day to localize
     * @param format - The formatting for the day
     * @param locale - The locale data used for formatting
     * @returns - A localized number for the day
     * @public
     */
    getDay(day?: number, format?: DayFormat, locale?: string): string;
    /**
     *
     * @param month - The month to localize
     * @param format - The formatting for the month
     * @param locale - The locale data used for formatting
     * @returns - A localized name of the month
     * @public
     */
    getMonth(month?: number, format?: MonthFormat, locale?: string): string;
    /**
     *
     * @param year - The year to localize
     * @param format - The formatting for the year
     * @param locale - The locale data used for formatting
     * @returns - A localized string for the year
     * @public
     */
    getYear(year?: number, format?: YearFormat, locale?: string): string;
    /**
     *
     * @param weekday - The number of the weekday, defaults to Sunday
     * @param format - The formatting for the weekday label
     * @param locale - The locale data used for formatting
     * @returns - A formatted weekday label
     * @public
     */
    getWeekday(weekday?: number, format?: WeekdayFormat, locale?: string): string;
    /**
     *
     * @param format - The formatting for the weekdays
     * @param locale - The locale data used for formatting
     * @returns - An array of the weekday labels
     * @public
     */
    getWeekdays(format?: WeekdayFormat, locale?: string): string[];
}

/**
 * A type representing the different day formats
 * @public
 */
export declare type DayFormat = "2-digit" | "numeric";

/**
 * The default implementation of ComponentPresentation, used by FoundationElement.
 * @public
 */
export declare class DefaultComponentPresentation implements ComponentPresentation {
    /**
     * The styles to apply to the element.
     * @public
     */
    readonly styles: ElementStyles | null;
    /**
     * The template to apply to the element.
     * @public
     */
    readonly template: ElementViewTemplate | null;
    /**
     * Creates an instance of DefaultComponentPresentation.
     * @param template - The template to apply to the element.
     * @param styles - The styles to apply to the element.
     * @public
     */
    constructor(template?: ElementViewTemplate, styles?: ComposableStyles | ComposableStyles[]);
    /**
     * Applies the presentation details to the specified element.
     * @param element - The element to apply the presentation details to.
     * @public
     */
    applyTo(element: FASTElement): void;
}

/**
 * A set of default resolvers useful in configuring a container.
 * @public
 */
export declare const DefaultResolver: Readonly<{
    /**
     * Disables auto-registration and throws for all un-registered dependencies.
     * @param key - The key to create the resolver for.
     */
    none(key: Key): Resolver;
    /**
     * Provides default singleton resolution behavior during auto-registration.
     * @param key - The key to create the resolver for.
     * @returns The resolver.
     */
    singleton(key: Key): Resolver;
    /**
     * Provides default transient resolution behavior during auto-registration.
     * @param key - The key to create the resolver for.
     * @returns The resolver.
     */
    transient(key: Key): Resolver;
}>;

/**
 * Includes ARIA states and properties relating to the ARIA button role
 *
 * @public
 */
export declare class DelegatesARIAButton {
    /**
     * See {@link https://www.w3.org/WAI/PF/aria/roles#button} for more information
     * @public
     * @remarks
     * HTML Attribute: aria-expanded
     */
    ariaExpanded: "true" | "false" | string | null;
    /**
     * See {@link https://www.w3.org/WAI/PF/aria/roles#button} for more information
     * @public
     * @remarks
     * HTML Attribute: aria-pressed
     */
    ariaPressed: "true" | "false" | "mixed" | string | null;
}

/* Excluded declaration from this release type: DelegatesARIAButton */

/**
 * Includes ARIA states and properties relating to the ARIA combobox role.
 *
 * @public
 */
export declare class DelegatesARIACombobox {
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#aria-autocomplete} for more information.
     *
     * @public
     * @remarks
     * HTML Attribute: `aria-autocomplete`
     */
    ariaAutoComplete: "inline" | "list" | "both" | "none" | string | null;
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#aria-controls} for more information.
     *
     * @public
     * @remarks
     * HTML Attribute: `aria-controls`
     */
    ariaControls: string | null;
}

/* Excluded declaration from this release type: DelegatesARIACombobox */

/**
 * Includes ARIA states and properties relating to the ARIA link role
 *
 * @public
 */
export declare class DelegatesARIALink {
    /**
     * See {@link https://www.w3.org/WAI/PF/aria/roles#link} for more information
     * @public
     * @remarks
     * HTML Attribute: aria-expanded
     */
    ariaExpanded: "true" | "false" | string | null;
}

/* Excluded declaration from this release type: DelegatesARIALink */

/**
 * Includes ARIA states and properties relating to the ARIA listbox role
 *
 * @public
 */
export declare class DelegatesARIAListbox {
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#listbox} for more information
     * @public
     * @remarks
     * HTML Attribute: `aria-activedescendant`
     */
    ariaActiveDescendant: string | null;
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#listbox} for more information
     * @public
     * @remarks
     * HTML Attribute: `aria-disabled`
     */
    ariaDisabled: "true" | "false" | string | null;
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#listbox} for more information
     * @public
     * @remarks
     * HTML Attribute: `aria-expanded`
     */
    ariaExpanded: "true" | "false" | string | null;
    /**
     * See {@link https://w3c.github.io/aria/#listbox} for more information
     * @public
     * @remarks
     * HTML Attribute: `aria-multiselectable`
     */
    ariaMultiSelectable: "true" | "false" | string | null;
}

/* Excluded declaration from this release type: DelegatesARIAListbox */

/**
 * States and properties relating to the ARIA `option` role.
 *
 * @public
 */
export declare class DelegatesARIAListboxOption {
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#option} for more information.
     * @public
     * @remarks
     * HTML Attribute: `aria-checked`
     */
    ariaChecked: "true" | "false" | string | null;
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#option} for more information.
     * @public
     * @remarks
     * HTML Attribute: `aria-posinset`
     */
    ariaPosInSet: string | null;
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#option} for more information.
     * @public
     * @remarks
     * HTML Attribute: `aria-selected`
     */
    ariaSelected: "true" | "false" | string | null;
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#option} for more information.
     * @public
     * @remarks
     * HTML Attribute: `aria-setsize`
     */
    ariaSetSize: string | null;
}

/* Excluded declaration from this release type: DelegatesARIAListboxOption */

/**
 * Includes ARIA states and properties relating to the ARIA textbox role
 *
 * @public
 */
export declare class DelegatesARIASearch {
}

/* Excluded declaration from this release type: DelegatesARIASearch */

/**
 * Includes ARIA states and properties relating to the ARIA select role.
 *
 * @public
 */
export declare class DelegatesARIASelect {
    /**
     * See {@link https://www.w3.org/TR/wai-aria-1.2/#combobox} for more information
     * @public
     * @remarks
     * HTML Attribute: `aria-controls`
     */
    ariaControls: string | null;
}

/* Excluded declaration from this release type: DelegatesARIASelect */

/**
 * Includes ARIA states and properties relating to the ARIA textbox role
 *
 * @public
 */
export declare class DelegatesARIATextbox {
}

/* Excluded declaration from this release type: DelegatesARIATextbox */

/**
 * Includes ARIA states and properties relating to the ARIA toolbar role
 *
 * @public
 */
export declare class DelegatesARIAToolbar {
    /**
     * The id of the element labeling the toolbar.
     * @public
     * @remarks
     * HTML Attribute: aria-labelledby
     */
    ariaLabelledby: string | null;
    /**
     * The label surfaced to assistive technologies.
     *
     * @public
     * @remarks
     * HTML Attribute: aria-label
     */
    ariaLabel: string | null;
}

/* Excluded declaration from this release type: DelegatesARIAToolbar */

/**
 * A {@link (DesignToken:interface)} value that is derived. These values can depend on other {@link (DesignToken:interface)}s
 * or arbitrary observable properties.
 * @public
 */
export declare type DerivedDesignTokenValue<T> = T extends Function ? never : (target: HTMLElement) => T;

/**
 * Represents a configurable design system.
 * @public
 */
export declare interface DesignSystem {
    /**
     * Registers components and services with the design system and the
     * underlying dependency injection container.
     * @param params - The registries to pass to the design system
     * and the underlying dependency injection container.
     * @public
     */
    register(...params: any[]): DesignSystem;
    /**
     * Configures the prefix to add to each custom element name.
     * @param prefix - The prefix to use for custom elements.
     * @public
     */
    withPrefix(prefix: string): DesignSystem;
    /**
     * Overrides the default Shadow DOM mode for custom elements.
     * @param mode - The Shadow DOM mode to use for custom elements.
     * @public
     */
    withShadowRootMode(mode: ShadowRootMode): DesignSystem;
    /**
     * Provides a custom callback capable of resolving scenarios where
     * two different elements request the same element name.
     * @param callback - The disambiguation callback.
     * @public
     */
    withElementDisambiguation(callback: ElementDisambiguationCallback): DesignSystem;
    /**
     * Overrides the {@link (DesignToken:interface)} root, controlling where
     * {@link (DesignToken:interface)} default value CSS custom properties
     * are emitted.
     *
     * Providing `null` disables automatic DesignToken registration.
     * @param root - the root to register
     * @public
     */
    withDesignTokenRoot(root: HTMLElement | Document | null): DesignSystem;
}

/**
 * An API gateway to design system features.
 * @public
 */
export declare const DesignSystem: Readonly<{
    /**
     * Returns the HTML element name that the type is defined as.
     * @param type - The type to lookup.
     * @public
     */
    tagFor(type: Constructable): string;
    /**
     * Searches the DOM hierarchy for the design system that is responsible
     * for the provided element.
     * @param element - The element to locate the design system for.
     * @returns The located design system.
     * @public
     */
    responsibleFor(element: HTMLElement): DesignSystem;
    /**
     * Gets the DesignSystem if one is explicitly defined on the provided element;
     * otherwise creates a design system defined directly on the element.
     * @param element - The element to get or create a design system for.
     * @returns The design system.
     * @public
     */
    getOrCreate(node?: Node): DesignSystem;
}>;

/**
 * Design system contextual APIs and configuration usable within component
 * registries.
 * @public
 */
export declare interface DesignSystemRegistrationContext {
    /**
     * The element prefix specified by the design system's configuration.
     * @public
     */
    readonly elementPrefix: string;
    /**
     * Used to attempt to define a custom element.
     * @param name - The name of the element to define.
     * @param type - The type of the constructor to use to define the element.
     * @param callback - A callback to invoke if definition will happen.
     * @public
     * @deprecated - Use the signature with the ElementDefinitionParams param type instead
     */
    tryDefineElement(name: string, type: Constructable<HTMLElement>, callback: ElementDefinitionCallback): void;
    /**
     * Used to attempt to define a custom element.
     * @param params - The custom element definition.
     * @public
     */
    tryDefineElement(params: ElementDefinitionParams): void;
}

/**
 * Describes a DesignToken instance.
 * @public
 */
export declare interface DesignToken<T extends string | number | boolean | BigInteger | null | Array<any> | symbol | {}> {
    /**
     * The name of the token
     */
    readonly name: string;
    /**
     * A list of elements for which the DesignToken has a value set
     */
    readonly appliedTo: HTMLElement[];
    /**
     * Get the token value for an element.
     * @param element - The element to get the value for
     * @returns - The value set for the element, or the value set for the nearest element ancestor.
     */
    getValueFor(element: HTMLElement): StaticDesignTokenValue<T>;
    /**
     * Sets the token to a value for an element.
     * @param element - The element to set the value for.
     * @param value - The value.
     */
    setValueFor(element: HTMLElement, value: DesignTokenValue<T> | DesignToken<T>): void;
    /**
     * Removes a value set for an element.
     * @param element - The element to remove the value from
     */
    deleteValueFor(element: HTMLElement): this;
    /**
     * Associates a default value to the token
     */
    withDefault(value: DesignTokenValue<T> | DesignToken<T>): this;
    /**
     * Subscribes a subscriber to change records for a token. If an element is provided, only
     * change records for that element will be emitted.
     */
    subscribe(subscriber: DesignTokenSubscriber<this>, target?: HTMLElement): void;
    /**
     * Unsubscribes a subscriber from change records for a token.
     */
    unsubscribe(subscriber: DesignTokenSubscriber<this>, target?: HTMLElement): void;
}

/**
 * Factory object for creating {@link (DesignToken:interface)} instances.
 * @public
 */
export declare const DesignToken: Readonly<{
    create: typeof create;
    /**
     * Informs DesignToken that an HTMLElement for which tokens have
     * been set has been connected to the document.
     *
     * The browser does not provide a reliable mechanism to observe an HTMLElement's connectedness
     * in all scenarios, so invoking this method manually is necessary when:
     *
     * 1. Token values are set for an HTMLElement.
     * 2. The HTMLElement does not inherit from FASTElement.
     * 3. The HTMLElement is not connected to the document when token values are set.
     *
     * @param element - The element to notify
     * @returns - true if notification was successful, otherwise false.
     */
    notifyConnection(element: HTMLElement): boolean;
    /**
     * Informs DesignToken that an HTMLElement for which tokens have
     * been set has been disconnected to the document.
     *
     * The browser does not provide a reliable mechanism to observe an HTMLElement's connectedness
     * in all scenarios, so invoking this method manually is necessary when:
     *
     * 1. Token values are set for an HTMLElement.
     * 2. The HTMLElement does not inherit from FASTElement.
     *
     * @param element - The element to notify
     * @returns - true if notification was successful, otherwise false.
     */
    notifyDisconnection(element: HTMLElement): boolean;
    /**
     * Registers and element or document as a DesignToken root.
     * {@link CSSDesignToken | CSSDesignTokens} with default values assigned via
     * {@link (DesignToken:interface).withDefault} will emit CSS custom properties to all
     * registered roots.
     * @param target - The root to register
     */
    registerRoot(target?: HTMLElement | Document): void;
    /**
     * Unregister an element or document as a DesignToken root.
     * @param target - The root to deregister
     */
    unregisterRoot(target?: HTMLElement | Document): void;
}>;

/**
 * Change record provided to to a {@link DesignTokenSubscriber} when a token changes for a target.
 * @public
 */
export declare interface DesignTokenChangeRecord<T extends DesignToken<any>> {
    /**
     * The element for which the value was changed
     */
    target: HTMLElement;
    /**
     * The token that was changed
     */
    token: T;
}

/**
 * Describes a {@link (DesignToken:interface)} configuration
 * @public
 */
export declare interface DesignTokenConfiguration {
    /**
     * The name of the {@link (DesignToken:interface)}.
     */
    name: string;
    /**
     * The name of the CSS custom property to associate to the {@link (DesignToken:interface)}, or null
     * if not CSS custom property should be associated.
     */
    cssCustomPropertyName?: string | null;
}

/**
 * A subscriber that should receive {@link DesignTokenChangeRecord | change records} when a token changes for a target
 * @public
 */
export declare interface DesignTokenSubscriber<T extends DesignToken<any>> {
    handleChange(record: DesignTokenChangeRecord<T>): void;
}

/**
 * The type that a {@link (DesignToken:interface)} can be set to.
 * @public
 */
export declare type DesignTokenValue<T> = StaticDesignTokenValue<T> | DerivedDesignTokenValue<T>;

/**
 * The gateway to dependency injection APIs.
 * @public
 */
export declare const DI: Readonly<{
    /**
     * Creates a new dependency injection container.
     * @param config - The configuration for the container.
     * @returns A newly created dependency injection container.
     */
    createContainer(config?: Partial<ContainerConfiguration>): Container;
    /**
     * Finds the dependency injection container responsible for providing dependencies
     * to the specified node.
     * @param node - The node to find the responsible container for.
     * @returns The container responsible for providing dependencies to the node.
     * @remarks
     * This will be the same as the parent container if the specified node
     * does not itself host a container configured with responsibleForOwnerRequests.
     */
    findResponsibleContainer(node: Node): Container;
    /**
     * Find the dependency injection container up the DOM tree from this node.
     * @param node - The node to find the parent container for.
     * @returns The parent container of this node.
     * @remarks
     * This will be the same as the responsible container if the specified node
     * does not itself host a container configured with responsibleForOwnerRequests.
     */
    findParentContainer(node: Node): Container;
    /**
     * Returns a dependency injection container if one is explicitly owned by the specified
     * node. If one is not owned, then a new container is created and assigned to the node.
     * @param node - The node to find or create the container for.
     * @param config - The configuration for the container if one needs to be created.
     * @returns The located or created container.
     * @remarks
     * This API does not search for a responsible or parent container. It looks only for a container
     * directly defined on the specified node and creates one at that location if one does not
     * already exist.
     */
    getOrCreateDOMContainer(node?: Node, config?: Partial<Omit<ContainerConfiguration, "parentLocator">>): Container;
    /**
     * Gets the "design:paramtypes" metadata for the specified type.
     * @param Type - The type to get the metadata for.
     * @returns The metadata array or undefined if no metadata is found.
     */
    getDesignParamtypes: (Type: Constructable | Injectable) => readonly Key[] | undefined;
    /**
     * Gets the "di:paramtypes" metadata for the specified type.
     * @param Type - The type to get the metadata for.
     * @returns The metadata array or undefined if no metadata is found.
     */
    getAnnotationParamtypes: (Type: Constructable | Injectable) => readonly Key[] | undefined;
    /**
     *
     * @param Type - Gets the "di:paramtypes" metadata for the specified type. If none is found,
     * an empty metadata array is created and added.
     * @returns The metadata array.
     */
    getOrCreateAnnotationParamTypes(Type: Constructable | Injectable): Key[];
    /**
     * Gets the dependency keys representing what is needed to instantiate the specified type.
     * @param Type - The type to get the dependencies for.
     * @returns An array of dependency keys.
     */
    getDependencies(Type: Constructable | Injectable): Key[];
    /**
     * Defines a property on a web component class. The value of this property will
     * be resolved from the dependency injection container responsible for the element
     * instance, based on where it is connected in the DOM.
     * @param target - The target to define the property on.
     * @param propertyName - The name of the property to define.
     * @param key - The dependency injection key.
     * @param respectConnection - Indicates whether or not to update the property value if the
     * hosting component is disconnected and then re-connected at a different location in the DOM.
     * @remarks
     * The respectConnection option is only applicable to elements that descend from FASTElement.
     */
    defineProperty(target: {}, propertyName: string, key: Key, respectConnection?: boolean): void;
    /**
     * Creates a dependency injection key.
     * @param nameConfigOrCallback - A friendly name for the key or a lambda that configures a
     * default resolution for the dependency.
     * @param configuror - If a friendly name was provided for the first parameter, then an optional
     * lambda that configures a default resolution for the dependency can be provided second.
     * @returns The created key.
     * @remarks
     * The created key can be used as a property decorator or constructor parameter decorator,
     * in addition to its standard use in an inject array or through direct container APIs.
     */
    createInterface<K extends Key>(nameConfigOrCallback?: string | InterfaceConfiguration | ((builder: ResolverBuilder<K>) => Resolver<K>) | undefined, configuror?: ((builder: ResolverBuilder<K>) => Resolver<K>) | undefined): InterfaceSymbol<K>;
    /**
     * A decorator that specifies what to inject into its target.
     * @param dependencies - The dependencies to inject.
     * @returns The decorator to be applied to the target class.
     * @remarks
     * The decorator can be used to decorate a class, listing all of the classes dependencies.
     * Or it can be used to decorate a constructor paramter, indicating what to inject for that
     * parameter.
     * Or it can be used for a web component property, indicating what that property should resolve to.
     */
    inject(...dependencies: Key[]): (target: any, key?: string | number, descriptor?: PropertyDescriptor | number) => void;
    /**
     * Registers the `target` class as a transient dependency; each time the dependency is resolved
     * a new instance will be created.
     *
     * @param target - The class / constructor function to register as transient.
     * @returns The same class, with a static `register` method that takes a container and returns the appropriate resolver.
     *
     * @example
     * On an existing class
     * ```ts
     * class Foo { }
     * DI.transient(Foo);
     * ```
     *
     * @example
     * Inline declaration
     *
     * ```ts
     * const Foo = DI.transient(class { });
     * // Foo is now strongly typed with register
     * Foo.register(container);
     * ```
     *
     * @public
     */
    transient<T extends Constructable<{}>>(target: T & Partial<RegisterSelf<T>>): T & RegisterSelf<T>;
    /**
     * Registers the `target` class as a singleton dependency; the class will only be created once. Each
     * consecutive time the dependency is resolved, the same instance will be returned.
     *
     * @param target - The class / constructor function to register as a singleton.
     * @returns The same class, with a static `register` method that takes a container and returns the appropriate resolver.
     * @example
     * On an existing class
     * ```ts
     * class Foo { }
     * DI.singleton(Foo);
     * ```
     *
     * @example
     * Inline declaration
     * ```ts
     * const Foo = DI.singleton(class { });
     * // Foo is now strongly typed with register
     * Foo.register(container);
     * ```
     *
     * @public
     */
    singleton<T_1 extends Constructable<{}>>(target: T_1 & Partial<RegisterSelf<T_1>>, options?: SingletonOptions): T_1 & RegisterSelf<T_1>;
}>;

/**
 * A Switch Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#dialog | ARIA dialog }.
 *
 * @slot - The default slot for the dialog content
 * @csspart positioning-region - A wrapping element used to center the dialog and position the modal overlay
 * @csspart overlay - The modal dialog overlay
 * @csspart control - The dialog element
 * @fires cancel - Fires a custom 'cancel' event when the modal overlay is clicked
 * @fires close - Fires a custom 'close' event when the dialog is hidden
 *
 * @public
 */
export declare class Dialog extends FoundationElement {
    /**
     * Indicates the element is modal. When modal, user mouse interaction will be limited to the contents of the element by a modal
     * overlay.  Clicks on the overlay will cause the dialog to emit a "dismiss" event.
     * @public
     * @defaultValue - false
     * @remarks
     * HTML Attribute: modal
     */
    modal: boolean;
    /**
     * The hidden state of the element.
     *
     * @public
     * @defaultValue - false
     * @remarks
     * HTML Attribute: hidden
     */
    hidden: boolean;
    /**
     * Indicates that the dialog should not trap focus.
     *
     * @public
     * @defaultValue - true
     * @remarks
     * HTML Attribute: no-focus-trap
     */
    noFocusTrap: boolean;
    private noFocusTrapChanged;
    /**
     * The id of the element describing the dialog.
     * @public
     * @remarks
     * HTML Attribute: aria-describedby
     */
    ariaDescribedby: string;
    /**
     * The id of the element labeling the dialog.
     * @public
     * @remarks
     * HTML Attribute: aria-labelledby
     */
    ariaLabelledby: string;
    /**
     * The label surfaced to assistive technologies.
     *
     * @public
     * @remarks
     * HTML Attribute: aria-label
     */
    ariaLabel: string;
    /* Excluded from this release type: dialog */
    /* Excluded from this release type: isTrappingFocus */
    /* Excluded from this release type: notifier */
    /* Excluded from this release type: dismiss */
    /**
     * The method to show the dialog.
     *
     * @public
     */
    show(): void;
    /**
     * The method to hide the dialog.
     *
     * @public
     */
    hide(): void;
    /* Excluded from this release type: connectedCallback */
    /* Excluded from this release type: disconnectedCallback */
    /* Excluded from this release type: handleChange */
    private handleDocumentKeydown;
    private handleDocumentFocus;
    private handleTabKeyDown;
    private getTabQueueBounds;
    /**
     * focus on first element of tab queue
     */
    private focusFirstElement;
    /**
     * we should only focus if focus has not already been brought to the dialog
     */
    private shouldForceFocus;
    /**
     * we should we be active trapping focus
     */
    private shouldTrapFocus;
    /* Excluded from this release type: updateTrapFocus */
    /* Excluded from this release type: reduceTabbableItems */
    /* Excluded from this release type: isFocusableFastElement */
    /* Excluded from this release type: hasTabbableShadow */
}

/**
 * The template for the {@link @microsoft/fast-foundation#Dialog} component.
 * @public
 */
export declare const dialogTemplate: FoundationElementTemplate<ViewTemplate<Dialog>>;

/**
 * The CSS value for disabled cursors.
 * @public
 * @deprecated - fast-foundation is removing styling utilities
 */
export declare const disabledCursor = "not-allowed";

/**
 * A Disclosure Custom HTML Element.
 * Based largely on the {@link https://w3c.github.io/aria-practices/#disclosure | disclosure element }.
 *
 * @slot start - Content which can be provided before the summary content
 * @slot end - Content which can be provided after the summary content
 * @slot title - The summary content
 * @slot - The default slot for the disclosure content
 * @fires toggle - fires a toggle event when the summary is toggled
 *
 * @public
 */
export declare class Disclosure extends FoundationElement {
    /**
     * Determines if the element should show the extra content or not.
     *
     * @public
     */
    expanded: boolean;
    /**
     * Invoker title
     *
     * @public
     */
    summary: string;
    /* Excluded from this release type: details */
    /* Excluded from this release type: connectedCallback */
    /* Excluded from this release type: disconnectedCallback */
    /**
     * Show extra content.
     */
    show(): void;
    /**
     * Hide extra content.
     */
    hide(): void;
    /**
     * Toggle the current(expanded/collapsed) state.
     */
    toggle(): void;
    /**
     * Register listener and set default disclosure mode
     */
    protected setup(): void;
    /**
     * Update the aria attr and fire `toggle` event
     */
    protected onToggle(): void;
}

/**
 * The template for the {@link @microsoft/fast-foundation#Disclosure} component.
 * @public
 */
export declare const disclosureTemplate: FoundationElementTemplate<ViewTemplate<Disclosure>>;

/**
 * Applies a CSS display property.
 * Also adds CSS rules to not display the element when the [hidden] attribute is applied to the element.
 * @param display - The CSS display property value
 * @public
 * @deprecated - fast-foundation is removing styling utilities
 */
export declare function display(displayValue: CSSDisplayPropertyValue): string;

/**
 * A Divider Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#separator | ARIA separator } or {@link https://www.w3.org/TR/wai-aria-1.1/#presentation | ARIA presentation}.
 *
 * @public
 */
export declare class Divider extends FoundationElement {
    /**
     * The role of the element.
     *
     * @public
     * @remarks
     * HTML Attribute: role
     */
    role: DividerRole;
    /**
     * The orientation of the divider.
     *
     * @public
     * @remarks
     * HTML Attribute: orientation
     */
    orientation: Orientation;
}

/**
 * Divider roles
 * @public
 */
export declare const DividerRole: {
    /**
     * The divider semantically separates content
     */
    readonly separator: "separator";
    /**
     * The divider has no semantic value and is for visual presentation only.
     */
    readonly presentation: "presentation";
};

/**
 * The types for Divider roles
 * @public
 */
export declare type DividerRole = typeof DividerRole[keyof typeof DividerRole];

/**
 * The template for the {@link @microsoft/fast-foundation#Divider} component.
 * @public
 */
export declare const dividerTemplate: FoundationElementTemplate<ViewTemplate<Divider>>;

declare type EagerOrLazyFoundationOption<T, K extends FoundationElementDefinition> = T | LazyFoundationOption<T, K>;

/**
 * The callback type that is invoked when an element can be defined by a design system.
 * @public
 */
export declare type ElementDefinitionCallback = (ctx: ElementDefinitionContext) => void;

/**
 * The design system context in which an element can be defined.
 * @public
 */
export declare interface ElementDefinitionContext {
    /**
     * The name that the element will be defined as.
     * @public
     */
    readonly name: string;
    /**
     * The type that will be defined.
     * @public
     */
    readonly type: Constructable<HTMLElement>;
    /**
     * The dependency injection container associated with the design system.
     * @public
     */
    readonly container: Container;
    /**
     * Indicates whether or not a platform define call will be made in order
     * to define the element.
     * @public
     */
    readonly willDefine: boolean;
    /**
     * The shadow root mode specified by the design system's configuration.
     * @public
     */
    readonly shadowRootMode: ShadowRootMode | undefined;
    /**
     * Defines the element.
     * @param definition - The definition for the element.
     * @public
     */
    defineElement(definition?: ContextualElementDefinition): void;
    /**
     * Defines a presentation for the element.
     * @param presentation - The presentation configuration.
     * @public
     */
    definePresentation(presentation: ComponentPresentation): void;
    /**
     * Returns the HTML element tag name that the type will be defined as.
     * @param type - The type to lookup.
     * @public
     */
    tagFor(type: Constructable): string;
}

/**
 * The element definition context interface. Designed to be used in `tryDefineElement`
 * @public
 */
export declare interface ElementDefinitionParams extends Pick<ElementDefinitionContext, "name" | "type"> {
    /**
     * FAST actual base class instance.
     * @public
     */
    readonly baseClass?: Constructable<HTMLElement>;
    /**
     * A callback to invoke if definition will happen.
     * @public
     */
    callback: ElementDefinitionCallback;
}

/**
 * Indicates what to do with an ambiguous (duplicate) element.
 * @public
 */
export declare const ElementDisambiguation: Readonly<{
    /**
     * Skip defining the element but still call the provided callback passed
     * to DesignSystemRegistrationContext.tryDefineElement
     */
    definitionCallbackOnly: null;
    /**
     * Ignore the duplicate element entirely.
     */
    ignoreDuplicate: symbol;
}>;

/**
 * The callback type that is invoked when two elements are trying to define themselves with
 * the same name.
 * @remarks
 * The callback should return either:
 * 1. A string to provide a new name used to disambiguate the element
 * 2. ElementDisambiguation.ignoreDuplicate to ignore the duplicate element entirely
 * 3. ElementDisambiguation.definitionCallbackOnly to skip defining the element but still
 * call the provided callback passed to DesignSystemRegistrationContext.tryDefineElement
 * @public
 */
export declare type ElementDisambiguationCallback = (nameAttempt: string, typeAttempt: Constructable, existingType: Constructable) => ElementDisambiguationResult;

/**
 * Represents the return values expected from an ElementDisambiguationCallback.
 * @public
 */
export declare type ElementDisambiguationResult = string | typeof ElementDisambiguation.ignoreDuplicate | typeof ElementDisambiguation.definitionCallbackOnly;

/**
 * Source:
 * https://html.spec.whatwg.org/multipage/custom-elements.html#elementinternals
 */
declare interface ElementInternals_2 {
    /**
     * Returns the form owner of internals target element.
     */
    readonly form: HTMLFormElement | null;
    /**
     * Returns a NodeList of all the label elements that internals target element is associated with.
     */
    readonly labels: NodeList;
    /**
     * Returns the error message that would be shown to the user if internals target element was to be checked for validity.
     */
    readonly validationMessage: string;
    /**
     * Returns the ValidityState object for internals target element.
     */
    readonly validity: ValidityState;
    /**
     * Returns true if internals target element will be validated when the form is submitted; false otherwise.
     */
    readonly willValidate: boolean;
    /**
     * Returns true if internals target element has no validity problems; false otherwise. Fires an invalid event at the element in the latter case.
     */
    checkValidity(): boolean;
    /**
     * Returns true if internals target element has no validity problems; otherwise,
     * returns false, fires an invalid event at the element, and (if the event isn't canceled) reports the problem to the user.
     */
    reportValidity(): boolean;
    /**
     * Sets both the state and submission value of internals target element to value.
     *
     * While "null" isn't enumerated as a argument type (here)[https://html.spec.whatwg.org/multipage/custom-elements.html#the-elementinternals-interface],
     * In practice it appears to remove the value from the form data on submission. Adding it as a valid type here
     * becuase that capability is required for checkbox and radio types
     */
    setFormValue(value: File | string | FormData | null, state?: File | string | FormData | null): void;
    /**
     * Marks internals target element as suffering from the constraints indicated by the flags argument,
     * and sets the element's validation message to message.
     * If anchor is specified, the user agent might use
     * it to indicate problems with the constraints of internals target
     * element when the form owner is validated interactively or reportValidity() is called.
     */
    setValidity(flags: ValidityStateFlags, message?: string, anchor?: HTMLElement): void;
}

declare let ElementInternals_2: {
    prototype: ElementInternals_2;
    new (): ElementInternals_2;
};

/**
 * End configuration options
 * @public
 */
export declare type EndOptions = {
    end?: string | SyntheticViewTemplate;
};

/**
 * The template for the end slot.
 * For use with {@link StartEnd}
 *
 * @public
 */
export declare const endSlotTemplate: (context: ElementDefinitionContext, definition: EndOptions) => ViewTemplate<StartEnd>;

/**
 * Used by the default Resolver to create instances of objects when needed.
 * @public
 */
export declare interface Factory<T extends Constructable = any> {
    /**
     * The concrete type this factory creates.
     */
    readonly Type: T;
    /**
     * Registers a transformer function to alter the object after instantiation but before
     * returning the final constructed instance.
     * @param transformer - The transformer function.
     */
    registerTransformer(transformer: Transformer_2<T>): void;
    /**
     * Constructs an instance of the factory's object.
     * @param container - The container the object is being constructor for.
     * @param dynamicDependencies - Dynamic dependencies supplied to the constructor.
     */
    construct(container: Container, dynamicDependencies?: Key[]): Resolved<T>;
}

/* Excluded from this release type: FactoryImpl */

/**
 * A Flipper Custom HTML Element.
 * Flippers are a form of button that implies directional content navigation, such as in a carousel.
 *
 * @slot next - The next flipper content
 * @slot previous - The previous flipper content
 * @csspart next - Wraps the next flipper content
 * @csspart previous - Wraps the previous flipper content
 * @fires click - Fires a custom 'click' event when Enter or Space is invoked via keyboard and the flipper is exposed to assistive technologies.
 *
 * @public
 */
export declare class Flipper extends FoundationElement {
    /**
     * The disabled state of the flipper.
     * @public
     * @remarks
     * HTML Attribute: disabled
     */
    disabled: boolean;
    /**
     * Indicates the flipper should be hidden from assistive technology. Because flippers are often supplementary navigation, they are often hidden from assistive technology.
     *
     * @public
     * @defaultValue - true
     * @remarks
     * HTML Attribute: aria-hidden
     */
    hiddenFromAT: boolean;
    /**
     * The direction that the flipper implies navigating.
     *
     * @public
     * @remarks
     * HTML Attribute: direction
     */
    direction: FlipperDirection;
    /**
     * Simulate a click event when the flipper has focus and the user hits enter or space keys
     * Blur focus if the user hits escape key
     * @param e - Keyboard event
     * @public
     */
    keyupHandler(e: Event & KeyboardEvent): void;
}

/**
 * The direction options for flipper.
 * @public
 */
export declare const FlipperDirection: {
    readonly next: "next";
    readonly previous: "previous";
};

/**
 * The types for the flipper direction options.
 * @public
 */
export declare type FlipperDirection = typeof FlipperDirection[keyof typeof FlipperDirection];

/**
 * Flipper configuration options
 * @public
 */
export declare type FlipperOptions = FoundationElementDefinition & {
    next?: string | SyntheticViewTemplate;
    previous?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#Flipper} component.
 * @public
 */
export declare const flipperTemplate: FoundationElementTemplate<ViewTemplate<Flipper>, FlipperOptions>;

/**
 * A region that always places itself below the anchor, has
 * a width to match the anchor, and is sized vertically by content
 *
 * @public
 */
export declare const FlyoutPosBottom: AnchoredRegionConfig;

/**
 * A region that always places itself below the anchor, has
 * a width to match the anchor, and is sized vertically by available space
 *
 * @public
 */
export declare const FlyoutPosBottomFill: AnchoredRegionConfig;

/**
 * A region that places itself above or below the anchor
 * based on available space, has a width to match the anchor,
 * and is sized vertically by content
 *
 * @public
 */
export declare const FlyoutPosTallest: AnchoredRegionConfig;

/**
 * A region that places itself above or below the anchor
 * based on available space, has a width to match the anchor,
 * and is sized vertically by available space
 *
 * @public
 */
export declare const FlyoutPosTallestFill: AnchoredRegionConfig;

/**
 * A region that always places itself above the anchor, has
 * a width to match the anchor, and is sized vertically by content
 *
 * @public
 */
export declare const FlyoutPosTop: AnchoredRegionConfig;

/**
 * A region that always places itself above the anchor, has
 * a width to match the anchor, and is sized vertically by available space
 *
 * @public
 */
export declare const FlyoutPosTopFill: AnchoredRegionConfig;

/**
 * This will always evaluate to 'focus-visible' due to modern browser support.
 * @public
 * @deprecated - fast-foundation is removing styling utilities
 */
export declare const focusVisible = "focus-visible";

/**
 * This can be used to construct a behavior to apply a forced-colors only stylesheet.
 * @public
 */
export declare const forcedColorsStylesheetBehavior: (styles: ElementStyles) => MatchMediaStyleSheetBehavior;

/**
 * Base function for providing Custom Element Form Association.
 *
 * @beta
 */
export declare function FormAssociated<T extends ConstructableFormAssociated>(BaseCtor: T): T;

/**
 * Base class for providing Custom Element Form Association.
 *
 * @beta
 */
export declare interface FormAssociated extends Omit<ElementInternals_2, "labels"> {
    dirtyValue: boolean;
    disabled: boolean;
    readonly elementInternals: ElementInternals_2 | null;
    readonly formAssociated: boolean;
    initialValue: string;
    readonly labels: ReadonlyArray<Node[]>;
    name: string;
    required: boolean;
    value: string;
    currentValue: string;
    attachProxy(): void;
    detachProxy(): void;
    disabledChanged?(previous: boolean, next: boolean): void;
    formDisabledCallback?(disabled: boolean): void;
    formResetCallback(): void;
    initialValueChanged?(previous: string, next: string): void;
    nameChanged?(previous: string, next: string): void;
    requiredChanged(prev: boolean, next: boolean): void;
    stopPropagation(e: Event): void;
    validate(): void;
    valueChanged(previous: string, next: string): void;
}

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(Button:class)} component.
 *
 * @beta
 */
declare class FormAssociatedButton extends FormAssociatedButton_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedButton_base: typeof _Button;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(Checkbox:class)} component.
 *
 * @beta
 */
declare class FormAssociatedCheckbox extends FormAssociatedCheckbox_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedCheckbox_base: typeof _Checkbox;

/**
 * A form-associated base class for the {@link (Combobox:class)} component.
 *
 * @beta
 */
declare class FormAssociatedCombobox extends FormAssociatedCombobox_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedCombobox_base: typeof _Combobox;

/**
 * Combined type to describe a Form-associated element.
 *
 * @beta
 */
export declare type FormAssociatedElement = FormAssociated & FASTElement & HTMLElement & FormAssociatedProxy;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(NumberField:class)} component.
 *
 * @beta
 */
declare class FormAssociatedNumberField extends FormAssociatedNumberField_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedNumberField_base: typeof _NumberField;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(Picker:class)} component.
 *
 * @beta
 */
declare class FormAssociatedPicker extends FormAssociatedPicker_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedPicker_base: typeof _Picker;

/**
 * Identifies a class as having a proxy element and optional submethods related
 * to the proxy element.
 *
 * @beta
 */
export declare interface FormAssociatedProxy {
    proxy: ProxyElement;
    disabledChanged?(previous: boolean, next: boolean): void;
    formDisabledCallback?(disabled: boolean): void;
    formResetCallback?(): void;
    initialValueChanged?(previous: string, next: string): void;
    valueChanged?(previous: string, next: string): void;
    nameChanged?(previous: string, next: string): void;
}

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(Radio:class)} component.
 *
 * @beta
 */
declare class FormAssociatedRadio extends FormAssociatedRadio_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedRadio_base: typeof _Radio;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(Search:class)} component.
 *
 * @beta
 */
declare class FormAssociatedSearch extends FormAssociatedSearch_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedSearch_base: typeof _Search;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(Select:class)} component.
 *
 * @beta
 */
declare class FormAssociatedSelect extends FormAssociatedSelect_base {
    proxy: HTMLSelectElement;
}

declare const FormAssociatedSelect_base: typeof _Select;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(Slider:class)} component.
 *
 * @beta
 */
declare class FormAssociatedSlider extends FormAssociatedSlider_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedSlider_base: typeof _Slider;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(Switch:class)} component.
 *
 * @beta
 */
declare class FormAssociatedSwitch extends FormAssociatedSwitch_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedSwitch_base: typeof _Switch;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(TextArea:class)} component.
 *
 * @beta
 */
declare class FormAssociatedTextArea extends FormAssociatedTextArea_base {
    proxy: HTMLTextAreaElement;
}

declare const FormAssociatedTextArea_base: typeof _TextArea;

/**
 * A form-associated base class for the {@link @microsoft/fast-foundation#(TextField:class)} component.
 *
 * @beta
 */
declare class FormAssociatedTextField extends FormAssociatedTextField_base {
    proxy: HTMLInputElement;
}

declare const FormAssociatedTextField_base: typeof _TextField;

/**
 * Defines a foundation element class that:
 * 1. Connects the element to its ComponentPresentation
 * 2. Allows resolving the element template from the instance or ComponentPresentation
 * 3. Allows resolving the element styles from the instance or ComponentPresentation
 *
 * @public
 */
export declare class FoundationElement extends FASTElement {
    private _presentation;
    /**
     * A property which resolves the ComponentPresentation instance
     * for the current component.
     * @public
     */
    protected get $presentation(): ComponentPresentation | null;
    /**
     * Sets the template of the element instance. When undefined,
     * the element will attempt to resolve the template from
     * the associated presentation or custom element definition.
     * @public
     */
    template: ElementViewTemplate | void | null;
    protected templateChanged(): void;
    /**
     * Sets the default styles for the element instance. When undefined,
     * the element will attempt to resolve default styles from
     * the associated presentation or custom element definition.
     * @public
     */
    styles: ElementStyles | void | null;
    protected stylesChanged(): void;
    /**
     * The connected callback for this FASTElement.
     * @remarks
     * This method is invoked by the platform whenever this FoundationElement
     * becomes connected to the document.
     * @public
     */
    connectedCallback(): void;
    /**
     * Defines an element registry function with a set of element definition defaults.
     * @param elementDefinition - The definition of the element to create the registry
     * function for.
     * @public
     */
    static compose<T extends FoundationElementDefinition = FoundationElementDefinition, K extends Constructable<FoundationElement> = Constructable<FoundationElement>>(this: K, elementDefinition: T): (overrideDefinition?: OverrideFoundationElementDefinition<T>) => FoundationElementRegistry<T, K>;
}

/**
 * An element definition used to define a FoundationElement when registered through the design
 * system registry.
 * @public
 */
export declare interface FoundationElementDefinition {
    /**
     * The non-prefixed name of the component.
     */
    baseName: string;
    /**
     * The actual FAST base class of the component if different from the class used to compose.
     */
    baseClass?: Constructable<HTMLElement>;
    /**
     * The template to render for the custom element.
     */
    readonly template?: EagerOrLazyFoundationOption<ElementViewTemplate, this>;
    /**
     * The styles to associate with the custom element.
     */
    readonly styles?: EagerOrLazyFoundationOption<ComposableStyles | ComposableStyles[], this>;
    /**
     * The custom attributes of the custom element.
     */
    readonly attributes?: EagerOrLazyFoundationOption<(AttributeConfiguration | string)[], this>;
    /**
     * Options controlling the creation of the custom element's shadow DOM.
     */
    readonly shadowOptions?: EagerOrLazyFoundationOption<Partial<ShadowRootInit> | null, this>;
    /**
     * Options controlling how the custom element is defined with the platform.
     */
    readonly elementOptions?: EagerOrLazyFoundationOption<ElementDefinitionOptions, this>;
}

/**
 * Registry capable of defining presentation properties for a DOM Container hierarchy.
 * @public
 */
export declare class FoundationElementRegistry<TDefinition extends FoundationElementDefinition, TType> implements Registry {
    readonly type: Constructable<FoundationElement>;
    private elementDefinition;
    private overrideDefinition;
    readonly definition: OverrideFoundationElementDefinition<TDefinition>;
    constructor(type: Constructable<FoundationElement>, elementDefinition: TDefinition, overrideDefinition: OverrideFoundationElementDefinition<TDefinition>);
    register(container: Container, context: DesignSystemRegistrationContext): void;
}

/**
 * A foundation element template function.
 * @public
 */
export declare type FoundationElementTemplate<T, K extends FoundationElementDefinition = FoundationElementDefinition> = LazyFoundationOption<T, K>;

/**
 * Enumerates the data grid auto generated header options
 * default option generates a non-sticky header row
 *
 * @public
 */
export declare const GenerateHeaderOptions: {
    readonly none: "none";
    readonly default: "default";
    readonly sticky: "sticky";
};

/**
 * The types for the data grid auto generated header options
 *
 * @public
 */
export declare type GenerateHeaderOptions = typeof GenerateHeaderOptions[keyof typeof GenerateHeaderOptions];

/**
 * a method to determine the current localization direction of the view
 * @param rootNode - the HTMLElement to begin the query from, usually "this" when used in a component controller
 * @public
 */
export declare const getDirection: (rootNode: HTMLElement) => Direction;

/**
 * A CSS fragment to set `display: none;` when the host is hidden using the [hidden] attribute.
 * @public
 * @deprecated - fast-foundation is removing styling utilities
 */
export declare const hidden = ":host([hidden]){display:none}";

/**
 * Defines the horizontal positioning options for an anchored region
 *
 * @public
 */
export declare type HorizontalPosition = "start" | "end" | "left" | "right" | "center" | "unset";

/**
 * A HorizontalScroll Custom HTML Element
 *
 * @slot start - Content which can be provided before the scroll area
 * @slot end - Content which can be provided after the scroll area
 * @csspart scroll-area - Wraps the entire scrollable region
 * @csspart scroll-view - The visible scroll area
 * @csspart content-container - The container for the content
 * @csspart scroll-prev - The previous flipper container
 * @csspart scroll-action-previous - The element wrapping the previous flipper
 * @csspart scroll-next - The next flipper container
 * @csspart scroll-action-next - The element wrapping the next flipper
 * @fires scrollstart - Fires a custom 'scrollstart' event when scrolling
 * @fires scrollend - Fires a custom 'scrollend' event when scrolling stops
 *
 * @public
 */
export declare class HorizontalScroll extends FoundationElement {
    /**
     * Reference to DOM element that scrolls the content
     * @public
     */
    scrollContainer: HTMLDivElement;
    /**
     * Reference to DOM element that holds the slotted content
     * @public
     */
    content: HTMLDivElement;
    /**
     * Reference to flipper to scroll to previous content
     * @public
     */
    previousFlipperContainer: HTMLDivElement;
    /**
     * Reference to flipper to scroll to the next content
     * @public
     */
    nextFlipperContainer: HTMLDivElement;
    /* Excluded from this release type: framesPerSecond */
    /* Excluded from this release type: frameTime */
    /* Excluded from this release type: resizeTimeout */
    /* Excluded from this release type: scrollTimeout */
    /* Excluded from this release type: updatingItems */
    /**
     * Speed of scroll in pixels per second
     * @public
     */
    speed: number;
    /**
     * The CSS time value for the scroll transition duration. Overrides the `speed` attribute.
     *
     * @remarks
     * When `duration` is set, the `speed` attribute has no effect.
     *
     * @public
     */
    duration: string;
    /**
     * Attribute used for easing, defaults to ease-in-out
     * @public
     */
    easing: ScrollEasing;
    /**
     * Attribute to hide flippers from assistive technology
     * @public
     */
    flippersHiddenFromAT: boolean;
    /* Excluded from this release type: scrolling */
    /* Excluded from this release type: scrollingChanged */
    /* Excluded from this release type: resizeDetector */
    /* Excluded from this release type: width */
    /* Excluded from this release type: scrollStops */
    /**
     * The default slotted items placed in the scrolling container.
     *
     * @public
     */
    scrollItems: HTMLElement[];
    /* Excluded from this release type: isRtl */
    /**
     * View: default | mobile
     * @public
     */
    view: HorizontalScrollView;
    connectedCallback(): void;
    disconnectedCallback(): void;
    /**
     * Updates scroll stops and flippers when scroll items change
     * @param previous - current scroll items
     * @param next - new updated scroll items
     * @public
     */
    scrollItemsChanged(previous: HTMLElement[], next: HTMLElement[]): void;
    /* Excluded from this release type: disconnectResizeDetector */
    /* Excluded from this release type: initializeResizeDetector */
    /* Excluded from this release type: updateScrollStops */
    /* Excluded from this release type: setStops */
    /**
     *
     */
    private fixScrollMisalign;
    /* Excluded from this release type: setFlippers */
    /**
     * Lets the user arrow left and right through the horizontal scroll
     * @param e - Keyboard event
     * @public
     */
    keyupHandler(e: Event & KeyboardEvent): void;
    /**
     * Scrolls items to the left
     * @public
     */
    scrollToPrevious(): void;
    /**
     * Scrolls items to the right
     * @public
     */
    scrollToNext(): void;
    /**
     * Handles scrolling with easing
     * @param position - starting position
     * @param newPosition - position to scroll to
     * @public
     */
    scrollToPosition(newPosition: number, position?: number): void;
    /**
     * Monitors resize event on the horizontal-scroll element
     * @public
     */
    resized(): void;
    /**
     * Monitors scrolled event on the content container
     * @public
     */
    scrolled(): void;
}

/**
 * Horizontal scroll configuration options
 * @public
 */
export declare type HorizontalScrollOptions = FoundationElementDefinition & StartEndOptions & {
    nextFlipper?: FoundationElementTemplate<SyntheticViewTemplate<any, HorizontalScroll>, HorizontalScrollOptions> | SyntheticViewTemplate | string;
    previousFlipper?: FoundationElementTemplate<SyntheticViewTemplate<any, HorizontalScroll>, HorizontalScrollOptions> | SyntheticViewTemplate | string;
};

/**
 * @public
 */
export declare const horizontalScrollTemplate: FoundationElementTemplate<ViewTemplate<HorizontalScroll>, HorizontalScrollOptions>;

/**
 * The views types for a horizontal-scroll {@link @microsoft/fast-foundation#(HorizontalScroll:class)}
 * @public
 */
export declare type HorizontalScrollView = "default" | "mobile";

/**
 * A decorator that tells the container not to try to inject a dependency.
 *
 * @public
 */
export declare function ignore(target: Injectable, property?: string | number, descriptor?: PropertyDescriptor | number): void;

/**
 * A decorator that specifies what to inject into its target.
 * @param dependencies - The dependencies to inject.
 * @returns The decorator to be applied to the target class.
 * @remarks
 * The decorator can be used to decorate a class, listing all of the classes dependencies.
 * Or it can be used to decorate a constructor paramter, indicating what to inject for that
 * parameter.
 * Or it can be used for a web component property, indicating what that property should resolve to.
 *
 * @public
 */
export declare const inject: (...dependencies: Key[]) => (target: any, key?: string | number, descriptor?: PropertyDescriptor | number) => void;

/**
 * A class that declares constructor injected dependencies through
 * a static "inject" field array of keys.
 * @public
 */
export declare type Injectable<T = {}> = Constructable<T> & {
    inject?: Key[];
};

/* Excluded from this release type: interactiveCalendarGridTemplate */

/**
 * Used to configure a dependency injection interface key.
 * @public
 */
export declare interface InterfaceConfiguration {
    /**
     * The friendly name for the interface. Useful for debugging.
     */
    friendlyName?: string;
    /**
     * When true, the dependency will be re-resolved when FASTElement connection changes.
     * If the resolved value changes due to connection change, a {@link @microsoft/fast-element#Observable | notification }
     * will be emitted for the property, with the previous and next values provided to any subscriber.
     */
    respectConnection?: boolean;
}

/**
 * A constant key that can be used to represent an interface to a registered dependency.
 * The key can be used in DI registrations but also doubles as a decorator for
 * resolving the associated, registered dependency.
 * @public
 */
export declare type InterfaceSymbol<K = any> = (target: any, property: string, index?: number) => void;

/**
 * Determines if the element is a {@link (ListboxOption:class)}
 *
 * @param element - the element to test.
 * @public
 */
export declare function isListboxOption(el: Element): el is ListboxOption;

/**
 * check if the item is a tree item
 * @public
 * @remarks
 * determines if element is an HTMLElement and if it has the role treeitem
 */
export declare function isTreeItemElement(el: Element): el is HTMLElement;

/**
 * A key that is used to register dependencies with a dependency injection container.
 * @public
 */
export declare type Key = PropertyKey | object | InterfaceSymbol | Constructable | Resolver;

/**
 * A decorator that lazily injects a dependency depending on whether the `Key` is present at the time of function call.
 *
 * @example
 * You need to make your argument a function that returns the type, for example
 * ```ts
 * class Foo {
 *   constructor( @lazy('random') public random: () => number )
 * }
 * const foo = container.get(Foo); // instanceof Foo
 * foo.random(); // throws
 * ```
 * would throw an exception because you haven't registered `'random'` before calling the method.
 * @example
 * This, would give you a new 'Math.random()' number each time.
 * ```ts
 * class Foo {
 *   constructor( @lazy('random') public random: () => random )
 * }
 * container.register(Registration.callback('random', Math.random ));
 * container.get(Foo).random(); // some random number
 * container.get(Foo).random(); // another random number
 * ```
 *
 * `@lazy` does not manage the lifecycle of the underlying key. If you want a singleton, you have to register as a
 * `singleton`, `transient` would also behave as you would expect, providing you a new instance each time.
 *
 * @param key - The key to lazily resolve.
 * see {@link DI.createInterface} on interactions with interfaces
 *
 * @public
 */
export declare const lazy: (key: any) => any;

declare type LazyFoundationOption<T, K extends FoundationElementDefinition> = (context: ElementDefinitionContext, definition: OverrideFoundationElementDefinition<K>) => T;

/**
 * This can be used to construct a behavior to apply a prefers color scheme: light only stylesheet.
 * @public
 */
export declare const lightModeStylesheetBehavior: (styles: ElementStyles) => MatchMediaStyleSheetBehavior;

/**
 * A Listbox Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#listbox | ARIA listbox }.
 *
 * @slot - The default slot for the listbox options
 *
 * @public
 */
export declare abstract class Listbox extends FoundationElement {
    /* Excluded from this release type: _options */
    /* Excluded from this release type: firstSelectedOption */
    /* Excluded from this release type: hasSelectableOptions */
    /**
     * The number of options.
     *
     * @public
     */
    get length(): number;
    /**
     * The list of options.
     *
     * @public
     */
    get options(): ListboxOption[];
    set options(value: ListboxOption[]);
    /* Excluded from this release type: typeAheadExpired */
    /* Excluded from this release type: typeAheadExpired */
    /**
     * The disabled state of the listbox.
     *
     * @public
     * @remarks
     * HTML Attribute: `disabled`
     */
    disabled: boolean;
    /**
     * The index of the selected option.
     *
     * @public
     */
    selectedIndex: number;
    /**
     * A collection of the selected options.
     *
     * @public
     */
    selectedOptions: ListboxOption[];
    /* Excluded from this release type: shouldSkipFocus */
    /**
     * A static filter to include only selectable options.
     *
     * @param n - element to filter
     * @public
     */
    static slottedOptionFilter: (n: HTMLElement) => boolean;
    /* Excluded from this release type: slottedOptions */
    /* Excluded from this release type: TYPE_AHEAD_TIMEOUT_MS */
    /* Excluded from this release type: typeaheadBuffer */
    /* Excluded from this release type: typeaheadExpired */
    /* Excluded from this release type: typeaheadTimeout */
    /* Excluded from this release type: clickHandler */
    /* Excluded from this release type: focusAndScrollOptionIntoView */
    /* Excluded from this release type: focusinHandler */
    /* Excluded from this release type: getTypeaheadMatches */
    /* Excluded from this release type: getSelectableIndex */
    /* Excluded from this release type: handleChange */
    /* Excluded from this release type: handleTypeAhead */
    /* Excluded from this release type: keydownHandler */
    /* Excluded from this release type: mousedownHandler */
    /* Excluded from this release type: multipleChanged */
    /* Excluded from this release type: selectedIndexChanged */
    /* Excluded from this release type: selectedOptionsChanged */
    /**
     * Moves focus to the first selectable option.
     *
     * @public
     */
    selectFirstOption(): void;
    /* Excluded from this release type: selectLastOption */
    /* Excluded from this release type: selectNextOption */
    /* Excluded from this release type: selectPreviousOption */
    /* Excluded from this release type: setDefaultSelectedOption */
    /**
     * Sets an option as selected and gives it focus.
     *
     * @public
     */
    protected setSelectedOptions(): void;
    /* Excluded from this release type: slottedOptionsChanged */
    /* Excluded from this release type: typeaheadBufferChanged */
}

/* Excluded declaration from this release type: Listbox */

/**
 * A Listbox Custom HTML Element.
 * Implements the {@link https://w3c.github.io/aria/#listbox | ARIA listbox }.
 *
 * @public
 */
export declare class ListboxElement extends Listbox {
    /* Excluded from this release type: activeIndex */
    /* Excluded from this release type: activeOption */
    /* Excluded from this release type: checkedOptions */
    /* Excluded from this release type: firstSelectedOptionIndex */
    /**
     * Indicates if the listbox is in multi-selection mode.
     *
     * @remarks
     * HTML Attribute: `multiple`
     *
     * @public
     */
    multiple: boolean;
    /* Excluded from this release type: rangeStartIndex */
    /**
     * The maximum number of options to display.
     *
     * @remarks
     * HTML Attribute: `size`.
     *
     * @public
     */
    size: number;
    /* Excluded from this release type: activeIndexChanged */
    /* Excluded from this release type: checkActiveIndex */
    /* Excluded from this release type: checkFirstOption */
    /* Excluded from this release type: checkLastOption */
    /* Excluded from this release type: connectedCallback */
    /* Excluded from this release type: disconnectedCallback */
    /* Excluded from this release type: checkNextOption */
    /* Excluded from this release type: checkPreviousOption */
    /* Excluded from this release type: clickHandler */
    /* Excluded from this release type: focusAndScrollOptionIntoView */
    /* Excluded from this release type: focusinHandler */
    /* Excluded from this release type: focusoutHandler */
    /* Excluded from this release type: keydownHandler */
    /* Excluded from this release type: mousedownHandler */
    /* Excluded from this release type: multipleChanged */
    /**
     * Sets an option as selected and gives it focus.
     *
     * @override
     * @public
     */
    protected setSelectedOptions(): void;
    /* Excluded from this release type: sizeChanged */
    /* Excluded from this release type: toggleSelectedForAllCheckedOptions */
    /* Excluded from this release type: typeaheadBufferChanged */
    /* Excluded from this release type: uncheckAllOptions */
}

/**
 * An Option Custom HTML Element.
 * Implements {@link https://www.w3.org/TR/wai-aria-1.1/#option | ARIA option }.
 *
 * @slot start - Content which can be provided before the listbox option content
 * @slot end - Content which can be provided after the listbox option content
 * @slot - The default slot for listbox option content
 * @csspart content - Wraps the listbox option content
 *
 * @public
 */
export declare class ListboxOption extends FoundationElement {
    /* Excluded from this release type: _value */
    /* Excluded from this release type: proxy */
    /**
     * The checked state is used when the parent listbox is in multiple selection mode.
     * To avoid accessibility conflicts, the checked state should not be present in
     * single selection mode.
     *
     * @public
     */
    checked?: boolean;
    /**
     * Updates the ariaChecked property when the checked property changes.
     *
     * @param prev - the previous checked value
     * @param next - the current checked value
     *
     * @public
     */
    protected checkedChanged(prev: boolean | unknown, next?: boolean): void;
    /**
     * The default slotted content.
     *
     * @public
     */
    content: Node[];
    /* Excluded from this release type: contentChanged */
    /**
     * The defaultSelected state of the option.
     * @public
     */
    defaultSelected: boolean;
    protected defaultSelectedChanged(): void;
    /* Excluded from this release type: dirtySelected */
    /**
     * The disabled state of the option.
     * @public
     * @remarks
     * HTML Attribute: disabled
     */
    disabled: boolean;
    protected disabledChanged(prev: boolean, next: boolean): void;
    /**
     * The selected attribute value. This sets the initial selected value.
     *
     * @public
     * @remarks
     * HTML Attribute: selected
     */
    selectedAttribute: boolean;
    protected selectedAttributeChanged(): void;
    /**
     * The checked state of the control.
     *
     * @public
     */
    selected: boolean;
    protected selectedChanged(): void;
    /**
     * Track whether the value has been changed from the initial value
     */
    dirtyValue: boolean;
    /**
     * The initial value of the option. This value sets the `value` property
     * only when the `value` property has not been explicitly set.
     *
     * @remarks
     * HTML Attribute: value
     */
    protected initialValue: string;
    initialValueChanged(previous: string, next: string): void;
    get label(): string;
    get text(): string;
    set value(next: string);
    get value(): string;
    get form(): HTMLFormElement | null;
    constructor(text?: string, value?: string, defaultSelected?: boolean, selected?: boolean);
}

/* Excluded declaration from this release type: ListboxOption */

/**
 * Listbox option configuration options
 * @public
 */
export declare type ListboxOptionOptions = FoundationElementDefinition & StartEndOptions;

/**
 * The template for the {@link @microsoft/fast-foundation#(ListboxOption:class)} component.
 * @public
 */
export declare const listboxOptionTemplate: FoundationElementTemplate<ViewTemplate<ListboxOption>, ListboxOptionOptions>;

/**
 * The template for the {@link @microsoft/fast-foundation#(Listbox:class)} component.
 * @public
 */
export declare const listboxTemplate: FoundationElementTemplate<ViewTemplate<ListboxElement>>;

/**
 * An abstract behavior to react to media queries. Implementations should implement
 * the `constructListener` method to perform some action based on media query changes.
 *
 * @public
 */
export declare abstract class MatchMediaBehavior implements Behavior {
    /**
     * The media query that the behavior operates on.
     */
    readonly query: MediaQueryList;
    /**
     *
     * @param query - The media query to operate from.
     */
    constructor(query: MediaQueryList);
    /**
     * Constructs a function that will be invoked with the MediaQueryList context
     * @param source - the element the behavior is acting on.
     */
    protected abstract constructListener(source: typeof FASTElement): MediaQueryListListener;
    /**
     * Binds the behavior to the element.
     * @param source - The element for which the behavior is bound.
     */
    bind(source: typeof FASTElement & HTMLElement): void;
    /**
     * Unbinds the behavior from the element.
     * @param source - The element for which the behavior is unbinding.
     */
    unbind(source: typeof FASTElement & HTMLElement): void;
    /**
     * The behavior needs to operate on element instances but elements might share a behavior instance.
     * To ensure proper attachment / detachment per instance, we construct a listener for
     * each bind invocation and cache the listeners by element reference.
     */
    private listenerCache;
}

/**
 * A behavior to add or remove a stylesheet from an element based on a media query. The behavior ensures that
 * styles are applied while the a query matches the environment and that styles are not applied if the query does
 * not match the environment.
 *
 * @public
 */
export declare class MatchMediaStyleSheetBehavior extends MatchMediaBehavior {
    /**
     * The media query that the behavior operates on.
     */
    readonly query: MediaQueryList;
    /**
     * The styles object to be managed by the behavior.
     */
    readonly styles: ElementStyles;
    /**
     * Constructs a {@link MatchMediaStyleSheetBehavior} instance.
     * @param query - The media query to operate from.
     * @param styles - The styles to coordinate with the query.
     */
    constructor(query: MediaQueryList, styles: ElementStyles);
    /**
     * Defines a function to construct {@link MatchMediaStyleSheetBehavior | MatchMediaStyleSheetBehaviors} for
     * a provided query.
     * @param query - The media query to operate from.
     *
     * @public
     * @example
     *
     * ```ts
     * import { css } from "@microsoft/fast-element";
     * import { MatchMediaStyleSheetBehavior } from "@microsoft/fast-foundation";
     *
     * const landscapeBehavior = MatchMediaStyleSheetBehavior.with(
     *   window.matchMedia("(orientation: landscape)")
     * );
     * const styles = css`
     *   :host {
     *     width: 200px;
     *     height: 400px;
     *   }
     * `
     * .withBehaviors(landscapeBehavior(css`
     *   :host {
     *     width: 400px;
     *     height: 200px;
     *   }
     * `))
     * ```
     */
    static with(query: MediaQueryList): (styles: ElementStyles) => MatchMediaStyleSheetBehavior;
    /**
     * Constructs a match-media listener for a provided element.
     * @param source - the element for which to attach or detach styles.
     */
    protected constructListener(source: typeof FASTElement): MediaQueryListListener;
    /* Excluded from this release type: unbind */
}

/**
 * An event listener fired by a {@link https://developer.mozilla.org/en-US/docs/Web/API/MediaQueryList | MediaQueryList }.
 * @public
 */
export declare type MediaQueryListListener = (this: MediaQueryList, ev?: MediaQueryListEvent) => void;

/**
 * A Menu Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#menu | ARIA menu }.
 *
 * @slot - The default slot for the menu items
 *
 * @public
 */
export declare class Menu extends FoundationElement {
    /* Excluded from this release type: items */
    protected itemsChanged(oldValue: HTMLElement[], newValue: HTMLElement[]): void;
    private menuItems;
    private expandedItem;
    /**
     * The index of the focusable element in the items array
     * defaults to -1
     */
    private focusIndex;
    private static focusableElementRoles;
    /* Excluded from this release type: connectedCallback */
    /* Excluded from this release type: disconnectedCallback */
    /* Excluded from this release type: isNestedMenu */
    /**
     * Focuses the first item in the menu.
     *
     * @public
     */
    focus(): void;
    /**
     * Collapses any expanded menu items.
     *
     * @public
     */
    collapseExpandedItem(): void;
    /* Excluded from this release type: handleMenuKeyDown */
    /* Excluded from this release type: handleFocusOut */
    private handleItemFocus;
    private handleExpandedChanged;
    private removeItemListeners;
    private setItems;
    /**
     * handle change from child element
     */
    private changeHandler;
    /**
     * get an array of valid DOM children
     */
    private domChildren;
    /**
     * check if the item is a menu item
     */
    private isMenuItemElement;
    /**
     * check if the item is focusable
     */
    private isFocusableElement;
    private setFocus;
}

/**
 * Defines the vertical positioning options for an anchored region
 *
 * @beta
 */
export declare type menuConfigs = "bottom" | "bottom-fill" | "tallest" | "tallest-fill" | "top" | "top-fill";

/**
 * A Switch Custom HTML Element.
 * Implements {@link https://www.w3.org/TR/wai-aria-1.1/#menuitem | ARIA menuitem }, {@link https://www.w3.org/TR/wai-aria-1.1/#menuitemcheckbox | ARIA menuitemcheckbox}, or {@link https://www.w3.org/TR/wai-aria-1.1/#menuitemradio | ARIA menuitemradio }.
 *
 * @slot checked-indicator - The checked indicator
 * @slot radio-indicator - The radio indicator
 * @slot start - Content which can be provided before the menu item content
 * @slot end - Content which can be provided after the menu item content
 * @slot - The default slot for menu item content
 * @slot expand-collapse-indicator - The expand/collapse indicator
 * @slot submenu - Used to nest menu's within menu items
 * @csspart input-container - The element representing the visual checked or radio indicator
 * @csspart checkbox - The element wrapping the `menuitemcheckbox` indicator
 * @csspart radio - The element wrapping the `menuitemradio` indicator
 * @csspart content - The element wrapping the menu item content
 * @csspart expand-collapse-glyph-container - The element wrapping the expand collapse element
 * @csspart expand-collapse - The expand/collapse element
 * @csspart submenu-region - The container for the submenu, used for positioning
 * @fires expanded-change - Fires a custom 'expanded-change' event when the expanded state changes
 * @fires change - Fires a custom 'change' event when a non-submenu item with a role of `menuitemcheckbox`, `menuitemradio`, or `menuitem` is invoked
 *
 * @public
 */
export declare class MenuItem extends FoundationElement {
    /**
     * The disabled state of the element.
     *
     * @public
     * @remarks
     * HTML Attribute: disabled
     */
    disabled: boolean;
    /**
     * The expanded state of the element.
     *
     * @public
     * @remarks
     * HTML Attribute: expanded
     */
    expanded: boolean;
    protected expandedChanged(oldValue: boolean): void;
    /* Excluded from this release type: startColumnCount */
    /**
     * The role of the element.
     *
     * @public
     * @remarks
     * HTML Attribute: role
     */
    role: MenuItemRole;
    /**
     * The checked value of the element.
     *
     * @public
     * @remarks
     * HTML Attribute: checked
     */
    checked: boolean;
    protected checkedChanged(oldValue: boolean, newValue: boolean): void;
    /* Excluded from this release type: submenuRegion */
    /* Excluded from this release type: hasSubmenu */
    /* Excluded from this release type: currentDirection */
    /* Excluded from this release type: submenu */
    private focusSubmenuOnLoad;
    private observer;
    /* Excluded from this release type: connectedCallback */
    /* Excluded from this release type: disconnectedCallback */
    /* Excluded from this release type: handleMenuItemKeyDown */
    /* Excluded from this release type: handleMenuItemClick */
    /* Excluded from this release type: submenuLoaded */
    /* Excluded from this release type: handleMouseOver */
    /* Excluded from this release type: handleMouseOut */
    /* Excluded from this release type: expandAndFocus */
    /* Excluded from this release type: invoke */
    /* Excluded from this release type: updateSubmenu */
    /**
     * get an array of valid DOM children
     */
    private domChildren;
}

/* Excluded declaration from this release type: MenuItem */

/**
 * Types of menu item column count.
 * @public
 */
export declare type MenuItemColumnCount = 0 | 1 | 2;

/**
 * Menu Item configuration options
 * @public
 */
export declare type MenuItemOptions = FoundationElementDefinition & StartEndOptions & {
    checkboxIndicator?: string | SyntheticViewTemplate;
    expandCollapseGlyph?: string | SyntheticViewTemplate;
    radioIndicator?: string | SyntheticViewTemplate;
};

/**
 * Menu items roles.
 * @public
 */
export declare const MenuItemRole: {
    /**
     * The menu item has a "menuitem" role
     */
    readonly menuitem: "menuitem";
    /**
     * The menu item has a "menuitemcheckbox" role
     */
    readonly menuitemcheckbox: "menuitemcheckbox";
    /**
     * The menu item has a "menuitemradio" role
     */
    readonly menuitemradio: "menuitemradio";
};

/**
 * The types for menu item roles
 * @public
 */
export declare type MenuItemRole = typeof MenuItemRole[keyof typeof MenuItemRole];

/**
 * Generates a template for the {@link @microsoft/fast-foundation#(MenuItem:class)} component using
 * the provided prefix.
 *
 * @public
 */
export declare const menuItemTemplate: FoundationElementTemplate<ViewTemplate<MenuItem>, MenuItemOptions>;

/**
 * The template for the {@link @microsoft/fast-foundation#Menu} component.
 * @public
 */
export declare const menuTemplate: FoundationElementTemplate<ViewTemplate<Menu>>;

/**
 * A type representing the different month formats
 * @public
 */
export declare type MonthFormat = "2-digit" | "long" | "narrow" | "numeric" | "short";

/**
 * Information about a month
 * @public
 */
export declare type MonthInfo = {
    month: number;
    year: number;
    length: number;
    start: number;
};

/**
 * A decorator that indicates that a new instance should be injected scoped to the
 * container that requested the instance.
 * @param key - The dependency key for the new instance.
 * @remarks
 * This creates a resolver with an instance strategy pointing to the new instance, effectively
 * making this a singleton, scoped to the container or DOM's subtree.
 *
 * @public
 */
export declare const newInstanceForScope: (key: any) => any;

/**
 * A decorator that indicates that a new instance should be injected.
 * @param key - The dependency key for the new instance.
 * @remarks
 * The instance is not internally cached with a resolver as newInstanceForScope does.
 *
 * @public
 */
export declare const newInstanceOf: (key: any) => any;

/* Excluded from this release type: noninteractiveCalendarTemplate */

/**
 * A Number Field Custom HTML Element.
 * Based largely on the {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input/number | <input type="number" /> element }.
 *
 * @slot start - Content which can be provided before the number field input
 * @slot end - Content which can be provided after the number field input
 * @slot - The default slot for the label
 * @slot step-up-glyph - The glyph for the step up control
 * @slot step-down-glyph - The glyph for the step down control
 * @csspart label - The label
 * @csspart root - The element wrapping the control, including start and end slots
 * @csspart control - The element representing the input
 * @csspart controls - The step up and step down controls
 * @csspart step-up - The step up control
 * @csspart step-down - The step down control
 * @fires input - Fires a custom 'input' event when the value has changed
 * @fires change - Fires a custom 'change' event when the value has changed
 *
 * @public
 */
export declare class NumberField extends FormAssociatedNumberField {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    /**
     * Indicates that this element should get focus after the page finishes loading. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#htmlattrdefautofocus | autofocus HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: autofocus
     */
    autofocus: boolean;
    /**
     * When true, spin buttons will not be rendered
     * @public
     * @remarks
     * HTML Attribute: autofocus
     */
    hideStep: boolean;
    /**
     * Sets the placeholder value of the element, generally used to provide a hint to the user.
     * @public
     * @remarks
     * HTML Attribute: placeholder
     * Using this attribute does is not a valid substitute for a labeling element.
     */
    placeholder: string;
    /**
     * Allows associating a {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/datalist | datalist} to the element by {@link https://developer.mozilla.org/en-US/docs/Web/API/Element/id}.
     * @public
     * @remarks
     * HTML Attribute: list
     */
    list: string;
    /**
     * The maximum number of characters a user can enter.
     * @public
     * @remarks
     * HTMLAttribute: maxlength
     */
    maxlength: number;
    /**
     * The minimum number of characters a user can enter.
     * @public
     * @remarks
     * HTMLAttribute: minlength
     */
    minlength: number;
    /**
     * Sets the width of the element to a specified number of characters.
     * @public
     * @remarks
     * HTMLAttribute: size
     */
    size: number;
    /**
     * Amount to increment or decrement the value by
     * @public
     * @remarks
     * HTMLAttribute: step
     */
    step: number;
    /**
     * The maximum the value can be
     * @public
     * @remarks
     * HTMLAttribute: max
     */
    max: number;
    /* Excluded from this release type: maxChanged */
    /**
     * The minimum the value can be
     * @public
     * @remarks
     * HTMLAttribute: min
     */
    min: number;
    /* Excluded from this release type: minChanged */
    /* Excluded from this release type: defaultSlottedNodes */
    /* Excluded from this release type: control */
    /* Excluded from this release type: isUserInput */
    /**
     * The value property, typed as a number.
     *
     * @public
     */
    get valueAsNumber(): number;
    set valueAsNumber(next: number);
    /* Excluded from this release type: valueChanged */
    /* Excluded from this release type: getValidValue */
    /**
     * Increments the value using the step value
     *
     * @public
     */
    stepUp(): void;
    /**
     * Decrements the value using the step value
     *
     * @public
     */
    stepDown(): void;
    /* Excluded from this release type: connectedCallback */
    /**
     * Selects all the text in the number field
     *
     * @public
     */
    protected select(): void;
    /* Excluded from this release type: handleTextInput */
    /* Excluded from this release type: handleChange */
    /* Excluded from this release type: handleKeyDown */
    /* Excluded from this release type: handleBlur */
}

/* Excluded declaration from this release type: NumberField */

declare class _NumberField extends FoundationElement {
}

declare interface _NumberField extends FormAssociated {
}

/**
 * Number Field configuration options
 * @public
 */
export declare type NumberFieldOptions = FoundationElementDefinition & StartEndOptions & {
    stepDownGlyph?: string | SyntheticViewTemplate;
    stepUpGlyph?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(NumberField:class)} component.
 * @public
 */
export declare const numberFieldTemplate: FoundationElementTemplate<ViewTemplate<NumberField>, NumberFieldOptions>;

/**
 * A decorator that allows you to optionally inject a dependency depending on whether the [[`Key`]] is present, for example:
 * @example
 * ```ts
 * class Foo {
 *   constructor( @inject('mystring') public str: string = 'somestring' )
 * }
 * container.get(Foo); // throws
 * ```
 * would fail
 *
 * @example
 * ```ts
 * class Foo {
 *   constructor( @optional('mystring') public str: string = 'somestring' )
 * }
 * container.get(Foo).str // somestring
 * ```
 * if you use it without a default it will inject `undefined`, so remember to mark your input type as
 * possibly `undefined`!
 *
 * @param key - The key to optionally resolve.
 * see {@link DI.createInterface} on interactions with interfaces
 *
 * @public
 */
export declare const optional: (key: any) => any;

/**
 * A set of properties which the component consumer can override during the element registration process.
 * @public
 */
export declare type OverrideFoundationElementDefinition<T extends FoundationElementDefinition> = Partial<Omit<T, "type" | "baseClass">> & {
    /**
     * An element prefix that overrides the design system configuration.
     * @public
     */
    prefix?: string;
};

/**
 * A function capable of locating the parent container based on a container's owner.
 * @remarks
 * A container owner is usually an HTMLElement instance.
 * @public
 */
export declare type ParentLocator = (owner: any) => Container | null;

/**
 * A Picker Custom HTML Element.  This is an early "alpha" version of the component.
 * Developers should expect the api to evolve, breaking changes are possible.
 *
 * @beta
 */
export declare class Picker extends FormAssociatedPicker {
    /**
     * Currently selected items. Comma delineated string ie. "apples,oranges".
     *
     * @remarks
     * HTML Attribute: selection
     */
    selection: string;
    protected selectionChanged(): void;
    /**
     * Currently available options. Comma delineated string ie. "apples,oranges".
     *
     * @remarks
     * HTML Attribute: options
     */
    options: string;
    protected optionsChanged(): void;
    /**
     * Whether the component should remove an option from the list when it is in the selection
     *
     * @remarks
     * HTML Attribute: filter-selected
     */
    filterSelected: boolean;
    /**
     * Whether the component should remove options based on the current query
     *
     * @remarks
     * HTML Attribute: filter-query
     */
    filterQuery: boolean;
    /**
     * The maximum number of items that can be selected.
     *
     * @remarks
     * HTML Attribute: max-selected
     */
    maxSelected: number | undefined;
    /**
     * The text to present to assistive technolgies when no suggestions are available.
     *
     * @remarks
     * HTML Attribute: no-suggestions-text
     */
    noSuggestionsText: string;
    /**
     *  The text to present to assistive technolgies when suggestions are available.
     *
     * @remarks
     * HTML Attribute: suggestions-available-text
     */
    suggestionsAvailableText: string;
    /**
     * The text to present to assistive technologies when suggestions are loading.
     *
     * @remarks
     * HTML Attribute: loading-text
     */
    loadingText: string;
    /**
     * Applied to the aria-label attribute of the input element
     *
     * @remarks
     * HTML Attribute: label
     */
    label: string;
    /**
     * Applied to the aria-labelledby attribute of the input element
     *
     * @remarks
     * HTML Attribute: labelledby
     */
    labelledBy: string;
    /**
     * Applied to the placeholder attribute of the input element
     *
     * @remarks
     * HTML Attribute: placholder
     */
    placeholder: string;
    /**
     * Controls menu placement
     *
     * @remarks
     * HTML Attribute: menu-placement
     */
    menuPlacement: menuConfigs;
    protected menuPlacementChanged(): void;
    /**
     * Whether to display a loading state if the menu is opened.
     *
     */
    showLoading: boolean;
    protected showLoadingChanged(): void;
    /**
     * Template used to generate selected items.
     * This is used in a repeat directive.
     *
     */
    listItemTemplate: ViewTemplate;
    protected listItemTemplateChanged(): void;
    /**
     * Default template to use for selected items (usually specified in the component template).
     * This is used in a repeat directive.
     *
     */
    defaultListItemTemplate?: ViewTemplate;
    protected defaultListItemTemplateChanged(): void;
    /* Excluded from this release type: activeListItemTemplate */
    /**
     * Template to use for available options.
     * This is used in a repeat directive.
     *
     */
    menuOptionTemplate: ViewTemplate;
    protected menuOptionTemplateChanged(): void;
    /**
     * Default template to use for available options (usually specified in the template).
     * This is used in a repeat directive.
     *
     */
    defaultMenuOptionTemplate?: ViewTemplate;
    protected defaultMenuOptionTemplateChanged(): void;
    /* Excluded from this release type: activeMenuOptionTemplate */
    /**
     *  Template to use for the contents of a selected list item
     *
     */
    listItemContentsTemplate: ViewTemplate;
    /**
     *  Template to use for the contents of menu options
     *
     */
    menuOptionContentsTemplate: ViewTemplate;
    /**
     *  Current list of options in array form
     *
     */
    optionsList: string[];
    private optionsListChanged;
    /**
     * The text value currently in the input field
     *
     */
    query: string;
    protected queryChanged(): void;
    /* Excluded from this release type: filteredOptionsList */
    protected filteredOptionsListChanged(): void;
    /* Excluded from this release type: flyoutOpen */
    protected flyoutOpenChanged(): void;
    /* Excluded from this release type: menuId */
    /* Excluded from this release type: selectedListTag */
    /* Excluded from this release type: menuTag */
    /* Excluded from this release type: menuFocusIndex */
    /* Excluded from this release type: menuFocusOptionId */
    /* Excluded from this release type: showNoOptions */
    private showNoOptionsChanged;
    /* Excluded from this release type: menuConfig */
    /**
     *  Reference to the placeholder element for the repeat directive
     *
     */
    itemsPlaceholderElement: Node;
    /* Excluded from this release type: inputElement */
    /* Excluded from this release type: listElement */
    /* Excluded from this release type: menuElement */
    /* Excluded from this release type: region */
    /* Excluded from this release type: selectedItems */
    private itemsRepeatBehavior;
    private optionsRepeatBehavior;
    private optionsPlaceholder;
    private inputElementView;
    /* Excluded from this release type: connectedCallback */
    disconnectedCallback(): void;
    /**
     * Move focus to the input element
     * @public
     */
    focus(): void;
    /**
     * Initialize the component.  This is delayed a frame to ensure children are connected as well.
     */
    private initialize;
    /**
     * Toggles the menu flyout
     */
    private toggleFlyout;
    /**
     * Handle input event from input element
     */
    private handleTextInput;
    /**
     * Handle click event from input element
     */
    private handleInputClick;
    /**
     * Handle the menu options updated event from the child menu
     */
    private handleMenuOptionsUpdated;
    /**
     * Handle key down events.
     */
    handleKeyDown(e: KeyboardEvent): boolean;
    /**
     * Handle focus in events.
     */
    handleFocusIn(e: FocusEvent): boolean;
    /**
     * Handle focus out events.
     */
    handleFocusOut(e: FocusEvent): boolean;
    /**
     * The list of selected items has changed
     */
    handleSelectionChange(): void;
    /**
     * Anchored region is loaded, menu and options exist in the DOM.
     */
    handleRegionLoaded(e: Event): void;
    /**
     * Sets properties on the anchored region once it is instanciated.
     */
    private setRegionProps;
    /**
     * Checks if the maximum number of items has been chosen and updates the ui.
     */
    private checkMaxItems;
    /**
     * A list item has been invoked.
     */
    handleItemInvoke(e: Event): boolean;
    /**
     * A menu option has been invoked.
     */
    handleOptionInvoke(e: Event): boolean;
    /**
     * Increments the focused list item by the specified amount
     */
    private incrementFocusedItem;
    /**
     * Disables the menu. Note that the menu can be open, just doens't have any valid options on display.
     */
    private disableMenu;
    /**
     * Sets the currently focused menu option by index
     */
    private setFocusedOption;
    /**
     * Updates the template used for the list item repeat behavior
     */
    private updateListItemTemplate;
    /**
     * Updates the template used for the menu option repeat behavior
     */
    private updateOptionTemplate;
    /**
     * Updates the filtered options array
     */
    private updateFilteredOptions;
    /**
     * Updates the menu configuration
     */
    private updateMenuConfig;
    /**
     * matches menu placement values with the associated menu config
     */
    private configLookup;
}

declare class _Picker extends FoundationElement {
}

declare interface _Picker extends FormAssociated {
}

/**
 * A List Picker Menu Custom HTML Element.
 *
 * @beta
 */
export declare class PickerList extends FoundationElement {
}

/**
 * A picker list item Custom HTML Element.
 *
 * @beta
 */
export declare class PickerListItem extends FoundationElement {
    /**
     * The underlying string value of the item
     *
     * @remarks
     * HTML Attribute: value
     */
    value: string;
    /**
     *  The template used to render the contents of the list item
     *
     */
    contentsTemplate: ViewTemplate;
    protected contentsTemplateChanged(): void;
    private customView;
    /* Excluded from this release type: connectedCallback */
    /* Excluded from this release type: disconnectedCallback */
    handleKeyDown(e: KeyboardEvent): boolean;
    handleClick(e: MouseEvent): boolean;
    private handleInvoke;
    private updateView;
    private disconnectView;
}

/**
 *
 * @public
 */
export declare const pickerListItemTemplate: FoundationElementTemplate<ViewTemplate<PickerListItem>>;

/**
 *
 * @public
 */
export declare const pickerListTemplate: FoundationElementTemplate<ViewTemplate<PickerList>>;

/**
 * A List Picker Menu Custom HTML Element.
 *
 * @beta
 */
export declare class PickerMenu extends FoundationElement {
    /* Excluded from this release type: menuElements */
    menuElementsChanged(): void;
    /* Excluded from this release type: headerElements */
    headerElementsChanged(): void;
    /* Excluded from this release type: footerElements */
    footerElementsChanged(): void;
    /**
     * Text to display to assistive technology when
     * suggestions are available
     *
     */
    suggestionsAvailableText: string;
    /* Excluded from this release type: optionElements */
    private updateOptions;
    private addSlottedListItems;
}

/**
 * A picker list item Custom HTML Element.
 *
 * @beta
 */
export declare class PickerMenuOption extends FoundationElement {
    /**
     * The underlying string value of the item
     *
     * @remarks
     * HTML Attribute: value
     */
    value: string;
    /**
     *  The template used to render the contents of the list item
     */
    contentsTemplate: ViewTemplate;
    protected contentsTemplateChanged(): void;
    private customView;
    /* Excluded from this release type: connectedCallback */
    /* Excluded from this release type: disconnectedCallback */
    handleClick(e: MouseEvent): boolean;
    private handleInvoked;
    private updateView;
    private disconnectView;
}

/**
 *
 * @public
 */
export declare const pickerMenuOptionTemplate: FoundationElementTemplate<ViewTemplate<PickerMenuOption>>;

/**
 * The template for the List Picker component.
 * @public
 */
export declare const pickerMenuTemplate: FoundationElementTemplate<ViewTemplate<PickerMenu>>;

/**
 * The template for the List Picker component.
 * @public
 */
export declare const pickerTemplate: FoundationElementTemplate<ViewTemplate<Picker>>;

/**
 * Progress configuration options
 * @public
 */
export declare type ProgressOptions = FoundationElementDefinition & {
    indeterminateIndicator1?: string | SyntheticViewTemplate;
    indeterminateIndicator2?: string | SyntheticViewTemplate;
};

/**
 * ProgressRing configuration options
 * @public
 */
export declare type ProgressRingOptions = FoundationElementDefinition & {
    indeterminateIndicator?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#BaseProgress} component.
 * @public
 */
export declare const progressRingTemplate: FoundationElementTemplate<ViewTemplate<BaseProgress>, ProgressRingOptions>;

/**
 * The template for the {@link @microsoft/fast-foundation#BaseProgress} component.
 * @public
 */
export declare const progressTemplate: FoundationElementTemplate<ViewTemplate<BaseProgress>, ProgressOptions>;

/**
 * A behavior to add or remove a stylesheet from an element based on a property. The behavior ensures that
 * styles are applied while the property matches and that styles are not applied if the property does
 * not match.
 *
 * @public
 */
export declare class PropertyStyleSheetBehavior implements Behavior {
    private propertyName;
    private value;
    private styles;
    /**
     * Constructs a {@link PropertyStyleSheetBehavior} instance.
     * @param propertyName - The property name to operate from.
     * @param value - The property value to operate from.
     * @param styles - The styles to coordinate with the property.
     */
    constructor(propertyName: string, value: any, styles: ElementStyles);
    /**
     * Binds the behavior to the element.
     * @param elementInstance - The element for which the property is applied.
     */
    bind(elementInstance: FASTElement): void;
    /**
     * Unbinds the behavior from the element.
     * @param source - The element for which the behavior is unbinding.
     */
    unbind(source: typeof FASTElement & HTMLElement): void;
    /* Excluded from this release type: handleChange */
}

/**
 * Avaiable types for the `proxy` property.
 * @beta
 */
export declare type ProxyElement = HTMLSelectElement | HTMLTextAreaElement | HTMLInputElement;

/**
 * A Radio Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#radio | ARIA radio }.
 *
 * @slot checked-indicator - The checked indicator
 * @slot - The default slot for the label
 * @csspart control - The element representing the visual radio control
 * @csspart label - The label
 * @fires change - Emits a custom change event when the checked state changes
 *
 * @public
 */
export declare class Radio extends FormAssociatedRadio implements RadioControl {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    protected readOnlyChanged(): void;
    /**
     * The name of the radio. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#htmlattrdefname | name attribute} for more info.
     */
    name: string;
    /* Excluded from this release type: initialValue */
    /* Excluded from this release type: defaultSlottedNodes */
    /* Excluded from this release type: defaultCheckedChanged */
    constructor();
    /* Excluded from this release type: connectedCallback */
    private isInsideRadioGroup;
    /**
     * Handles key presses on the radio.
     * @beta
     */
    keypressHandler(e: KeyboardEvent): boolean | void;
    /**
     * Handles clicks on the radio.
     * @beta
     */
    clickHandler(e: MouseEvent): boolean | void;
}

declare class _Radio extends FoundationElement {
}

declare interface _Radio extends CheckableFormAssociated {
}

/**
 * A structure representing a {@link @microsoft/fast-foundation#(Radio:class)} element
 * @public
 */
export declare type RadioControl = Pick<HTMLInputElement, "checked" | "disabled" | "readOnly" | "focus" | "setAttribute" | "getAttribute">;

/**
 * An Radio Group Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#radiogroup | ARIA radiogroup }.
 *
 * @slot label - The slot for the label
 * @slot - The default slot for radio buttons
 * @csspart positioning-region - The positioning region for laying out the radios
 * @fires change - Fires a custom 'change' event when the value changes
 *
 * @public
 */
export declare class RadioGroup extends FoundationElement {
    /**
     * When true, the child radios will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    protected readOnlyChanged(): void;
    /**
     * Disables the radio group and child radios.
     *
     * @public
     * @remarks
     * HTML Attribute: disabled
     */
    disabled: boolean;
    protected disabledChanged(): void;
    /**
     * The name of the radio group. Setting this value will set the name value
     * for all child radio elements.
     *
     * @public
     * @remarks
     * HTML Attribute: name
     */
    name: string;
    protected nameChanged(): void;
    /**
     * The value of the checked radio
     *
     * @public
     * @remarks
     * HTML Attribute: value
     */
    value: string;
    protected valueChanged(): void;
    /**
     * The orientation of the group
     *
     * @public
     * @remarks
     * HTML Attribute: orientation
     */
    orientation: Orientation | "horizontal" | "vertical";
    childItems: HTMLElement[];
    /* Excluded from this release type: slottedRadioButtons */
    protected slottedRadioButtonsChanged(oldValue: unknown, newValue: HTMLElement[]): void;
    private selectedRadio;
    private focusedRadio;
    private direction;
    private get parentToolbar();
    private get isInsideToolbar();
    private get isInsideFoundationToolbar();
    /* Excluded from this release type: connectedCallback */
    disconnectedCallback(): void;
    private setupRadioButtons;
    private radioChangeHandler;
    private moveToRadioByIndex;
    private moveRightOffGroup;
    private moveLeftOffGroup;
    /* Excluded from this release type: focusOutHandler */
    /* Excluded from this release type: clickHandler */
    private shouldMoveOffGroupToTheRight;
    private shouldMoveOffGroupToTheLeft;
    private checkFocusedRadio;
    private moveRight;
    private moveLeft;
    /* Excluded from this release type: keydownHandler */
}

/**
 * The template for the {@link @microsoft/fast-foundation#RadioGroup} component.
 * @public
 */
export declare const radioGroupTemplate: FoundationElementTemplate<ViewTemplate<RadioGroup>>;

/**
 * Radio configuration options
 * @public
 */
export declare type RadioOptions = FoundationElementDefinition & {
    checkedIndicator?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(Radio:class)} component.
 * @public
 */
export declare const radioTemplate: FoundationElementTemplate<ViewTemplate<Radio>, RadioOptions>;

/**
 * Reflects attributes from the host element to the target element of the directive.
 * @param attributes - The attributes to reflect
 *
 * @beta
 * @example
 * ```ts
 * const template = html`
 *     <button
 *         ${reflectAttributes("aria-label", "aria-describedby")}
 *     >
 *          hello world
 *     </button
 * `
 * ```
 */
export declare function reflectAttributes<T = any>(...attributes: string[]): CaptureType<T>;

/**
 * Represents an object that can register itself.
 * @public
 */
export declare type RegisterSelf<T extends Constructable> = {
    /**
     * Registers itself with the container.
     * @param container - The container to register with.
     */
    register(container: Container): Resolver<InstanceType<T>>;
    /**
     * Indicates whether during auto registration the object should be
     * registered in the requesting container rather than the handling container.
     */
    registerInRequestor: boolean;
};

/**
 * Implemented by objects that wish to register dependencies in the container
 * by creating resolvers.
 * @public
 */
export declare interface Registration<K = any> {
    /**
     * Creates a resolver for a desired dependency.
     * @param container - The container to register the dependency within.
     * @param key - The key to register dependency under, if overridden.
     */
    register(container: Container): Resolver<K>;
}

/**
 * You can use the resulting Registration of any of the factory methods
 * to register with the container.
 *
 * @example
 * ```
 * class Foo {}
 * const container = DI.createContainer();
 * container.register(Registration.instance(Foo, new Foo()));
 * container.get(Foo);
 * ```
 *
 * @public
 */
export declare const Registration: Readonly<{
    /**
     * Allows you to pass an instance.
     * Every time you request this {@link Key} you will get this instance back.
     *
     * @example
     * ```
     * Registration.instance(Foo, new Foo()));
     * ```
     *
     * @param key - The key to register the instance under.
     * @param value - The instance to return when the key is requested.
     */
    instance<T>(key: Key, value: T): Registration<T>;
    /**
     * Creates an instance from the class.
     * Every time you request this {@link Key} you will get the same one back.
     *
     * @example
     * ```
     * Registration.singleton(Foo, Foo);
     * ```
     *
     * @param key - The key to register the singleton under.
     * @param value - The class to instantiate as a singleton when first requested.
     */
    singleton<T_1 extends Constructable<{}>>(key: Key, value: T_1): Registration<InstanceType<T_1>>;
    /**
     * Creates an instance from a class.
     * Every time you request this {@link Key} you will get a new instance.
     *
     * @example
     * ```
     * Registration.instance(Foo, Foo);
     * ```
     *
     * @param key - The key to register the instance type under.
     * @param value - The class to instantiate each time the key is requested.
     */
    transient<T_2 extends Constructable<{}>>(key: Key, value: T_2): Registration<InstanceType<T_2>>;
    /**
     * Delegates to a callback function to provide the dependency.
     * Every time you request this {@link Key} the callback will be invoked to provide
     * the dependency.
     *
     * @example
     * ```
     * Registration.callback(Foo, () => new Foo());
     * Registration.callback(Bar, (c: Container) => new Bar(c.get(Foo)));
     * ```
     *
     * @param key - The key to register the callback for.
     * @param callback - The function that is expected to return the dependency.
     */
    callback<T_3>(key: Key, callback: ResolveCallback<T_3>): Registration<Resolved<T_3>>;
    /**
     * Delegates to a callback function to provide the dependency and then caches the
     * dependency for future requests.
     *
     * @example
     * ```
     * Registration.cachedCallback(Foo, () => new Foo());
     * Registration.cachedCallback(Bar, (c: Container) => new Bar(c.get(Foo)));
     * ```
     *
     * @param key - The key to register the callback for.
     * @param callback - The function that is expected to return the dependency.
     * @remarks
     * If you pass the same Registration to another container, the same cached value will be used.
     * Should all references to the resolver returned be removed, the cache will expire.
     */
    cachedCallback<T_4>(key: Key, callback: ResolveCallback<T_4>): Registration<Resolved<T_4>>;
    /**
     * Creates an alternate {@link Key} to retrieve an instance by.
     *
     * @example
     * ```
     * Register.singleton(Foo, Foo)
     * Register.aliasTo(Foo, MyFoos);
     *
     * container.getAll(MyFoos) // contains an instance of Foo
     * ```
     *
     * @param originalKey - The original key that has been registered.
     * @param aliasKey - The alias to the original key.
     */
    aliasTo<T_5>(originalKey: T_5, aliasKey: Key): Registration<Resolved<T_5>>;
}>;

/**
 * Implemented by objects that which to register dependencies in a container.
 * @public
 */
export declare interface Registry {
    /**
     * Registers dependencies in the specified container.
     * @param container - The container to register dependencies in.
     * @param params - Parameters that affect the registration process.
     * @remarks
     * If this registry doubles as a Registration, it should return a Resolver
     * for the registered dependency.
     */
    register(container: Container, ...params: unknown[]): void | Resolver;
}

/**
 * Represents a custom callback for resolving a request from the container.
 * The handler is the container that is invoking the callback. The requestor
 * is the original container that made the request. The handler and the requestor
 * may not be the same if the request has bubbled up to a parent container in the DI hierarchy.
 * The resolver is the instance of the resolver that stores the callback. This is provided in case
 * the callback needs a place or key against which to store state across resolutions.
 * @public
 */
export declare type ResolveCallback<T = any> = (handler: Container, requestor: Container, resolver: Resolver<T>) => T;

/**
 * Represents something resolved from a service locator.
 * @public
 */
export declare type Resolved<K> = K extends InterfaceSymbol<infer T> ? T : K extends Constructable ? InstanceType<K> : K extends ResolverLike<any, infer T1> ? T1 extends Constructable ? InstanceType<T1> : T1 : K;

/**
 * Internally, the DI system maps "keys" to "resolvers". A resolver controls
 * how a dependency is resolved. Resolvers for transient, singleton, etc. are
 * provided out of the box, but you can also implement Resolver yourself and supply
 * custom logic for resolution.
 * @public
 */
export declare interface Resolver<K = any> extends ResolverLike<Container, K> {
}

/**
 * A utility class used that constructs and registers resolvers for a dependency
 * injection container. Supports a standard set of object lifetimes.
 * @public
 */
export declare class ResolverBuilder<K> {
    private container;
    private key;
    /**
     *
     * @param container - The container to create resolvers for.
     * @param key - The key to register resolvers under.
     */
    constructor(container: Container, key: Key);
    /**
     * Creates a resolver for an existing object instance.
     * @param value - The instance to resolve.
     * @returns The resolver.
     */
    instance(value: K): Resolver<K>;
    /**
     * Creates a resolver that enforces a singleton lifetime.
     * @param value - The type to create and cache the singleton for.
     * @returns The resolver.
     */
    singleton(value: Constructable): Resolver<K>;
    /**
     * Creates a resolver that creates a new instance for every dependency request.
     * @param value - The type to create instances of.
     * @returns - The resolver.
     */
    transient(value: Constructable): Resolver<K>;
    /**
     * Creates a resolver that invokes a callback function for every dependency resolution
     * request, allowing custom logic to return the dependency.
     * @param value - The callback to call during resolution.
     * @returns The resolver.
     */
    callback(value: ResolveCallback<K>): Resolver<K>;
    /**
     * Creates a resolver that invokes a callback function the first time that a dependency
     * resolution is requested. The returned value is then cached and provided for all
     * subsequent requests.
     * @param value - The callback to call during the first resolution.
     * @returns The resolver.
     */
    cachedCallback(value: ResolveCallback<K>): Resolver<K>;
    /**
     * Aliases the current key to a different key.
     * @param destinationKey - The key to point the alias to.
     * @returns The resolver.
     */
    aliasTo(destinationKey: Key): Resolver<K>;
    private registerResolver;
}

/* Excluded from this release type: ResolverImpl */

declare interface ResolverLike<C, K = any> {
    readonly $isResolver: true;
    resolve(handler: C, requestor: C): Resolved<K>;
    getFactory?(container: C): (K extends Constructable ? Factory<K> : never) | null;
}

/* Excluded from this release type: ResolverStrategy */

/* Excluded from this release type: roleForMenuItem */

/**
 * The easing types available for the horizontal-scroll {@link @microsoft/fast-foundation#(HorizontalScroll:class)}
 * @public
 */
export declare type ScrollEasing = "linear" | "ease-in" | "ease-out" | "ease-in-out" | string;

/**
 * A Search Custom HTML Element.
 * Based largely on the {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input/search | <input type="search" /> element }.
 *
 * @slot start - Content which can be provided before the search input
 * @slot end - Content which can be provided after the search clear button
 * @slot - The default slot for the label
 * @slot clear-button - The clear button
 * @slot clear-glyph - The clear glyph
 * @csspart label - The label
 * @csspart root - The element wrapping the control, including start and end slots
 * @csspart control - The element representing the input
 * @csspart clear-button - The button to clear the input
 *
 * @public
 */
export declare class Search extends FormAssociatedSearch {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    protected readOnlyChanged(): void;
    /**
     * Indicates that this element should get focus after the page finishes loading. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#htmlattrdefautofocus | autofocus HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: autofocus
     */
    autofocus: boolean;
    protected autofocusChanged(): void;
    /**
     * Sets the placeholder value of the element, generally used to provide a hint to the user.
     * @public
     * @remarks
     * HTML Attribute: placeholder
     * Using this attribute does is not a valid substitute for a labeling element.
     */
    placeholder: string;
    protected placeholderChanged(): void;
    /**
     * Allows associating a {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/datalist | datalist} to the element by {@link https://developer.mozilla.org/en-US/docs/Web/API/Element/id}.
     * @public
     * @remarks
     * HTML Attribute: list
     */
    list: string;
    protected listChanged(): void;
    /**
     * The maximum number of characters a user can enter.
     * @public
     * @remarks
     * HTMLAttribute: maxlength
     */
    maxlength: number;
    protected maxlengthChanged(): void;
    /**
     * The minimum number of characters a user can enter.
     * @public
     * @remarks
     * HTMLAttribute: minlength
     */
    minlength: number;
    protected minlengthChanged(): void;
    /**
     * A regular expression that the value must match to pass validation.
     * @public
     * @remarks
     * HTMLAttribute: pattern
     */
    pattern: string;
    protected patternChanged(): void;
    /**
     * Sets the width of the element to a specified number of characters.
     * @public
     * @remarks
     * HTMLAttribute: size
     */
    size: number;
    protected sizeChanged(): void;
    /**
     * Controls whether or not to enable spell checking for the input field, or if the default spell checking configuration should be used.
     * @public
     * @remarks
     * HTMLAttribute: size
     */
    spellcheck: boolean;
    protected spellcheckChanged(): void;
    /* Excluded from this release type: defaultSlottedNodes */
    /* Excluded from this release type: root */
    /* Excluded from this release type: control */
    /* Excluded from this release type: connectedCallback */
    /* Excluded from this release type: handleTextInput */
    /**
     * Handles the control's clear value event
     * @public
     */
    handleClearInput(): void;
    /* Excluded from this release type: handleChange */
}

/* Excluded declaration from this release type: Search */

declare class _Search extends FoundationElement {
}

declare interface _Search extends FormAssociated {
}

/**
 * Search configuration options
 * @public
 */
export declare type SearchOptions = FoundationElementDefinition & StartEndOptions;

/**
 * The template for the {@link @microsoft/fast-foundation#(Search:class)} component.
 * @public
 */
export declare const searchTemplate: FoundationElementTemplate<ViewTemplate<Search>, SearchOptions>;

/**
 * A Select Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#select | ARIA select }.
 *
 * @slot start - Content which can be provided before the button content
 * @slot end - Content which can be provided after the button content
 * @slot button-container - The element representing the select button
 * @slot selected-value - The selected value
 * @slot indicator - The visual indicator for the expand/collapse state of the button
 * @slot - The default slot for slotted options
 * @csspart control - The element representing the select invoking element
 * @csspart selected-value - The element wrapping the selected value
 * @csspart indicator - The element wrapping the visual indicator
 * @csspart listbox - The listbox element
 * @fires input - Fires a custom 'input' event when the value updates
 * @fires change - Fires a custom 'change' event when the value updates
 *
 * @public
 */
export declare class Select extends FormAssociatedSelect {
    /**
     * The open attribute.
     *
     * @public
     * @remarks
     * HTML Attribute: open
     */
    open: boolean;
    /* Excluded from this release type: openChanged */
    /* Excluded from this release type: indexWhenOpened */
    /* Excluded from this release type: _value */
    /* Excluded from this release type: collapsible */
    /* Excluded from this release type: control */
    /**
     * The value property.
     *
     * @public
     */
    get value(): string;
    set value(next: string);
    /* Excluded from this release type: updateValue */
    /* Excluded from this release type: selectedIndexChanged */
    /**
     * Reflects the placement for the listbox when the select is open.
     *
     * @public
     */
    positionAttribute?: SelectPosition;
    /* Excluded from this release type: forcedPosition */
    /**
     * Holds the current state for the calculated position of the listbox.
     *
     * @public
     */
    position?: SelectPosition;
    protected positionChanged(prev: SelectPosition | undefined, next: SelectPosition | undefined): void;
    /* Excluded from this release type: listbox */
    /* Excluded from this release type: listboxId */
    /**
     * Calculate and apply listbox positioning based on available viewport space.
     *
     * @public
     */
    setPositioning(): void;
    /* Excluded from this release type: maxHeight */
    /**
     * The value displayed on the button.
     *
     * @public
     */
    get displayValue(): string;
    /* Excluded from this release type: disabledChanged */
    /* Excluded from this release type: formResetCallback */
    /* Excluded from this release type: clickHandler */
    /* Excluded from this release type: focusoutHandler */
    /* Excluded from this release type: handleChange */
    /* Excluded from this release type: slottedOptionsChanged */
    /* Excluded from this release type: mousedownHandler */
    /**
     * Sets the multiple property on the proxy element.
     *
     * @param prev - the previous multiple value
     * @param next - the current multiple value
     */
    multipleChanged(prev: boolean | undefined, next: boolean): void;
    /* Excluded from this release type: selectedOptionsChanged */
    /* Excluded from this release type: setDefaultSelectedOption */
    /* Excluded from this release type: setProxyOptions */
    /* Excluded from this release type: keydownHandler */
    connectedCallback(): void;
    disconnectedCallback(): void;
    /* Excluded from this release type: sizeChanged */
    /* Excluded from this release type: updateDisplayValue */
}

/* Excluded declaration from this release type: Select */

declare class _Select extends ListboxElement {
}

declare interface _Select extends FormAssociated {
}

/**
 * Select configuration options
 * @public
 */
export declare type SelectOptions = FoundationElementDefinition & StartEndOptions & {
    indicator?: string | SyntheticViewTemplate;
};

/**
 * Positioning directions for the listbox when a select is open.
 * @public
 */
export declare const SelectPosition: {
    readonly above: "above";
    readonly below: "below";
};

/**
 * Types for positioning the select element listbox when open
 * @public
 */
export declare type SelectPosition = typeof SelectPosition[keyof typeof SelectPosition];

/**
 * The template for the {@link @microsoft/fast-foundation#(Select:class)} component.
 * @public
 */
export declare const selectTemplate: FoundationElementTemplate<ViewTemplate<Select>, SelectOptions>;

/**
 * Implemented by objects capable of resolving services and other dependencies.
 * @public
 */
export declare interface ServiceLocator {
    /**
     * Determines whether the locator has the ability to provide an implementation
     * for the requested key.
     * @param key - The dependency key to lookup.
     * @param searchAncestors - Indicates whether to search the entire hierarchy of service locators.
     */
    has<K extends Key>(key: K | Key, searchAncestors: boolean): boolean;
    /**
     * Gets a dependency by key.
     * @param key - The key to lookup.
     */
    get<K extends Key>(key: K): Resolved<K>;
    /**
     * Gets a dependency by key.
     * @param key - The key to lookup.
     */
    get<K extends Key>(key: Key): Resolved<K>;
    /**
     * Gets a dependency by key.
     * @param key - The key to lookup.
     */
    get<K extends Key>(key: K | Key): Resolved<K>;
    /**
     * Gets an array of all dependencies by key.
     * @param key - The key to lookup.
     * @param searchAncestors - Indicates whether to search the entire hierarchy of service locators.
     */
    getAll<K extends Key>(key: K, searchAncestors?: boolean): readonly Resolved<K>[];
    /**
     * Gets an array of all dependencies by key.
     * @param key - The key to lookup.
     * @param searchAncestors - Indicates whether to search the entire hierarchy of service locators.
     */
    getAll<K extends Key>(key: Key, searchAncestors?: boolean): readonly Resolved<K>[];
    /**
     * Gets an array of all dependencies by key.
     * @param key - The key to lookup.
     * @param searchAncestors - Indicates whether to search the entire hierarchy of service locators.
     */
    getAll<K extends Key>(key: K | Key, searchAncestors?: boolean): readonly Resolved<K>[];
}

/**
 * The interface key that resolves the service locator itself.
 * @public
 */
export declare const ServiceLocator: InterfaceSymbol<ServiceLocator>;

/**
 * Registers the decorated class as a singleton dependency; the class will only be created once. Each
 * consecutive time the dependency is resolved, the same instance will be returned.
 *
 * @example
 * ```ts
 * @singleton()
 * class Foo { }
 * ```
 *
 * @public
 */
export declare function singleton<T extends Constructable>(): typeof singletonDecorator;

/**
 * @public
 */
export declare function singleton<T extends Constructable>(options?: SingletonOptions): typeof singletonDecorator;

/**
 * Registers the `target` class as a singleton dependency; the class will only be created once. Each
 * consecutive time the dependency is resolved, the same instance will be returned.
 *
 * @param target - The class / constructor function to register as a singleton.
 *
 * @example
 * ```ts
 * @singleton()
 * class Foo { }
 * ```
 *
 * @public
 */
export declare function singleton<T extends Constructable>(target: T & Partial<RegisterSelf<T>>): T & RegisterSelf<T>;

declare function singletonDecorator<T extends Constructable>(target: T & Partial<RegisterSelf<T>>): T & RegisterSelf<T>;

declare type SingletonOptions = {
    scoped: boolean;
};

/**
 * A Skeleton Custom HTML Element.
 *
 * @slot - The default slot
 *
 * @public
 */
export declare class Skeleton extends FoundationElement {
    /**
     * Indicates the Skeleton should have a filled style.
     *
     * @public
     * @remarks
     * HTML Attribute: fill
     */
    fill: string;
    /**
     * Indicates what the shape of the Skeleton should be.
     *
     * @public
     * @remarks
     * HTML Attribute: shape
     */
    shape: SkeletonShape;
    /**
     * Indicates that the component can accept a pattern URL.
     *
     * @public
     * @remarks
     * HTML Attribute: shape
     */
    pattern: string;
    /**
     * Indicates that the component has an activated shimmer effect
     *
     * @public
     * @remarks
     * HTML Attribute: shimmer
     */
    shimmer: boolean;
}

/**
 * A structure representing skeleton shapes
 * @public
 */
export declare type SkeletonShape = "rect" | "circle";

/**
 * The template for the fast-skeleton component
 * @public
 */
export declare const skeletonTemplate: FoundationElementTemplate<ViewTemplate<Skeleton>>;

/**
 * A Slider Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#slider | ARIA slider }.
 *
 * @slot track - The track of the slider
 * @slot track-start - The track-start visual indicator
 * @slot thumb - The slider thumb
 * @slot - The default slot for labels
 * @csspart positioning-region - The region used to position the elements of the slider
 * @csspart track-container - The region containing the track elements
 * @csspart track-start - The element wrapping the track start slot
 * @csspart thumb-container - The thumb container element which is programatically positioned
 * @fires change - Fires a custom 'change' event when the slider value changes
 *
 * @public
 */
export declare class Slider extends FormAssociatedSlider implements SliderConfiguration {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     *
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    protected readOnlyChanged(): void;
    /* Excluded from this release type: track */
    /* Excluded from this release type: thumb */
    /* Excluded from this release type: stepMultiplier */
    /* Excluded from this release type: direction */
    /* Excluded from this release type: isDragging */
    /* Excluded from this release type: position */
    /* Excluded from this release type: trackWidth */
    /* Excluded from this release type: trackMinWidth */
    /* Excluded from this release type: trackHeight */
    /* Excluded from this release type: trackLeft */
    /* Excluded from this release type: trackMinHeight */
    /**
     * The value property, typed as a number.
     *
     * @public
     */
    get valueAsNumber(): number;
    set valueAsNumber(next: number);
    /**
     * Custom function that generates a string for the component's "aria-valuetext" attribute based on the current value.
     *
     * @public
     */
    valueTextFormatter: (value: string) => string | null;
    /* Excluded from this release type: valueChanged */
    /**
     * The minimum allowed value.
     *
     * @defaultValue - 0
     * @public
     * @remarks
     * HTML Attribute: min
     */
    min: number;
    private minChanged;
    /**
     * The maximum allowed value.
     *
     * @defaultValue - 10
     * @public
     * @remarks
     * HTML Attribute: max
     */
    max: number;
    private maxChanged;
    /**
     * Value to increment or decrement via arrow keys, mouse click or drag.
     *
     * @public
     * @remarks
     * HTML Attribute: step
     */
    step: number;
    private stepChanged;
    /**
     * The orientation of the slider.
     *
     * @public
     * @remarks
     * HTML Attribute: orientation
     */
    orientation: Orientation;
    private orientationChanged;
    /**
     * The selection mode.
     *
     * @public
     * @remarks
     * HTML Attribute: mode
     */
    mode: SliderMode;
    /* Excluded from this release type: connectedCallback */
    /* Excluded from this release type: disconnectedCallback */
    /**
     * Increment the value by the step
     *
     * @public
     */
    increment(): void;
    /**
     * Decrement the value by the step
     *
     * @public
     */
    decrement(): void;
    protected keypressHandler: (e: KeyboardEvent) => void;
    /**
     * Places the thumb based on the current value
     *
     * @public
     * @param direction - writing mode
     */
    private setThumbPositionForOrientation;
    /**
     * Update the step multiplier used to ensure rounding errors from steps that
     * are not whole numbers
     */
    private updateStepMultiplier;
    private setupTrackConstraints;
    private setupListeners;
    /* Excluded from this release type: initialValue */
    private get midpoint();
    private setupDefaultValue;
    /**
     *  Handle mouse moves during a thumb drag operation
     *  If the event handler is null it removes the events
     */
    private handleThumbMouseDown;
    /**
     *  Handle mouse moves during a thumb drag operation
     */
    private handleMouseMove;
    private calculateNewValue;
    /**
     * Handle a window mouse up during a drag operation
     */
    private handleWindowMouseUp;
    private stopDragging;
    /**
     *
     * @param e - MouseEvent or null. If there is no event handler it will remove the events
     */
    private handleMouseDown;
    private convertToConstrainedValue;
}

declare class _Slider extends FoundationElement {
}

declare interface _Slider extends FormAssociated {
}

/**
 * The configuration structure of {@link @microsoft/fast-foundation#(Slider:class)}.
 * @public
 */
export declare interface SliderConfiguration {
    max: number;
    min: number;
    orientation?: Orientation;
    direction?: Direction;
    disabled?: boolean;
}

/**
 * A label element intended to be used with the {@link @microsoft/fast-foundation#(Slider:class)} component.
 *
 * @slot - The default slot for the label content
 * @csspart root - The element wrapping the label mark and text
 *
 * @public
 */
export declare class SliderLabel extends FoundationElement {
    /* Excluded from this release type: positionStyle */
    /* Excluded from this release type: root */
    /**
     * The position of the label relative to the min and max value of the parent {@link @microsoft/fast-foundation#(Slider:class)}.
     *
     * @public
     * @remarks
     * HTML Attribute: position
     */
    position: string;
    protected positionChanged(): void;
    /**
     * Hides the tick mark.
     *
     * @public
     * @remarks
     * HTML Attribute: hide-mark
     */
    hideMark: boolean;
    /**
     * The disabled state of the label. This is generally controlled by the parent {@link @microsoft/fast-foundation#(Slider:class)}.
     *
     * @public
     * @remarks
     * HTML Attribute: disabled
     */
    disabled: boolean;
    /* Excluded from this release type: sliderOrientation */
    /* Excluded from this release type: sliderOrientationChanged */
    /* Excluded from this release type: sliderMinPosition */
    /* Excluded from this release type: sliderMaxPosition */
    /* Excluded from this release type: sliderDirection */
    private notifier;
    /* Excluded from this release type: connectedCallback */
    /* Excluded from this release type: disconnectedCallback */
    /* Excluded from this release type: handleChange */
    private isSliderConfig;
    private getSliderConfiguration;
    private positionAsStyle;
}

/**
 * The template for the {@link @microsoft/fast-foundation#(SliderLabel:class)} component.
 * @public
 */
export declare const sliderLabelTemplate: FoundationElementTemplate<ViewTemplate<SliderLabel>>;

/**
 * The selection modes of a {@link @microsoft/fast-foundation#(Slider:class)}.
 * @public
 */
export declare const SliderMode: {
    readonly singleValue: "single-value";
};

/**
 * The types for the selection mode of the slider
 * @public
 */
export declare type SliderMode = typeof SliderMode[keyof typeof SliderMode];

/**
 * Slider configuration options
 * @public
 */
export declare type SliderOptions = FoundationElementDefinition & {
    thumb?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(Slider:class)} component.
 * @public
 */
export declare const sliderTemplate: FoundationElementTemplate<ViewTemplate<Slider>, SliderOptions>;

/**
 * A mixin class implementing start and end slots.
 * These are generally used to decorate text elements with icons or other visual indicators.
 * @public
 */
export declare class StartEnd {
    start: HTMLSlotElement;
    end: HTMLSlotElement;
}

/**
 * Start/End configuration options
 * @public
 */
export declare type StartEndOptions = StartOptions & EndOptions;

/**
 * Start configuration options
 * @public
 */
export declare type StartOptions = {
    start?: string | SyntheticViewTemplate;
};

/**
 * The template for the start slots.
 * For use with {@link StartEnd}
 *
 * @public
 */
export declare const startSlotTemplate: (context: ElementDefinitionContext, definition: StartOptions) => ViewTemplate<StartEnd>;

/**
 * A design token value with no observable dependencies
 * @public
 */
export declare type StaticDesignTokenValue<T> = T extends Function ? never : T;

/* Excluded from this release type: supportsElementInternals */

/**
 * A Switch Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#switch | ARIA switch }.
 *
 * @slot - The deafult slot for the label
 * @slot checked-message - The message when in a checked state
 * @slot unchecked-message - The message when in an unchecked state
 * @csspart label - The label
 * @csspart switch - The element representing the switch, which wraps the indicator
 * @csspart status-message - The wrapper for the status messages
 * @csspart checked-message - The checked message
 * @csspart unchecked-message - The unchecked message
 * @fires change - Emits a custom change event when the checked state changes
 *
 * @public
 */
export declare class Switch extends FormAssociatedSwitch {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    protected readOnlyChanged(): void;
    /* Excluded from this release type: initialValue */
    /* Excluded from this release type: defaultSlottedNodes */
    constructor();
    /* Excluded from this release type: keypressHandler */
    /* Excluded from this release type: clickHandler */
    /* Excluded from this release type: checkedChanged */
}

declare class _Switch extends FoundationElement {
}

declare interface _Switch extends CheckableFormAssociated {
}

/**
 * Switch configuration options
 * @public
 */
export declare type SwitchOptions = FoundationElementDefinition & {
    switch?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(Switch:class)} component.
 * @public
 */
export declare const switchTemplate: FoundationElementTemplate<ViewTemplate<Switch>, SwitchOptions>;

/**
 * A Tab Component to be used with {@link @microsoft/fast-foundation#(Tabs:class)}
 *
 * @slot - The default slot for the tab content
 *
 * @public
 */
export declare class Tab extends FoundationElement {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/disabled | disabled HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: disabled
     */
    disabled: boolean;
}

/**
 * A TabPanel Component to be used with {@link @microsoft/fast-foundation#(Tabs:class)}
 *
 * @slot - The default slot for the tabpanel content
 *
 * @public
 */
export declare class TabPanel extends FoundationElement {
}

/**
 * The template for the {@link @microsoft/fast-foundation#TabPanel} component.
 * @public
 */
export declare const tabPanelTemplate: FoundationElementTemplate<ViewTemplate<TabPanel>>;

/**
 * A Tabs Custom HTML Element.
 * Implements the {@link https://www.w3.org/TR/wai-aria-1.1/#tablist | ARIA tablist }.
 *
 * @slot start - Content which can be provided before the tablist element
 * @slot end - Content which can be provided after the tablist element
 * @slot tab - The slot for tabs
 * @slot tabpanel - The slot for tabpanels
 * @csspart tablist - The element wrapping for the tabs
 * @csspart activeIndicator - The visual indicator
 * @fires change - Fires a custom 'change' event when a tab is clicked or during keyboard navigation
 *
 * @public
 */
export declare class Tabs extends FoundationElement {
    /**
     * The orientation
     * @public
     * @remarks
     * HTML Attribute: orientation
     */
    orientation: TabsOrientation;
    /* Excluded from this release type: orientationChanged */
    /**
     * The id of the active tab
     *
     * @public
     * @remarks
     * HTML Attribute: activeid
     */
    activeid: string;
    /* Excluded from this release type: activeidChanged */
    /* Excluded from this release type: tabs */
    /* Excluded from this release type: tabsChanged */
    /* Excluded from this release type: tabpanels */
    /* Excluded from this release type: tabpanelsChanged */
    /**
     * Whether or not to show the active indicator
     * @public
     * @remarks
     * HTML Attribute: activeindicator
     */
    hideActiveIndicator: boolean;
    /* Excluded from this release type: activeIndicatorRef */
    /* Excluded from this release type: showActiveIndicator */
    /**
     * A reference to the active tab
     * @public
     */
    activetab: HTMLElement;
    private prevActiveTabIndex;
    private activeTabIndex;
    private ticking;
    private tabIds;
    private tabpanelIds;
    private change;
    private isDisabledElement;
    private isFocusableElement;
    private getActiveIndex;
    private setTabs;
    private setTabPanels;
    private getTabIds;
    private getTabPanelIds;
    private setComponent;
    private handleTabClick;
    private isHorizontal;
    private handleTabKeyDown;
    private handleActiveIndicatorPosition;
    private animateActiveIndicator;
    /**
     * The adjust method for FASTTabs
     * @public
     * @remarks
     * This method allows the active index to be adjusted by numerical increments
     */
    adjust(adjustment: number): void;
    private adjustForward;
    private adjustBackward;
    private moveToTabByIndex;
    private focusTab;
    /* Excluded from this release type: connectedCallback */
}

/* Excluded declaration from this release type: Tabs */

/**
 * Tabs option configuration options
 * @public
 */
export declare type TabsOptions = FoundationElementDefinition & StartEndOptions;

/**
 * The orientation of the {@link @microsoft/fast-foundation#(Tabs:class)} component
 * @public
 */
export declare const TabsOrientation: {
    readonly vertical: "vertical";
    readonly horizontal: "horizontal";
};

/**
 * The types for the Tabs component
 * @public
 */
export declare type TabsOrientation = typeof TabsOrientation[keyof typeof TabsOrientation];

/**
 * The template for the {@link @microsoft/fast-foundation#(Tabs:class)} component.
 * @public
 */
export declare const tabsTemplate: FoundationElementTemplate<ViewTemplate<Tabs>, TabsOptions>;

/**
 * The template for the {@link @microsoft/fast-foundation#Tab} component.
 * @public
 */
export declare const tabTemplate: FoundationElementTemplate<ViewTemplate<Tab>>;

/**
 * A Text Area Custom HTML Element.
 * Based largely on the {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/textarea | <textarea> element }.
 *
 * @slot - The default slot for the label
 * @csspart label - The label
 * @csspart root - The element wrapping the control
 * @csspart control - The textarea element
 * @fires change - Emits a custom 'change' event when the textarea emits a change event
 *
 * @public
 */
export declare class TextArea extends FormAssociatedTextArea {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    protected readOnlyChanged(): void;
    /**
     * The resize mode of the element.
     * @public
     * @remarks
     * HTML Attribute: resize
     */
    resize: TextAreaResize;
    /* Excluded from this release type: control */
    /**
     * Indicates that this element should get focus after the page finishes loading.
     * @public
     * @remarks
     * HTML Attribute: autofocus
     */
    autofocus: boolean;
    protected autofocusChanged(): void;
    /**
     * The {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Global_attributes/id | id} of the {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/form | form} the element is associated to
     * @public
     */
    formId: string;
    /**
     * Allows associating a {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/datalist | datalist} to the element by {@link https://developer.mozilla.org/en-US/docs/Web/API/Element/id}.
     * @public
     * @remarks
     * HTML Attribute: list
     */
    list: string;
    protected listChanged(): void;
    /**
     * The maximum number of characters a user can enter.
     * @public
     * @remarks
     * HTMLAttribute: maxlength
     */
    maxlength: number;
    protected maxlengthChanged(): void;
    /**
     * The minimum number of characters a user can enter.
     * @public
     * @remarks
     * HTMLAttribute: minlength
     */
    minlength: number;
    protected minlengthChanged(): void;
    /**
     * The name of the element.
     * @public
     * @remarks
     * HTML Attribute: name
     */
    name: string;
    /**
     * Sets the placeholder value of the element, generally used to provide a hint to the user.
     * @public
     * @remarks
     * HTML Attribute: placeholder
     * Using this attribute does is not a valid substitute for a labeling element.
     */
    placeholder: string;
    /**
     * Sizes the element horizontally by a number of character columns.
     *
     * @public
     * @remarks
     * HTML Attribute: cols
     */
    cols: number;
    /**
     * Sizes the element vertically by a number of character rows.
     *
     * @public
     * @remarks
     * HTML Attribute: rows
     */
    rows: number;
    /**
     * Sets if the element is eligible for spell checking
     * but the UA.
     * @public
     * @remarks
     * HTML Attribute: spellcheck
     */
    spellcheck: boolean;
    protected spellcheckChanged(): void;
    /* Excluded from this release type: defaultSlottedNodes */
    /**
     * Selects all the text in the text area
     *
     * @public
     */
    protected select(): void;
    /* Excluded from this release type: handleTextInput */
    /* Excluded from this release type: handleChange */
}

/* Excluded declaration from this release type: TextArea */

declare class _TextArea extends FoundationElement {
}

declare interface _TextArea extends FormAssociated {
}

/**
 * Resize mode for a TextArea
 * @public
 */
export declare const TextAreaResize: {
    /**
     * No resize.
     */
    readonly none: "none";
    /**
     * Resize vertically and horizontally.
     */
    readonly both: "both";
    /**
     * Resize horizontally.
     */
    readonly horizontal: "horizontal";
    /**
     * Resize vertically.
     */
    readonly vertical: "vertical";
};

/**
 * Types for the Text Area resize mode
 * @public
 */
export declare type TextAreaResize = typeof TextAreaResize[keyof typeof TextAreaResize];

/**
 * The template for the {@link @microsoft/fast-foundation#(TextArea:class)} component.
 * @public
 */
export declare const textAreaTemplate: FoundationElementTemplate<ViewTemplate<TextArea>>;

/**
 * A Text Field Custom HTML Element.
 * Based largely on the {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input/text | <input type="text" /> element }.
 *
 * @slot start - Content which can be provided before the number field input
 * @slot end - Content which can be provided after the number field input
 * @slot - The default slot for the label
 * @csspart label - The label
 * @csspart root - The element wrapping the control, including start and end slots
 * @csspart control - The text field element
 * @fires change - Fires a custom 'change' event when the value has changed
 *
 * @public
 */
export declare class TextField extends FormAssociatedTextField {
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/readonly | readonly HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: readonly
     */
    readOnly: boolean;
    protected readOnlyChanged(): void;
    /**
     * Indicates that this element should get focus after the page finishes loading. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input#htmlattrdefautofocus | autofocus HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: autofocus
     */
    autofocus: boolean;
    protected autofocusChanged(): void;
    /**
     * Sets the placeholder value of the element, generally used to provide a hint to the user.
     * @public
     * @remarks
     * HTML Attribute: placeholder
     * Using this attribute does is not a valid substitute for a labeling element.
     */
    placeholder: string;
    protected placeholderChanged(): void;
    /**
     * Allows setting a type or mode of text.
     * @public
     * @remarks
     * HTML Attribute: type
     */
    type: TextFieldType;
    private typeChanged;
    /**
     * Allows associating a {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Element/datalist | datalist} to the element by {@link https://developer.mozilla.org/en-US/docs/Web/API/Element/id}.
     * @public
     * @remarks
     * HTML Attribute: list
     */
    list: string;
    protected listChanged(): void;
    /**
     * The maximum number of characters a user can enter.
     * @public
     * @remarks
     * HTMLAttribute: maxlength
     */
    maxlength: number;
    protected maxlengthChanged(): void;
    /**
     * The minimum number of characters a user can enter.
     * @public
     * @remarks
     * HTMLAttribute: minlength
     */
    minlength: number;
    protected minlengthChanged(): void;
    /**
     * A regular expression that the value must match to pass validation.
     * @public
     * @remarks
     * HTMLAttribute: pattern
     */
    pattern: string;
    protected patternChanged(): void;
    /**
     * Sets the width of the element to a specified number of characters.
     * @public
     * @remarks
     * HTMLAttribute: size
     */
    size: number;
    protected sizeChanged(): void;
    /**
     * Controls whether or not to enable spell checking for the input field, or if the default spell checking configuration should be used.
     * @public
     * @remarks
     * HTMLAttribute: size
     */
    spellcheck: boolean;
    protected spellcheckChanged(): void;
    /* Excluded from this release type: defaultSlottedNodes */
    /* Excluded from this release type: control */
    /* Excluded from this release type: connectedCallback */
    /**
     * Selects all the text in the text field
     *
     * @public
     */
    protected select(): void;
    /* Excluded from this release type: handleTextInput */
    /* Excluded from this release type: handleChange */
}

/* Excluded declaration from this release type: TextField */

declare class _TextField extends FoundationElement {
}

declare interface _TextField extends FormAssociated {
}

/**
 * Text field configuration options
 * @public
 */
export declare type TextFieldOptions = FoundationElementDefinition & StartEndOptions;

/**
 * The template for the {@link @microsoft/fast-foundation#(TextField:class)} component.
 * @public
 */
export declare const textFieldTemplate: FoundationElementTemplate<ViewTemplate<TextField>, TextFieldOptions>;

/**
 * Text field sub-types
 * @public
 */
export declare const TextFieldType: {
    /**
     * An email TextField
     */
    readonly email: "email";
    /**
     * A password TextField
     */
    readonly password: "password";
    /**
     * A telephone TextField
     */
    readonly tel: "tel";
    /**
     * A text TextField
     */
    readonly text: "text";
    /**
     * A URL TextField
     */
    readonly url: "url";
};

/**
 * Types for the text field sub-types
 * @public
 */
export declare type TextFieldType = typeof TextFieldType[keyof typeof TextFieldType];

/**
 * A Toolbar Custom HTML Element.
 * Implements the {@link https://w3c.github.io/aria-practices/#Toolbar|ARIA Toolbar}.
 *
 * @slot start - Content which can be provided before the slotted items
 * @slot end - Content which can be provided after the slotted items
 * @slot - The default slot for slotted items
 * @slot label - The toolbar label
 * @csspart positioning-region - The element containing the items, start and end slots
 *
 * @public
 */
export declare class Toolbar extends FoundationElement {
    /* Excluded from this release type: _activeIndex */
    /* Excluded from this release type: activeIndex */
    /* Excluded from this release type: activeIndex */
    /* Excluded from this release type: direction */
    /* Excluded from this release type: focusableElements */
    /**
     * The orientation of the toolbar.
     *
     * @public
     * @remarks
     * HTML Attribute: `orientation`
     */
    orientation: Orientation;
    /* Excluded from this release type: slottedItems */
    protected slottedItemsChanged(): void;
    /* Excluded from this release type: slottedLabel */
    /* Excluded from this release type: clickHandler */
    childItems: Element[];
    protected childItemsChanged(prev: undefined | Element[], next: Element[]): void;
    /* Excluded from this release type: connectedCallback */
    /* Excluded from this release type: focusinHandler */
    /* Excluded from this release type: getDirectionalIncrementer */
    /* Excluded from this release type: keydownHandler */
    /* Excluded from this release type: allSlottedItems */
    /* Excluded from this release type: reduceFocusableElements */
    /* Excluded from this release type: setFocusedElement */
    /* Excluded from this release type: reduceFocusableItems */
    /* Excluded from this release type: setFocusableElements */
}

/* Excluded declaration from this release type: Toolbar */

/**
 * Toolbar configuration options
 * @public
 */
export declare type ToolbarOptions = FoundationElementDefinition & StartEndOptions;

/**
 * The template for the {@link @microsoft/fast-foundation#(Toolbar:class)} component.
 *
 * @public
 */
export declare const toolbarTemplate: FoundationElementTemplate<ViewTemplate<Toolbar>, ToolbarOptions>;

/**
 * An Tooltip Custom HTML Element.
 *
 * @slot - The default slot for the tooltip content
 * @csspart tooltip - The tooltip element
 * @fires dismiss - Fires a custom 'dismiss' event when the tooltip is visible and escape key is pressed
 *
 * @public
 */
export declare class Tooltip extends FoundationElement {
    /**
     * Whether the tooltip is visible or not.
     * If undefined tooltip is shown when anchor element is hovered
     *
     * @defaultValue - undefined
     * @public
     * HTML Attribute: visible
     */
    visible: boolean;
    protected visibleChanged(): void;
    /**
     * The id of the element the tooltip is anchored to
     *
     * @defaultValue - undefined
     * @public
     * HTML Attribute: anchor
     */
    anchor: string;
    protected anchorChanged(): void;
    /**
     * The delay in milliseconds before a tooltip is shown after a hover event
     *
     * @defaultValue - 300
     * @public
     * HTML Attribute: delay
     */
    delay: number;
    /**
     * Controls the placement of the tooltip relative to the anchor.
     * When the position is undefined the tooltip is placed above or below the anchor based on available space.
     *
     * @defaultValue - undefined
     * @public
     * HTML Attribute: position
     */
    position: TooltipPosition;
    private positionChanged;
    /**
     * Controls when the tooltip updates its position, default is 'anchor' which only updates when
     * the anchor is resized.  'auto' will update on scroll/resize events.
     * Corresponds to anchored-region auto-update-mode.
     * @public
     * @remarks
     * HTML Attribute: auto-update-mode
     */
    autoUpdateMode: AutoUpdateMode;
    /**
     * Controls if the tooltip will always remain fully in the viewport on the horizontal axis
     * @public
     * @remarks
     * HTML Attribute: horizontal-viewport-lock
     */
    horizontalViewportLock: boolean;
    /**
     * Controls if the tooltip will always remain fully in the viewport on the vertical axis
     * @public
     * @remarks
     * HTML Attribute: vertical-viewport-lock
     */
    verticalViewportLock: boolean;
    /**
     * the html element currently being used as anchor.
     * Setting this directly overrides the anchor attribute.
     *
     * @public
     */
    anchorElement: HTMLElement | null;
    protected anchorElementChanged(oldValue: HTMLElement | null): void;
    /* Excluded from this release type: viewportElement */
    protected viewportElementChanged(): void;
    /* Excluded from this release type: verticalPositioningMode */
    /* Excluded from this release type: horizontalPositioningMode */
    /* Excluded from this release type: horizontalInset */
    /* Excluded from this release type: verticalInset */
    /* Excluded from this release type: horizontalScaling */
    /* Excluded from this release type: verticalScaling */
    /* Excluded from this release type: verticalDefaultPosition */
    /* Excluded from this release type: horizontalDefaultPosition */
    /* Excluded from this release type: tooltipVisible */
    /* Excluded from this release type: currentDirection */
    /* Excluded from this release type: region */
    /**
     * The timer that tracks delay time before the tooltip is shown on hover
     */
    private showDelayTimer;
    /**
     * The timer that tracks delay time before the tooltip is hidden
     */
    private hideDelayTimer;
    /**
     * Indicates whether the anchor is currently being hovered or has focus
     */
    private isAnchorHoveredFocused;
    /**
     * Indicates whether the region is currently being hovered
     */
    private isRegionHovered;
    connectedCallback(): void;
    disconnectedCallback(): void;
    /* Excluded from this release type: handlePositionChange */
    /**
     * mouse enters region
     */
    private handleRegionMouseOver;
    /**
     * mouse leaves region
     */
    private handleRegionMouseOut;
    /**
     * mouse enters anchor
     */
    private handleAnchorMouseOver;
    /**
     * mouse leaves anchor
     */
    private handleAnchorMouseOut;
    /**
     * anchor gets focus
     */
    private handleAnchorFocusIn;
    /**
     * anchor loses focus
     */
    private handleAnchorFocusOut;
    /**
     * starts the hide timer
     */
    private startHideDelayTimer;
    /**
     * clears the hide delay
     */
    private clearHideDelayTimer;
    /**
     * starts the show timer if not currently running
     */
    private startShowDelayTimer;
    /**
     * start hover
     */
    private startHover;
    /**
     * clears the show delay
     */
    private clearShowDelayTimer;
    /**
     * updated the properties being passed to the anchored region
     */
    private updateLayout;
    /**
     *  Gets the anchor element by id
     */
    private getAnchor;
    /**
     * handles key down events to check for dismiss
     */
    private handleDocumentKeydown;
    /**
     * determines whether to show or hide the tooltip based on current state
     */
    private updateTooltipVisibility;
    /**
     * shows the tooltip
     */
    private showTooltip;
    /**
     * hides the tooltip
     */
    private hideTooltip;
    /**
     * updates the tooltip anchored region props after it has been
     * added to the DOM
     */
    private setRegionProps;
}

/**
 * Enumerates possible tooltip positions
 *
 * @public
 */
export declare const TooltipPosition: {
    /**
     * The tooltip is positioned above the element
     */
    readonly top: "top";
    /**
     * The tooltip is positioned to the right of the element
     */
    readonly right: "right";
    /**
     * The tooltip is positioned below the element
     */
    readonly bottom: "bottom";
    /**
     * The tooltip is positioned to the left of the element
     */
    readonly left: "left";
    /**
     * The tooltip is positioned before the element
     */
    readonly start: "start";
    /**
     * The tooltip is positioned after the element
     */
    readonly end: "end";
    /**
     * The tooltip is positioned above the element and to the left
     */
    readonly topLeft: "top-left";
    /**
     * The tooltip is positioned above the element and to the right
     */
    readonly topRight: "top-right";
    /**
     * The tooltip is positioned below the element and to the left
     */
    readonly bottomLeft: "bottom-left";
    /**
     * The tooltip is positioned below the element and to the right
     */
    readonly bottomRight: "bottom-right";
    /**
     * The tooltip is positioned above the element and to the left
     */
    readonly topStart: "top-start";
    /**
     * The tooltip is positioned above the element and to the right
     */
    readonly topEnd: "top-end";
    /**
     * The tooltip is positioned below the element and to the left
     */
    readonly bottomStart: "bottom-start";
    /**
     * The tooltip is positioned below the element and to the right
     */
    readonly bottomEnd: "bottom-end";
};

/**
 * The possible tooltip positions
 *
 * @public
 */
export declare type TooltipPosition = typeof TooltipPosition[keyof typeof TooltipPosition];

/**
 * Creates a template for the {@link @microsoft/fast-foundation#(Tooltip:class)} component using the provided prefix.
 * @public
 */
export declare const tooltipTemplate: FoundationElementTemplate<ViewTemplate<Tooltip>>;

/**
 * Transforms an object after it is created but before it is returned
 * to the requestor.
 * @public
 */
declare type Transformer_2<K> = (instance: Resolved<K>) => Resolved<K>;
export { Transformer_2 as Transformer }

/**
 * Registers the decorated class as a transient dependency; each time the dependency is resolved
 * a new instance will be created.
 *
 * @example
 * ```ts
 * @transient()
 * class Foo { }
 * ```
 *
 * @public
 */
export declare function transient<T extends Constructable>(): typeof transientDecorator;

/**
 * Registers the `target` class as a transient dependency; each time the dependency is resolved
 * a new instance will be created.
 *
 * @param target - The class / constructor function to register as transient.
 *
 * @example
 * ```ts
 * @transient()
 * class Foo { }
 * ```
 *
 * @public
 */
export declare function transient<T extends Constructable>(target: T & Partial<RegisterSelf<T>>): T & RegisterSelf<T>;

declare function transientDecorator<T extends Constructable>(target: T & Partial<RegisterSelf<T>>): T & RegisterSelf<T>;

/**
 * A Tree item Custom HTML Element.
 *
 * @slot start - Content which can be provided before the tree item content
 * @slot end - Content which can be provided after the tree item content
 * @slot - The default slot for tree item text content
 * @slot item - The slot for tree items (fast tree items manage this assignment themselves)
 * @slot expand-collapse-button - The expand/collapse button
 * @csspart positioning-region - The element used to position the tree item content with exception of any child nodes
 * @csspart content-region - The element containing the expand/collapse, start, and end slots
 * @csspart items - The element wrapping any child items
 * @csspart expand-collapse-button - The expand/collapse button
 * @fires expanded-change - Fires a custom 'expanded-change' event when the expanded state changes
 * @fires selected-change - Fires a custom 'selected-change' event when the selected state changes
 *
 * @public
 */
export declare class TreeItem extends FoundationElement {
    /**
     * When true, the control will be appear expanded by user interaction.
     * @public
     * @remarks
     * HTML Attribute: expanded
     */
    expanded: boolean;
    protected expandedChanged(): void;
    /**
     * When true, the control will appear selected by user interaction.
     * @public
     * @remarks
     * HTML Attribute: selected
     */
    selected: boolean;
    protected selectedChanged(): void;
    /**
     * When true, the control will be immutable by user interaction. See {@link https://developer.mozilla.org/en-US/docs/Web/HTML/Attributes/disabled | disabled HTML attribute} for more information.
     * @public
     * @remarks
     * HTML Attribute: disabled
     */
    disabled: boolean;
    /* Excluded from this release type: expandCollapseButton */
    /* Excluded from this release type: focusable */
    /* Excluded from this release type: childItems */
    /* Excluded from this release type: items */
    protected itemsChanged(oldValue: unknown, newValue: HTMLElement[]): void;
    /* Excluded from this release type: nested */
    /* Excluded from this release type: renderCollapsedChildren */
    /**
     * Places document focus on a tree item
     *
     * @public
     * @param el - the element to focus
     */
    static focusItem(el: HTMLElement): void;
    /**
     * Whether the tree is nested
     *
     * @public
     */
    readonly isNestedItem: () => boolean;
    /* Excluded from this release type: handleExpandCollapseButtonClick */
    /* Excluded from this release type: handleFocus */
    /* Excluded from this release type: handleBlur */
    /* Excluded from this release type: childItemLength */
}

/* Excluded declaration from this release type: TreeItem */

/**
 * Tree Item configuration options
 * @public
 */
export declare type TreeItemOptions = FoundationElementDefinition & StartEndOptions & {
    expandCollapseGlyph?: string | SyntheticViewTemplate;
};

/**
 * The template for the {@link @microsoft/fast-foundation#(TreeItem:class)} component.
 * @public
 */
export declare const treeItemTemplate: FoundationElementTemplate<ViewTemplate<TreeItem>, TreeItemOptions>;

/**
 * A Tree view Custom HTML Element.
 * Implements the {@link https://w3c.github.io/aria-practices/#TreeView | ARIA TreeView }.
 *
 * @slot - The default slot for tree items
 *
 * @public
 */
export declare class TreeView extends FoundationElement {
    /**
     /**
     * When true, the control will be appear expanded by user interaction.
     * @public
     * @remarks
     * HTML Attribute: render-collapsed-nodes
     */
    renderCollapsedNodes: boolean;
    /**
     * The currently selected tree item
     * @public
     */
    currentSelected: HTMLElement | TreeItem | null;
    /* Excluded from this release type: slottedTreeItems */
    protected slottedTreeItemsChanged(): void;
    /* Excluded from this release type: currentFocused */
    /* Excluded from this release type: handleFocus */
    /* Excluded from this release type: handleBlur */
    /* Excluded from this release type: treeView */
    private nested;
    connectedCallback(): void;
    /* Excluded from this release type: handleKeyDown */
    /* Excluded from this release type: handleClick */
    /* Excluded from this release type: handleSelectedChange */
    /**
     * Move focus to a tree item based on its offset from the provided item
     */
    private focusNextNode;
    /**
     * Updates the tree view when slottedTreeItems changes
     */
    private setItems;
    /**
     * checks if there are any nested tree items
     */
    private getValidFocusableItem;
    /**
     * checks if there are any nested tree items
     */
    private checkForNestedItems;
    /**
     * check if the item is focusable
     */
    private isFocusableElement;
    private isSelectedElement;
    private getVisibleNodes;
}

/**
 * The template for the {@link @microsoft/fast-foundation#TreeView} component.
 * @public
 */
export declare const treeViewTemplate: FoundationElementTemplate<ViewTemplate<TreeView>>;

/* Excluded from this release type: validateKey */

/**
 * This file enables typing support for ElementInternals APIs.
 * It is largely taken from https://github.com/microsoft/TSJS-lib-generator/pull/818/files.
 *
 * When TypeScript adds support for these APIs we can delete this file.
 */
declare interface ValidityStateFlags {
    badInput?: boolean;
    customError?: boolean;
    patternMismatch?: boolean;
    rangeOverflow?: boolean;
    rangeUnderflow?: boolean;
    stepMismatch?: boolean;
    tooLong?: boolean;
    tooShort?: boolean;
    typeMismatch?: boolean;
    valueMissing?: boolean;
}

/**
 * Defines the vertical positioning options for an anchored region
 *
 * @public
 */
export declare type VerticalPosition = "top" | "bottom" | "center" | "unset";

/**
 * A type representing the different weekday formats
 * @public
 */
export declare type WeekdayFormat = "long" | "narrow" | "short";

/**
 * Calendar weekday text.
 * @public
 */
export declare type WeekdayText = {
    text: string;
    abbr?: string;
};

/**
 * a method to filter out any whitespace _only_ nodes, to be used inside a template
 * @param value - The Node that is being inspected
 * @param index - The index of the node within the array
 * @param array - The Node array that is being filtered
 *
 * @public
 */
export declare function whitespaceFilter(value: Node, index: number, array: Node[]): boolean;

/**
 * A type representing the different year formats
 * @public
 */
export declare type YearFormat = "2-digit" | "numeric";

export { }
